@@ -410,11 +410,19 @@ if [ "$BUILD_TYPE" = "LINT" ]; then
   LINT_RESULT=0
 
   if [ "$BUILD_JAVA" == "1" ]; then
+    # Run Java static code analysis via Gradle.
     pushd $SOURCE_ROOT/java
     if ! ./gradlew $EXTRA_GRADLE_FLAGS clean check -x test $EXTRA_GRADLE_TEST_FLAGS; then
       LINT_RESULT=1
       LINT_FAILURES="$LINT_FAILURES"$'Java Gradle check failed\n'
     fi
+
+    # Verify the contents of the JARs to ensure the shading and
+    # packaging is correct.
+    if ! $SOURCE_ROOT/build-support/verify_jars.pl .; then
+      LINT_RESULT=1
+      LINT_FAILURES="$LINT_FAILURES"$'Java verify Jars check failed\n'
+    fi
     popd
   fi
 
@@ -535,17 +543,11 @@ if [ "$BUILD_JAVA" == "1" ]; then
       FAILURES="$FAILURES"$'Could not submit Java distributed test job\n'
     fi
   else
-    # TODO: Run `gradle check` in BUILD_TYPE DEBUG when static code analysis is fixed
     if ! ./gradlew $EXTRA_GRADLE_FLAGS clean test $EXTRA_GRADLE_TEST_FLAGS; then
       TESTS_FAILED=1
       FAILURES="$FAILURES"$'Java Gradle build/test failed\n'
     fi
   fi
-
-  # Run a script to verify the contents of the JARs to ensure the shading and
-  # packaging is correct.
-  $SOURCE_ROOT/build-support/verify_jars.pl .
-
   set +x
   popd
 fi
