@@ -82,6 +82,29 @@ elif [[ -f "/usr/bin/apt-get" ]]; then
   rm -rf /var/lib/apt/lists/* /tmp/* /var/tmp/*
 
   unset DEBIAN_FRONTEND
+# OpenSUSE/SLES
+elif [[ -f "/usr/bin/zypper" ]]; then
+  # Update the repo.
+  zypper update -y
+
+  zypper install -y \
+    cyrus-sasl-gssapi \
+    cyrus-sasl-plain \
+    krb5-devel \
+    krb5-server \
+    openssl
+
+  # Install extra impala runtime packages used by the impala-runtime image. They are nominal in size.
+  # TODO(ghenke): tzdata equivalent package. This is not an issue given we currently
+  # only build the Impala images with in CentOS 7.
+  zypper install -y \
+    cyrus-sasl-devel \
+    liblzo2-2 \
+    which
+
+  # Reduce the image size by cleaning up after the install.
+  zypper clean --all
+  rm -rf /var/lib/zypp/* /tmp/* /var/tmp/*
 else
   echo "Unsupported OS"
   exit 1
