@@ -46,6 +46,7 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.kudu.client.AsyncKuduClient.LookupType;
 import org.apache.kudu.util.AsyncUtil;
+import org.apache.kudu.util.LogThrottler;
 import org.apache.kudu.util.Slice;
 
 /**
@@ -114,6 +115,10 @@ import org.apache.kudu.util.Slice;
 public class AsyncKuduSession implements SessionConfiguration {
 
   public static final Logger LOG = LoggerFactory.getLogger(AsyncKuduSession.class);
+  /**
+   * Instance of LogThrottler isn't static so we can throttle messages per session
+   */
+  private final LogThrottler throttleClosedLog = new LogThrottler(LOG);
 
   private final AsyncKuduClient client;
   private final Random randomizer = new Random();
@@ -123,6 +128,7 @@ public class AsyncKuduSession implements SessionConfiguration {
   private FlushMode flushMode;
   private ExternalConsistencyMode consistencyMode;
   private long timeoutMillis;
+  private final long txnId;
 
   /**
    * Protects internal state from concurrent access. {@code AsyncKuduSession} is not threadsafe
@@ -183,6 +189,24 @@ public class AsyncKuduSession implements SessionConfiguration {
    */
   AsyncKuduSession(AsyncKuduClient client) {
     this.client = client;
+    this.txnId = AsyncKuduClient.INVALID_TXN_ID;
+    flushMode = FlushMode.AUTO_FLUSH_SYNC;
+    consistencyMode = CLIENT_PROPAGATED;
+    timeoutMillis = client.getDefaultOperationTimeoutMs();
+    inactiveBuffers.add(bufferA);
+    inactiveBuffers.add(bufferB);
+    errorCollector = new ErrorCollector(mutationBufferMaxOps);
+  }
+
+  /**
+   * Constructor for a transactional session.
+   * @param client client that creates this session
+   * @param txnId transaction identifier for all operations within the session
+   */
+  AsyncKuduSession(AsyncKuduClient client, long txnId) {
+    assert txnId > AsyncKuduClient.INVALID_TXN_ID;
+    this.client = client;
+    this.txnId = txnId;
     flushMode = FlushMode.AUTO_FLUSH_SYNC;
     consistencyMode = CLIENT_PROPAGATED;
     timeoutMillis = client.getDefaultOperationTimeoutMs();
@@ -381,7 +405,7 @@ public class AsyncKuduSession implements SessionConfiguration {
         Batch batch = batches.get(tabletId);
         if (batch == null) {
           batch = new Batch(operation.getTable(), tablet, ignoreAllDuplicateRows,
-              ignoreAllNotFoundRows);
+              ignoreAllNotFoundRows, txnId);
           batches.put(tabletId, batch);
         }
         batch.add(operation, currentIndex++);
@@ -520,6 +544,7 @@ public class AsyncKuduSession implements SessionConfiguration {
     operation.setExternalConsistencyMode(consistencyMode);
     operation.setIgnoreAllDuplicateRows(ignoreAllDuplicateRows);
     operation.setIgnoreAllNotFoundRows(ignoreAllNotFoundRows);
+    operation.setTxnId(txnId);
 
     return client.sendRpcToTablet(operation)
         .addCallbackDeferring(resp -> {
@@ -549,7 +574,7 @@ public class AsyncKuduSession implements SessionConfiguration {
     if (closed) {
       // Ideally this would be a precondition, but that may break existing
       // clients who have grown to rely on this unsafe behavior.
-      LOG.warn("Applying an operation in a closed session; this is unsafe");
+      throttleClosedLog.warn(60L, "Applying an operation in a closed session; this is unsafe");
     }
 
     // Freeze the row so that the client cannot concurrently modify it while it is in flight.
