@@ -92,6 +92,18 @@ public class KuduClient implements AutoCloseable {
     return asyncClient.getLocationString();
   }
 
+  /**
+   * Returns the ID of the cluster that this client is connected to.
+   * It will be an empty string if the client is not connected or
+   * the client is connected to a cluster that doesn't support
+   * cluster IDs.
+   *
+   * @return the ID of the cluster that this client is connected to
+   */
+  public String getClusterId() {
+    return asyncClient.getClusterId();
+  }
+
   /**
    * Returns the unique client id assigned to this client.
    * @return the unique client id assigned to this client.
@@ -287,6 +299,22 @@ public class KuduClient implements AutoCloseable {
     return new KuduSession(session);
   }
 
+  /**
+   * Start a new multi-row distributed transaction.
+   * <p>
+   * Start a new multi-row transaction and return a handle for the transactional
+   * object to manage the newly started transaction. Under the hood, this makes
+   * an RPC call to the Kudu cluster and registers a newly created transaction
+   * in the system. This call is blocking.
+   *
+   * @return a handle to the newly started transaction in case of success
+   */
+  public KuduTransaction newTransaction() throws KuduException {
+    KuduTransaction txn = new KuduTransaction(asyncClient);
+    txn.begin();
+    return txn;
+  }
+
   /**
    * Check if statistics collection is enabled for this client.
    * @return true if it is enabled, else false
@@ -398,6 +426,15 @@ public class KuduClient implements AutoCloseable {
     return asyncClient.getMasterAddressesAsString();
   }
 
+  /**
+   * Sends a request to the master to check if the cluster supports ignore operations.
+   * @return true if the cluster supports ignore operations
+   */
+  @InterfaceAudience.Private
+  public boolean supportsIgnoreOperations() throws KuduException {
+    return joinAndHandleException(asyncClient.supportsIgnoreOperations());
+  }
+
   /**
    * @return a HostAndPort describing the current leader master
    * @throws KuduException if a leader master could not be found in time
