@@ -22,7 +22,9 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.sql.Date;
 import java.sql.Timestamp;
+import java.time.LocalDate;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -44,6 +46,7 @@ import org.apache.kudu.ColumnTypeAttributes;
 import org.apache.kudu.Common;
 import org.apache.kudu.Schema;
 import org.apache.kudu.Type;
+import org.apache.kudu.util.DateUtil;
 import org.apache.kudu.util.DecimalUtil;
 import org.apache.kudu.util.TimestampUtil;
 
@@ -166,7 +169,8 @@ public class KuduPredicate {
   public static KuduPredicate newComparisonPredicate(ColumnSchema column,
                                                      ComparisonOp op,
                                                      long value) {
-    checkColumn(column, Type.INT8, Type.INT16, Type.INT32, Type.INT64, Type.UNIXTIME_MICROS);
+    checkColumn(column, Type.INT8, Type.INT16, Type.INT32, Type.INT64, Type.UNIXTIME_MICROS,
+        Type.DATE);
     long minValue = minIntValue(column.getType());
     long maxValue = maxIntValue(column.getType());
     Preconditions.checkArgument(value <= maxValue && value >= minValue,
@@ -202,6 +206,7 @@ public class KuduPredicate {
         bytes = Bytes.fromShort((short) value);
         break;
       }
+      case DATE:
       case INT32: {
         bytes = Bytes.fromInt((int) value);
         break;
@@ -314,6 +319,20 @@ public class KuduPredicate {
     return newComparisonPredicate(column, op, micros);
   }
 
+  /**
+   * Creates a new comparison predicate on a date column.
+   * @param column the column schema
+   * @param op the comparison operation
+   * @param value the value to compare against
+   */
+  public static KuduPredicate newComparisonPredicate(ColumnSchema column,
+                                                     ComparisonOp op,
+                                                     Date value) {
+    checkColumn(column, Type.DATE);
+    int days = DateUtil.sqlDateToEpochDays(value);
+    return newComparisonPredicate(column, op, days);
+  }
+
   /**
    * Creates a new comparison predicate on a float column.
    * @param column the column schema
@@ -413,7 +432,7 @@ public class KuduPredicate {
   public static KuduPredicate newComparisonPredicate(ColumnSchema column,
                                                      ComparisonOp op,
                                                      String value) {
-    checkColumn(column, Type.STRING);
+    checkColumn(column, Type.STRING, Type.VARCHAR);
 
     byte[] bytes = Bytes.fromString(value);
     if (op == ComparisonOp.LESS_EQUAL) {
@@ -495,8 +514,10 @@ public class KuduPredicate {
    *  Type.FLOAT -> java.lang.Float
    *  Type.DOUBLE -> java.lang.Double
    *  Type.STRING -> java.lang.String
+   *  Type.VARCHAR -> java.lang.String
    *  Type.BINARY -> byte[]
    *  Type.DECIMAL -> java.math.BigDecimal
+   *  Type.DATE -> java.sql.Date
    *
    * @param column column the column schema
    * @param op the comparison operation
@@ -527,6 +548,8 @@ public class KuduPredicate {
       return newComparisonPredicate(column, op, (String) value);
     } else if (value instanceof byte[]) {
       return newComparisonPredicate(column, op, (byte[]) value);
+    } else if (value instanceof Date) {
+      return newComparisonPredicate(column, op, (Date) value);
     } else {
       throw new IllegalArgumentException(String.format("illegal type for %s predicate: %s",
               op, value.getClass().getName()));
@@ -572,7 +595,7 @@ public class KuduPredicate {
         vals.add(Bytes.fromShort((Short) value));
       }
     } else if (t instanceof Integer) {
-      checkColumn(column, Type.INT32);
+      checkColumn(column, Type.INT32, Type.DATE);
       for (T value : values) {
         vals.add(Bytes.fromInt((Integer) value));
       }
@@ -592,13 +615,13 @@ public class KuduPredicate {
         vals.add(Bytes.fromDouble((Double) value));
       }
     } else if (t instanceof BigDecimal) {
-        checkColumn(column, Type.DECIMAL);
-        for (T value : values) {
-          vals.add(Bytes.fromBigDecimal((BigDecimal) value,
-              column.getTypeAttributes().getPrecision()));
-        }
+      checkColumn(column, Type.DECIMAL);
+      for (T value : values) {
+        vals.add(Bytes.fromBigDecimal((BigDecimal) value,
+            column.getTypeAttributes().getPrecision()));
+      }
     } else if (t instanceof String) {
-      checkColumn(column, Type.STRING);
+      checkColumn(column, Type.STRING, Type.VARCHAR);
       for (T value : values) {
         vals.add(Bytes.fromString((String) value));
       }
@@ -607,6 +630,11 @@ public class KuduPredicate {
       for (T value : values) {
         vals.add((byte[]) value);
       }
+    } else if (t instanceof Date) {
+      checkColumn(column, Type.DATE);
+      for (T value : values) {
+        vals.add(Bytes.fromInt(DateUtil.sqlDateToEpochDays((Date) value)));
+      }
     } else {
       throw new IllegalArgumentException(String.format("illegal type for IN list values: %s",
                                                        t.getClass().getName()));
@@ -973,6 +1001,7 @@ public class KuduPredicate {
       case INT16:
         return Short.compare(Bytes.getShort(a), Bytes.getShort(b));
       case INT32:
+      case DATE:
       case DECIMAL32:
         return Integer.compare(Bytes.getInt(a), Bytes.getInt(b));
       case INT64:
@@ -984,6 +1013,7 @@ public class KuduPredicate {
       case DOUBLE:
         return Double.compare(Bytes.getDouble(a), Bytes.getDouble(b));
       case STRING:
+      case VARCHAR:
       case BINARY:
         return UnsignedBytes.lexicographicalComparator().compare(a, b);
       case DECIMAL128:
@@ -1013,7 +1043,8 @@ public class KuduPredicate {
         return m < n && m + 1 == n;
       }
       case INT32:
-      case DECIMAL32:{
+      case DATE:
+      case DECIMAL32: {
         int m = Bytes.getInt(a);
         int n = Bytes.getInt(b);
         return m < n && m + 1 == n;
@@ -1036,6 +1067,7 @@ public class KuduPredicate {
         return m < n && Math.nextAfter(m, Double.POSITIVE_INFINITY) == n;
       }
       case STRING:
+      case VARCHAR:
       case BINARY: {
         if (a.length + 1 != b.length || b[a.length] != 0) {
           return false;
@@ -1095,6 +1127,8 @@ public class KuduPredicate {
       case UNIXTIME_MICROS:
       case INT64:
         return Long.MAX_VALUE;
+      case DATE:
+        return DateUtil.MAX_DATE_VALUE;
       default:
         throw new IllegalArgumentException("type must be an integer type");
     }
@@ -1117,6 +1151,8 @@ public class KuduPredicate {
       case UNIXTIME_MICROS:
       case INT64:
         return Long.MIN_VALUE;
+      case DATE:
+        return DateUtil.MIN_DATE_VALUE;
       default:
         throw new IllegalArgumentException("type must be an integer type");
     }
@@ -1150,9 +1186,11 @@ public class KuduPredicate {
       case INT16: return Short.toString(Bytes.getShort(value));
       case INT32: return Integer.toString(Bytes.getInt(value));
       case INT64: return Long.toString(Bytes.getLong(value));
+      case DATE: return DateUtil.epochDaysToDateString(Bytes.getInt(value));
       case UNIXTIME_MICROS: return TimestampUtil.timestampToString(Bytes.getLong(value));
       case FLOAT: return Float.toString(Bytes.getFloat(value));
       case DOUBLE: return Double.toString(Bytes.getDouble(value));
+      case VARCHAR:
       case STRING: {
         String v = Bytes.getString(value);
         StringBuilder sb = new StringBuilder(2 + v.length());
@@ -1165,9 +1203,9 @@ public class KuduPredicate {
       case DECIMAL32:
       case DECIMAL64:
       case DECIMAL128:
-       ColumnTypeAttributes typeAttributes = column.getTypeAttributes();
-       return Bytes.getDecimal(value, typeAttributes.getPrecision(),
-           typeAttributes.getScale()).toString();
+        ColumnTypeAttributes typeAttributes = column.getTypeAttributes();
+        return Bytes.getDecimal(value, typeAttributes.getPrecision(),
+            typeAttributes.getScale()).toString();
       default:
         throw new IllegalStateException(String.format("unknown column type %s", column.getType()));
     }
