@@ -37,7 +37,9 @@ import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
 
+import org.apache.kudu.ColumnSchema;
 import org.apache.kudu.Schema;
+import org.apache.kudu.Type;
 import org.apache.kudu.test.ClientTestUtil;
 import org.apache.kudu.test.KuduTestHarness;
 
@@ -191,7 +193,63 @@ public class TestKuduSession {
     session.flush();
 
     for (PartialRow row : rows) {
-      Delete del = table.newDelete();
+      Operation del;
+      if (row.getInt(0) % 2 == 0) {
+        del = table.newDelete();
+      } else {
+        del = table.newDeleteIgnore();
+      }
+      del.setRow(row);
+      session.apply(del);
+    }
+    session.flush();
+
+    assertEquals(0, session.countPendingErrors());
+    assertEquals(0, countRowsInScan(client.newScannerBuilder(table).build()));
+  }
+
+  /** Regression test for KUDU-3198. Delete with full row from a 64-column table. */
+  @Test(timeout = 100000)
+  public void testDeleteWithFullRowFrom64ColumnTable() throws Exception {
+    ArrayList<ColumnSchema> columns = new ArrayList<>(64);
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.INT32).key(true).build());
+    for (int i = 1; i < 64; i++) {
+      columns.add(new ColumnSchema.ColumnSchemaBuilder("column_" + i, Type.STRING)
+          .nullable(true)
+          .build());
+    }
+    Schema schema = new Schema(columns);
+
+    KuduTable table = client.createTable(tableName, schema, getBasicCreateTableOptions());
+
+    KuduSession session = client.newSession();
+    session.setFlushMode(SessionConfiguration.FlushMode.MANUAL_FLUSH);
+
+    // Insert 25 rows and then delete them.
+    List<PartialRow> rows = new ArrayList<>();
+    for (int i = 0; i < 25; i++) {
+      Insert insert = table.newInsert();
+      PartialRow row = insert.getRow();
+      row.addInt(0, 1);
+      for (int j = 1; j < 64; j++) {
+        if (j % 2 == 0) {
+          row.setNull(j);
+        } else {
+          row.addString(j, "val_" + j);
+        }
+      }
+      rows.add(row);
+      session.apply(insert);
+    }
+    session.flush();
+
+    for (PartialRow row : rows) {
+      Operation del;
+      if (row.getInt(0) % 2 == 0) {
+        del = table.newDelete();
+      } else {
+        del = table.newDeleteIgnore();
+      }
       del.setRow(row);
       session.apply(del);
     }
@@ -374,6 +432,44 @@ public class TestKuduSession {
 
   }
 
+  @Test(timeout = 10000)
+  public void testUpdateIgnore() throws Exception {
+    KuduTable table = client.createTable(tableName, basicSchema, getBasicCreateTableOptions());
+    KuduSession session = client.newSession();
+
+    // Test update ignore does not return a row error.
+    assertFalse(session.apply(createUpdateIgnore(table, 1, 1, false)).hasRowError());
+    assertEquals(0, scanTableToStrings(table).size());
+
+    assertFalse(session.apply(createInsert(table, 1)).hasRowError());
+    assertEquals(1, scanTableToStrings(table).size());
+
+    // Test update ignore implements normal update.
+    assertFalse(session.apply(createUpdateIgnore(table, 1, 2, false)).hasRowError());
+    List<String> rowStrings = scanTableToStrings(table);
+    assertEquals(1, rowStrings.size());
+    assertEquals(
+        "INT32 key=1, INT32 column1_i=2, INT32 column2_i=3, " +
+            "STRING column3_s=a string, BOOL column4_b=true",
+        rowStrings.get(0));
+  }
+
+  @Test(timeout = 10000)
+  public void testDeleteIgnore() throws Exception {
+    KuduTable table = client.createTable(tableName, basicSchema, getBasicCreateTableOptions());
+    KuduSession session = client.newSession();
+
+    // Test delete ignore does not return a row error.
+    assertFalse(session.apply(createDeleteIgnore(table, 1)).hasRowError());
+
+    assertFalse(session.apply(createInsert(table, 1)).hasRowError());
+    assertEquals(1, scanTableToStrings(table).size());
+
+    // Test delete ignore implements normal delete.
+    assertFalse(session.apply(createDeleteIgnore(table, 1)).hasRowError());
+    assertEquals(0, scanTableToStrings(table).size());
+  }
+
   @Test(timeout = 10000)
   public void testInsertManualFlushNonCoveredRange() throws Exception {
     CreateTableOptions createOptions = getBasicTableOptionsWithNonCoveredRange();
@@ -503,7 +599,18 @@ public class TestKuduSession {
 
   private Upsert createUpsert(KuduTable table, int key, int secondVal, boolean hasNull) {
     Upsert upsert = table.newUpsert();
-    PartialRow row = upsert.getRow();
+    populateUpdateRow(upsert.getRow(), key, secondVal, hasNull);
+    return upsert;
+  }
+
+  private UpdateIgnore createUpdateIgnore(KuduTable table, int key, int secondVal,
+                                          boolean hasNull) {
+    UpdateIgnore updateIgnore = table.newUpdateIgnore();
+    populateUpdateRow(updateIgnore.getRow(), key, secondVal, hasNull);
+    return updateIgnore;
+  }
+
+  private void populateUpdateRow(PartialRow row, int key, int secondVal, boolean hasNull) {
     row.addInt(0, key);
     row.addInt(1, secondVal);
     row.addInt(2, 3);
@@ -513,7 +620,6 @@ public class TestKuduSession {
       row.addString(3, "a string");
     }
     row.addBoolean(4, true);
-    return upsert;
   }
 
   private Delete createDelete(KuduTable table, int key) {
@@ -523,6 +629,13 @@ public class TestKuduSession {
     return delete;
   }
 
+  private DeleteIgnore createDeleteIgnore(KuduTable table, int key) {
+    DeleteIgnore deleteIgnore = table.newDeleteIgnore();
+    PartialRow row = deleteIgnore.getRow();
+    row.addInt(0, key);
+    return deleteIgnore;
+  }
+
   protected InsertIgnore createInsertIgnore(KuduTable table, int key) {
     InsertIgnore insertIgnore = table.newInsertIgnore();
     PartialRow row = insertIgnore.getRow();
