@@ -35,9 +35,12 @@ import static org.junit.Assert.fail;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 
 import com.google.common.collect.ImmutableList;
+import com.google.common.collect.Iterables;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
@@ -662,4 +665,72 @@ public class TestScanToken {
     assertTrue(tokenWithTableMetadataBytes.length > tokenWithTabletMetadataBytes.length);
     assertTrue(tokenWithTabletMetadataBytes.length > tokenBytes.length);
   }
+
+  @Test
+  public void testScanTokensWithExtraPredicate() throws IOException {
+    final int NUM_ROWS_DESIRED = 100;
+    final int PREDICATE_INDEX = 0;
+    final int PREDICATE_VAL = 1;
+    KuduTable table = createDefaultTable(client, testTableName);
+    loadDefaultTable(client, testTableName, NUM_ROWS_DESIRED);
+    KuduScanToken.KuduScanTokenBuilder builder =
+        new KuduScanToken.KuduScanTokenBuilder(asyncClient, table);
+    List<KuduScanToken> tokens = builder.build();
+    ColumnSchema cs = table.getSchema().getColumnByIndex(PREDICATE_INDEX);
+    KuduPredicate predicate = KuduPredicate.newComparisonPredicate(
+        cs, KuduPredicate.ComparisonOp.EQUAL, PREDICATE_VAL);
+    Set<Integer> resultKeys = new HashSet<>();
+    for (KuduScanToken token : tokens) {
+      byte[] serialized = token.serialize();
+      KuduScanner.KuduScannerBuilder scannerBuilder = KuduScanToken.deserializeIntoScannerBuilder(
+          serialized, client);
+      scannerBuilder.addPredicate(predicate);
+      KuduScanner scanner = scannerBuilder.build();
+      for (RowResult rowResult : scanner) {
+        resultKeys.add(rowResult.getInt(PREDICATE_INDEX));
+      }
+    }
+    assertEquals(1, resultKeys.size());
+    assertEquals(PREDICATE_VAL, Iterables.getOnlyElement(resultKeys).intValue());
+  }
+
+  /**
+   * Regression test for KUDU-3205.
+   */
+  @Test
+  public void testBuildTokensWithDownTabletServer() throws Exception {
+    Schema schema = getBasicSchema();
+    CreateTableOptions createOptions = new CreateTableOptions();
+    createOptions.setRangePartitionColumns(ImmutableList.of());
+    createOptions.setNumReplicas(3);
+    KuduTable table = client.createTable(testTableName, schema, createOptions);
+
+    // Insert a row.
+    KuduSession session = client.newSession();
+    Insert insert = createBasicSchemaInsert(table, 1);
+    session.apply(insert);
+    session.close();
+
+    // Remove a tablet server from the remote tablet by calling `removeTabletClient`.
+    // This is done in normal applications via AsyncKuduClient.invalidateTabletCache
+    // when a tablet not found error is handled.
+    TableLocationsCache.Entry entry =
+        asyncClient.getTableLocationEntry(table.getTableId(), insert.partitionKey());
+    RemoteTablet remoteTablet = entry.getTablet();
+    List<ServerInfo> tabletServers = remoteTablet.getTabletServersCopy();
+    remoteTablet.removeTabletClient(tabletServers.get(0).getUuid());
+
+    // Ensure we can build and use the token without an error.
+    KuduScanToken.KuduScanTokenBuilder tokenBuilder = client.newScanTokenBuilder(table);
+    tokenBuilder.includeTableMetadata(true);
+    tokenBuilder.includeTabletMetadata(true);
+    List<KuduScanToken> tokens = tokenBuilder.build();
+    assertEquals(1, tokens.size());
+
+    // Use a new client to simulate hydrating in a new process.
+    KuduClient newClient =
+        new KuduClient.KuduClientBuilder(harness.getMasterAddressesAsString()).build();
+    KuduScanner scanner = tokens.get(0).intoScanner(newClient);
+    assertEquals(1, countRowsInScan(scanner));
+  }
 }
