@@ -53,6 +53,7 @@
 #include "kudu/master/master.h"
 #include "kudu/master/master.pb.h"
 #include "kudu/master/master.proxy.h"
+#include "kudu/master/txn_manager.proxy.h"
 #include "kudu/rpc/connection.h"
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/request_tracker.h"
@@ -73,7 +74,34 @@ DECLARE_int32(dns_resolver_max_threads_num);
 DECLARE_uint32(dns_resolver_cache_capacity_mb);
 DECLARE_uint32(dns_resolver_cache_ttl_sec);
 
+
 using boost::container::small_vector;
+using kudu::client::internal::AsyncLeaderMasterRpc;
+using kudu::client::internal::ConnectToClusterRpc;
+using kudu::client::internal::RemoteTablet;
+using kudu::client::internal::RemoteTabletServer;
+using kudu::master::AlterTableRequestPB;
+using kudu::master::AlterTableResponsePB;
+using kudu::master::ConnectToMasterResponsePB;
+using kudu::master::CreateTableRequestPB;
+using kudu::master::CreateTableResponsePB;
+using kudu::master::DeleteTableRequestPB;
+using kudu::master::DeleteTableResponsePB;
+using kudu::master::GetTableSchemaRequestPB;
+using kudu::master::GetTableSchemaResponsePB;
+using kudu::master::IsAlterTableDoneRequestPB;
+using kudu::master::IsAlterTableDoneResponsePB;
+using kudu::master::IsCreateTableDoneRequestPB;
+using kudu::master::IsCreateTableDoneResponsePB;
+using kudu::master::ListTabletServersRequestPB;
+using kudu::master::ListTabletServersResponsePB;
+using kudu::master::MasterFeatures;
+using kudu::master::MasterServiceProxy;
+using kudu::master::TableIdentifierPB;
+using kudu::rpc::BackoffType;
+using kudu::rpc::CredentialsPolicy;
+using kudu::security::SignedTokenPB;
+using kudu::transactions::TxnManagerServiceProxy;
 using std::map;
 using std::pair;
 using std::set;
@@ -81,6 +109,7 @@ using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::vector;
+using strings::Substitute;
 
 namespace kudu {
 
@@ -88,36 +117,8 @@ namespace security {
 class SignedTokenPB;
 } // namespace security
 
-using master::AlterTableRequestPB;
-using master::AlterTableResponsePB;
-using master::ConnectToMasterResponsePB;
-using master::CreateTableRequestPB;
-using master::CreateTableResponsePB;
-using master::DeleteTableRequestPB;
-using master::DeleteTableResponsePB;
-using master::GetTableSchemaRequestPB;
-using master::GetTableSchemaResponsePB;
-using master::IsAlterTableDoneRequestPB;
-using master::IsAlterTableDoneResponsePB;
-using master::IsCreateTableDoneRequestPB;
-using master::IsCreateTableDoneResponsePB;
-using master::ListTabletServersResponsePB;
-using master::ListTabletServersRequestPB;
-using master::MasterFeatures;
-using master::MasterServiceProxy;
-using master::TableIdentifierPB;
-using rpc::BackoffType;
-using rpc::CredentialsPolicy;
-using security::SignedTokenPB;
-using strings::Substitute;
-
 namespace client {
 
-using internal::AsyncLeaderMasterRpc;
-using internal::ConnectToClusterRpc;
-using internal::RemoteTablet;
-using internal::RemoteTabletServer;
-
 Status RetryFunc(const MonoTime& deadline,
                  const string& retry_msg,
                  const string& timeout_msg,
@@ -483,15 +484,15 @@ Status KuduClient::Data::ListTabletServers(KuduClient* client,
   return Status::OK();
 }
 
+bool KuduClient::Data::FetchCachedAuthzToken(const string& table_id, SignedTokenPB* token) {
+  return authz_token_cache_.Fetch(table_id, token);
+}
+
 void KuduClient::Data::StoreAuthzToken(const string& table_id,
                                        const SignedTokenPB& token) {
   authz_token_cache_.Put(table_id, token);
 }
 
-bool KuduClient::Data::FetchCachedAuthzToken(const string& table_id, SignedTokenPB* token) {
-  return authz_token_cache_.Fetch(table_id, token);
-}
-
 Status KuduClient::Data::OpenTable(KuduClient* client,
                                    const TableIdentifierPB& table_identifier,
                                    client::sp::shared_ptr<KuduTable>* table) {
@@ -560,7 +561,7 @@ Status KuduClient::Data::GetTableSchema(KuduClient* client,
   rpc.SendRpc();
   RETURN_NOT_OK(sync.Wait());
   // Parse the server schema out of the response.
-  unique_ptr<Schema> new_schema(new Schema());
+  unique_ptr<Schema> new_schema(new Schema);
   RETURN_NOT_OK(SchemaFromPB(resp.schema(), new_schema.get()));
 
   // Parse the server partition schema out of the response.
@@ -660,9 +661,15 @@ void KuduClient::Data::ConnectedToClusterCb(
       hive_metastore_uuid_ = hive_config.hms_uuid();
 
       location_ = connect_response.client_location();
+      cluster_id_ = connect_response.cluster_id();
 
-      master_proxy_.reset(new MasterServiceProxy(messenger_, leader_addr, leader_hostname));
+      master_proxy_ = std::make_shared<MasterServiceProxy>(
+          messenger_, leader_addr, leader_hostname);
       master_proxy_->set_user_credentials(user_credentials_);
+
+      txn_manager_proxy_ = std::make_shared<TxnManagerServiceProxy>(
+          messenger_, leader_addr, leader_hostname);
+      txn_manager_proxy_->set_user_credentials(user_credentials_);
     }
   }
 
@@ -685,6 +692,10 @@ void KuduClient::Data::ConnectToClusterAsync(KuduClient* client,
                                              CredentialsPolicy creds_policy) {
   DCHECK(deadline.Initialized());
 
+  if (master_server_addrs_.empty()) {
+    cb(Status::InvalidArgument("no master address specified"));
+  }
+
   vector<pair<Sockaddr, string>> master_addrs_with_names;
   for (const string& master_server_addr : master_server_addrs_) {
     vector<Sockaddr> addrs;
@@ -810,11 +821,22 @@ string KuduClient::Data::location() const {
   return location_;
 }
 
+string KuduClient::Data::cluster_id() const {
+  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  return cluster_id_;
+}
+
 shared_ptr<master::MasterServiceProxy> KuduClient::Data::master_proxy() const {
   std::lock_guard<simple_spinlock> l(leader_master_lock_);
   return master_proxy_;
 }
 
+std::shared_ptr<transactions::TxnManagerServiceProxy>
+KuduClient::Data::txn_manager_proxy() const {
+  std::lock_guard<simple_spinlock> l(leader_master_lock_);
+  return txn_manager_proxy_;
+}
+
 uint64_t KuduClient::Data::GetLatestObservedTimestamp() const {
   return latest_observed_timestamp_.Load();
 }
