@@ -56,6 +56,7 @@
 #include "kudu/client/table_statistics-internal.h"
 #include "kudu/client/tablet-internal.h"
 #include "kudu/client/tablet_server-internal.h"
+#include "kudu/client/transaction-internal.h"
 #include "kudu/client/value.h"
 #include "kudu/client/write_op.h"
 #include "kudu/common/common.pb.h"
@@ -65,6 +66,7 @@
 #include "kudu/common/row_operations.h"
 #include "kudu/common/scan_spec.h"
 #include "kudu/common/schema.h"
+#include "kudu/common/txn_id.h"
 #include "kudu/common/wire_protocol.h"
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/metadata.pb.h"
@@ -99,10 +101,9 @@
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/version_info.h"
 
-namespace kudu {
-class BlockBloomFilter;
-}  // namespace kudu
-
+using kudu::client::internal::AsyncLeaderMasterRpc;
+using kudu::client::internal::MetaCache;
+using kudu::client::sp::shared_ptr;
 using kudu::consensus::RaftPeerPB;
 using kudu::master::AlterTableRequestPB;
 using kudu::master::AlterTableResponsePB;
@@ -158,16 +159,13 @@ struct tm;
 
 namespace kudu {
 
+class BlockBloomFilter;
 class simple_spinlock;
 
 namespace client {
 
 class ResourceMetrics;
 
-using internal::AsyncLeaderMasterRpc;
-using internal::MetaCache;
-using sp::shared_ptr;
-
 const char* kVerboseEnvVar = "KUDU_CLIENT_VERBOSE";
 
 #if defined(kudu_client_exported_EXPORTS)
@@ -303,6 +301,12 @@ KuduClientBuilder& KuduClientBuilder::default_rpc_timeout(const MonoDelta& timeo
   return *this;
 }
 
+KuduClientBuilder& KuduClientBuilder::connection_negotiation_timeout(
+    const MonoDelta& timeout) {
+  data_->connection_negotiation_timeout_ = timeout;
+  return *this;
+}
+
 KuduClientBuilder& KuduClientBuilder::import_authentication_credentials(string authn_creds) {
   data_->authn_creds_ = std::move(authn_creds);
   return *this;
@@ -349,6 +353,10 @@ Status KuduClientBuilder::Build(shared_ptr<KuduClient>* client) {
 
   // Init messenger.
   MessengerBuilder builder("client");
+  if (data_->connection_negotiation_timeout_.Initialized()) {
+    builder.set_rpc_negotiation_timeout_ms(
+        data_->connection_negotiation_timeout_.ToMilliseconds());
+  }
   if (data_->num_reactors_) {
     builder.set_num_reactors(data_->num_reactors_.get());
   }
@@ -391,6 +399,61 @@ Status KuduClientBuilder::Build(shared_ptr<KuduClient>* client) {
   return Status::OK();
 }
 
+KuduTransaction::SerializationOptions::SerializationOptions()
+    : data_(new Data) {
+}
+
+KuduTransaction::SerializationOptions::~SerializationOptions() {
+  delete data_;
+}
+
+bool KuduTransaction::SerializationOptions::keepalive() const {
+  return data_->enable_keepalive_;
+}
+
+KuduTransaction::SerializationOptions&
+KuduTransaction::SerializationOptions::enable_keepalive(bool enable) {
+  data_->enable_keepalive_ = enable;
+  return *this;
+}
+
+KuduTransaction::KuduTransaction(const sp::shared_ptr<KuduClient>& client)
+    : data_(new KuduTransaction::Data(client)) {
+}
+
+KuduTransaction::~KuduTransaction() {
+  delete data_;
+}
+
+Status KuduTransaction::CreateSession(sp::shared_ptr<KuduSession>* session) {
+  return data_->CreateSession(session);
+}
+
+Status KuduTransaction::Commit(bool wait) {
+  return data_->Commit(wait);
+}
+
+Status KuduTransaction::IsCommitComplete(
+    bool* is_complete, Status* completion_status) {
+  return data_->IsCommitComplete(is_complete, completion_status);
+}
+
+Status KuduTransaction::Rollback() {
+  return data_->Rollback();
+}
+
+Status KuduTransaction::Serialize(
+    string* serialized_txn,
+    const SerializationOptions& options) const {
+  return data_->Serialize(serialized_txn, options);
+}
+
+Status KuduTransaction::Deserialize(const sp::shared_ptr<KuduClient>& client,
+                                    const string& serialized_txn,
+                                    sp::shared_ptr<KuduTransaction>* txn) {
+  return Data::Deserialize(client, serialized_txn, txn);
+}
+
 KuduClient::KuduClient()
   : data_(new KuduClient::Data()) {
   static ObjectIdGenerator oid_generator;
@@ -520,6 +583,15 @@ shared_ptr<KuduSession> KuduClient::NewSession() {
   return ret;
 }
 
+Status KuduClient::NewTransaction(sp::shared_ptr<KuduTransaction>* txn) {
+  shared_ptr<KuduTransaction> ret(new KuduTransaction(shared_from_this()));
+  const auto s = ret->data_->Begin(ret);
+  if (s.ok()) {
+    *txn = std::move(ret);
+  }
+  return s;
+}
+
 Status KuduClient::GetTablet(const string& tablet_id, KuduTablet** tablet) {
   GetTabletLocationsRequestPB req;
   GetTabletLocationsResponsePB resp;
@@ -631,6 +703,12 @@ const MonoDelta& KuduClient::default_rpc_timeout() const {
   return data_->default_rpc_timeout_;
 }
 
+MonoDelta KuduClient::connection_negotiation_timeout() const {
+  DCHECK(data_->messenger_);
+  return MonoDelta::FromMilliseconds(
+      data_->messenger_->rpc_negotiation_timeout_ms());
+}
+
 const uint64_t KuduClient::kNoTimestamp = 0;
 
 uint64_t KuduClient::GetLatestObservedTimestamp() const {
@@ -683,6 +761,10 @@ string KuduClient::location() const {
   return data_->location();
 }
 
+string KuduClient::cluster_id() const {
+  return data_->cluster_id();
+}
+
 ////////////////////////////////////////////////////////////
 // KuduTableCreator
 ////////////////////////////////////////////////////////////
@@ -962,10 +1044,18 @@ KuduUpdate* KuduTable::NewUpdate() {
   return new KuduUpdate(shared_from_this());
 }
 
+KuduUpdateIgnore* KuduTable::NewUpdateIgnore() {
+  return new KuduUpdateIgnore(shared_from_this());
+}
+
 KuduDelete* KuduTable::NewDelete() {
   return new KuduDelete(shared_from_this());
 }
 
+KuduDeleteIgnore* KuduTable::NewDeleteIgnore() {
+  return new KuduDeleteIgnore(shared_from_this());
+}
+
 KuduClient* KuduTable::client() const {
   return data_->client_.get();
 }
@@ -1149,7 +1239,11 @@ KuduError::~KuduError() {
 ////////////////////////////////////////////////////////////
 
 KuduSession::KuduSession(const shared_ptr<KuduClient>& client)
-  : data_(new KuduSession::Data(client, client->data_->messenger_)) {
+    : data_(new KuduSession::Data(client, client->data_->messenger_)) {
+}
+
+KuduSession::KuduSession(const shared_ptr<KuduClient>& client, const TxnId& txn_id)
+    : data_(new KuduSession::Data(client, client->data_->messenger_, txn_id)) {
 }
 
 KuduSession::~KuduSession() {
@@ -1157,10 +1251,6 @@ KuduSession::~KuduSession() {
   delete data_;
 }
 
-Status KuduSession::Close() {
-  return data_->Close(false);
-}
-
 Status KuduSession::SetFlushMode(FlushMode m) {
   if (!tight_enum_test<FlushMode>(m)) {
     // Be paranoid in client code.
@@ -1198,18 +1288,6 @@ void KuduSession::SetTimeoutMillis(int timeout_ms) {
   data_->SetTimeoutMillis(timeout_ms);
 }
 
-Status KuduSession::Flush() {
-  return data_->Flush();
-}
-
-void KuduSession::FlushAsync(KuduStatusCallback* user_callback) {
-  data_->FlushAsync(user_callback);
-}
-
-bool KuduSession::HasPendingOperations() const {
-  return data_->HasPendingOperations();
-}
-
 Status KuduSession::Apply(KuduWriteOperation* write_op) {
   RETURN_NOT_OK(data_->ApplyWriteOp(write_op));
   // Thread-safety note: this method should not be called concurrently
@@ -1221,6 +1299,22 @@ Status KuduSession::Apply(KuduWriteOperation* write_op) {
   return Status::OK();
 }
 
+Status KuduSession::Flush() {
+  return data_->Flush();
+}
+
+void KuduSession::FlushAsync(KuduStatusCallback* user_callback) {
+  data_->FlushAsync(user_callback);
+}
+
+Status KuduSession::Close() {
+  return data_->Close(false);
+}
+
+bool KuduSession::HasPendingOperations() const {
+  return data_->HasPendingOperations();
+}
+
 int KuduSession::CountBufferedOperations() const {
   return data_->CountBufferedOperations();
 }
