@@ -19,6 +19,7 @@
 
 #include <algorithm>
 #include <cstring>
+#include <iterator>
 #include <set>
 #include <string>
 #include <unordered_set>
@@ -26,7 +27,6 @@
 #include <vector>
 
 #include <glog/logging.h>
-#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/key_encoder.h"
@@ -115,46 +115,49 @@ namespace {
 Status ExtractColumnIds(const RepeatedPtrField<PartitionSchemaPB_ColumnIdentifierPB>& identifiers,
                         const Schema& schema,
                         vector<ColumnId>* column_ids) {
-    column_ids->reserve(identifiers.size());
-    for (const auto& identifier : identifiers) {
-      switch (identifier.identifier_case()) {
-        case PartitionSchemaPB_ColumnIdentifierPB::kId: {
-          ColumnId column_id(identifier.id());
-          if (schema.find_column_by_id(column_id) == Schema::kColumnNotFound) {
-            return Status::InvalidArgument("unknown column id", SecureDebugString(identifier));
-          }
-          column_ids->push_back(column_id);
-          continue;
+  vector<ColumnId> new_column_ids;
+  new_column_ids.reserve(identifiers.size());
+  for (const auto& identifier : identifiers) {
+    switch (identifier.identifier_case()) {
+      case PartitionSchemaPB_ColumnIdentifierPB::kId: {
+        ColumnId column_id(identifier.id());
+        if (schema.find_column_by_id(column_id) == Schema::kColumnNotFound) {
+          return Status::InvalidArgument("unknown column id", SecureDebugString(identifier));
         }
-        case PartitionSchemaPB_ColumnIdentifierPB::kName: {
-          int32_t column_idx = schema.find_column(identifier.name());
-          if (column_idx == Schema::kColumnNotFound) {
-            return Status::InvalidArgument("unknown column", SecureDebugString(identifier));
-          }
-          column_ids->push_back(schema.column_id(column_idx));
-          continue;
-        }
-        default: return Status::InvalidArgument("unknown column", SecureDebugString(identifier));
+        new_column_ids.emplace_back(std::move(column_id));
+        continue;
       }
+      case PartitionSchemaPB_ColumnIdentifierPB::kName: {
+        int32_t column_idx = schema.find_column(identifier.name());
+        if (column_idx == Schema::kColumnNotFound) {
+          return Status::InvalidArgument("unknown column", SecureDebugString(identifier));
+        }
+        new_column_ids.emplace_back(schema.column_id(column_idx));
+        continue;
+      }
+      default: return Status::InvalidArgument("unknown column", SecureDebugString(identifier));
     }
-    return Status::OK();
+  }
+  *column_ids = std::move(new_column_ids);
+  return Status::OK();
 }
+
 // Sets a repeated field of column identifiers to the provided column IDs.
 void SetColumnIdentifiers(const vector<ColumnId>& column_ids,
                           RepeatedPtrField<PartitionSchemaPB_ColumnIdentifierPB>* identifiers) {
-    identifiers->Reserve(column_ids.size());
-    for (const ColumnId& column_id : column_ids) {
-      identifiers->Add()->set_id(column_id);
-    }
+  identifiers->Reserve(column_ids.size());
+  for (const ColumnId& column_id : column_ids) {
+    identifiers->Add()->set_id(column_id);
+  }
 }
 } // namespace
 
-Status PartitionSchema::FromPB(const PartitionSchemaPB& pb,
-                               const Schema& schema,
-                               PartitionSchema* partition_schema) {
-  partition_schema->Clear();
 
-  for (const PartitionSchemaPB_HashBucketSchemaPB& hash_bucket_pb : pb.hash_bucket_schemas()) {
+Status PartitionSchema::ExtractHashBucketSchemasFromPB(
+    const Schema& schema,
+    const RepeatedPtrField<PartitionSchemaPB_HashBucketSchemaPB>& hash_buckets_pb,
+    HashBucketSchemas* hash_bucket_schemas) {
+  for (const PartitionSchemaPB_HashBucketSchemaPB& hash_bucket_pb : hash_buckets_pb) {
     HashBucketSchema hash_bucket;
     RETURN_NOT_OK(ExtractColumnIds(hash_bucket_pb.columns(), schema, &hash_bucket.column_ids));
 
@@ -166,8 +169,17 @@ Status PartitionSchema::FromPB(const PartitionSchemaPB& pb,
 
     hash_bucket.seed = hash_bucket_pb.seed();
     hash_bucket.num_buckets = hash_bucket_pb.num_buckets();
-    partition_schema->hash_bucket_schemas_.push_back(hash_bucket);
+    hash_bucket_schemas->push_back(std::move(hash_bucket));
   }
+  return Status::OK();
+}
+
+Status PartitionSchema::FromPB(const PartitionSchemaPB& pb,
+                               const Schema& schema,
+                               PartitionSchema* partition_schema) {
+  partition_schema->Clear();
+  RETURN_NOT_OK(ExtractHashBucketSchemasFromPB(schema, pb.hash_bucket_schemas(),
+                             &partition_schema->hash_bucket_schemas_));
 
   if (pb.has_range_schema()) {
     const PartitionSchemaPB_RangeSchemaPB& range_pb = pb.range_schema();
@@ -248,6 +260,9 @@ Status PartitionSchema::EncodeRangeSplits(const vector<KuduPartialRow>& split_ro
                                           const Schema& schema,
                                           vector<string>* splits) const {
   DCHECK(splits->empty());
+  if (split_rows.empty()) {
+    return Status::OK();
+  }
   for (const KuduPartialRow& row : split_rows) {
     string split;
     RETURN_NOT_OK(EncodeRangeKey(row, schema, &split));
@@ -268,14 +283,17 @@ Status PartitionSchema::EncodeRangeSplits(const vector<KuduPartialRow>& split_ro
 
 Status PartitionSchema::EncodeRangeBounds(const vector<pair<KuduPartialRow,
                                                             KuduPartialRow>>& range_bounds,
+                                          const RangeHashSchema& range_hash_schemas,
                                           const Schema& schema,
-                                          vector<pair<string, string>>* range_partitions) const {
-  DCHECK(range_partitions->empty());
+                                          vector<RangeWithHashSchemas>*
+                                              bounds_with_hash_schemas) const {
+  DCHECK(bounds_with_hash_schemas->empty());
   if (range_bounds.empty()) {
-    range_partitions->emplace_back("", "");
+    bounds_with_hash_schemas->emplace_back(RangeWithHashSchemas{"", "", {}});
     return Status::OK();
   }
 
+  int j = 0;
   for (const auto& bound : range_bounds) {
     string lower;
     string upper;
@@ -287,24 +305,31 @@ Status PartitionSchema::EncodeRangeBounds(const vector<pair<KuduPartialRow,
           "range partition lower bound must be less than the upper bound",
           RangePartitionDebugString(bound.first, bound.second));
     }
-    range_partitions->emplace_back(std::move(lower), std::move(upper));
+    RangeWithHashSchemas temp{std::move(lower), std::move(upper), {}};
+    if (!range_hash_schemas.empty()) {
+      temp.hash_schemas = range_hash_schemas[j++];
+    }
+    bounds_with_hash_schemas->emplace_back(std::move(temp));
   }
 
+  std::sort(bounds_with_hash_schemas->begin(), bounds_with_hash_schemas->end(),
+            [](const RangeWithHashSchemas& s1, const RangeWithHashSchemas& s2) {
+    return s1.lower < s2.lower;
+  });
   // Check that the range bounds are non-overlapping
-  std::sort(range_partitions->begin(), range_partitions->end());
-  for (int i = 0; i < range_partitions->size() - 1; i++) {
-    const string& first_upper = range_partitions->at(i).second;
-    const string& second_lower = range_partitions->at(i + 1).first;
+  for (int i = 0; i < bounds_with_hash_schemas->size() - 1; i++) {
+    const string& first_upper = bounds_with_hash_schemas->at(i).upper;
+    const string& second_lower = bounds_with_hash_schemas->at(i + 1).lower;
 
     if (first_upper.empty() || second_lower.empty() || first_upper > second_lower) {
       return Status::InvalidArgument(
           "overlapping range partitions",
           strings::Substitute("first range partition: $0, second range partition: $1",
-                              RangePartitionDebugString(range_partitions->at(i).first,
-                                                        range_partitions->at(i).second,
+                              RangePartitionDebugString(bounds_with_hash_schemas->at(i).lower,
+                                                        bounds_with_hash_schemas->at(i).upper,
                                                         schema),
-                              RangePartitionDebugString(range_partitions->at(i + 1).first,
-                                                        range_partitions->at(i + 1).second,
+                              RangePartitionDebugString(bounds_with_hash_schemas->at(i + 1).lower,
+                                                        bounds_with_hash_schemas->at(i + 1).upper,
                                                         schema)));
     }
   }
@@ -314,19 +339,23 @@ Status PartitionSchema::EncodeRangeBounds(const vector<pair<KuduPartialRow,
 
 Status PartitionSchema::SplitRangeBounds(const Schema& schema,
                                          vector<string> splits,
-                                         vector<pair<string, string>>* bounds) const {
-  int expected_bounds = std::max(1UL, bounds->size()) + splits.size();
+                                         vector<RangeWithHashSchemas>*
+                                             bounds_with_hash_schemas) const {
+  if (splits.empty()) {
+    return Status::OK();
+  }
 
-  vector<pair<string, string>> new_bounds;
-  new_bounds.reserve(expected_bounds);
+  auto expected_bounds = std::max(1UL, bounds_with_hash_schemas->size()) + splits.size();
+  vector<RangeWithHashSchemas> new_bounds_with_hash_schemas;
+  new_bounds_with_hash_schemas.reserve(expected_bounds);
 
   // Iterate through the sorted bounds and sorted splits, splitting the bounds
   // as appropriate and adding them to the result list ('new_bounds').
 
   auto split = splits.begin();
-  for (auto& bound : *bounds) {
-    string& lower = bound.first;
-    const string& upper = bound.second;
+  for (auto& bound : *bounds_with_hash_schemas) {
+    string& lower = bound.lower;
+    const string& upper = bound.upper;
 
     for (; split != splits.end() && (upper.empty() || *split <= upper); split++) {
       if (!lower.empty() && *split < lower) {
@@ -338,36 +367,33 @@ Status PartitionSchema::SplitRangeBounds(const Schema& schema,
       }
       // Split the current bound. Add the lower section to the result list,
       // and continue iterating on the upper section.
-      new_bounds.emplace_back(std::move(lower), *split);
+      new_bounds_with_hash_schemas.emplace_back(RangeWithHashSchemas{std::move(lower), *split, {}});
       lower = std::move(*split);
     }
 
-    new_bounds.emplace_back(std::move(lower), upper);
+    new_bounds_with_hash_schemas.emplace_back(RangeWithHashSchemas{std::move(lower), upper, {}});
   }
 
   if (split != splits.end()) {
     return Status::InvalidArgument("split out of bounds", RangeKeyDebugString(*split, schema));
   }
 
-  bounds->swap(new_bounds);
-  CHECK_EQ(expected_bounds, bounds->size());
+  bounds_with_hash_schemas->swap(new_bounds_with_hash_schemas);
+  CHECK_EQ(expected_bounds, bounds_with_hash_schemas->size());
   return Status::OK();
 }
 
-Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_rows,
-                                         const vector<pair<KuduPartialRow,
-                                                           KuduPartialRow>>& range_bounds,
-                                         const Schema& schema,
-                                         vector<Partition>* partitions) const {
-  const KeyEncoder<string>& hash_encoder = GetKeyEncoder<string>(GetTypeInfo(UINT32));
-
-  // Create a partition per hash bucket combination.
-  *partitions = vector<Partition>(1);
-  for (const HashBucketSchema& bucket_schema : hash_bucket_schemas_) {
+vector<Partition> PartitionSchema::GenerateHashPartitions(const HashBucketSchemas& hash_schemas,
+                                                          const KeyEncoder<string>& hash_encoder) {
+  vector<Partition> hash_partitions(1);
+  // Create a partition for each hash bucket combination.
+  for (const HashBucketSchema& bucket_schema : hash_schemas) {
+    auto expected_partitions = hash_partitions.size() * bucket_schema.num_buckets;
     vector<Partition> new_partitions;
+    new_partitions.reserve(expected_partitions);
     // For each of the partitions created so far, replicate it
-    // by the number of buckets in the next hash bucketing component
-    for (const Partition& base_partition : *partitions) {
+    // by the number of buckets in the next hash bucketing component.
+    for (const Partition& base_partition : hash_partitions) {
       for (int32_t bucket = 0; bucket < bucket_schema.num_buckets; bucket++) {
         Partition partition = base_partition;
         partition.hash_buckets_.push_back(bucket);
@@ -376,8 +402,32 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
         new_partitions.push_back(partition);
       }
     }
-    partitions->swap(new_partitions);
+    hash_partitions = std::move(new_partitions);
   }
+  return hash_partitions;
+}
+
+Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_rows,
+                                         const vector<pair<KuduPartialRow,
+                                                           KuduPartialRow>>& range_bounds,
+                                         const RangeHashSchema& range_hash_schemas,
+                                         const Schema& schema,
+                                         vector<Partition>* partitions) const {
+  const auto& hash_encoder = GetKeyEncoder<string>(GetTypeInfo(UINT32));
+
+  if (!range_hash_schemas.empty()) {
+    if (!split_rows.empty()) {
+      return Status::InvalidArgument("Both 'split_rows' and 'range_hash_schemas' cannot be "
+                                     "populated at the same time.");
+      }
+    if (range_bounds.size() != range_hash_schemas.size()) {
+      return Status::InvalidArgument("The number of range bounds does not match the number of per "
+                                     "range hash schemas.");
+    }
+  }
+
+  vector<Partition> base_hash_partitions = GenerateHashPartitions(hash_bucket_schemas_,
+                                                                  hash_encoder);
 
   std::unordered_set<int> range_column_idxs;
   for (const ColumnId& column_id : range_schema_.column_ids) {
@@ -392,24 +442,54 @@ Status PartitionSchema::CreatePartitions(const vector<KuduPartialRow>& split_row
     }
   }
 
-  vector<pair<string, string>> bounds;
+  vector<RangeWithHashSchemas> bounds_with_hash_schemas;
   vector<string> splits;
-  RETURN_NOT_OK(EncodeRangeBounds(range_bounds, schema, &bounds));
+  RETURN_NOT_OK(EncodeRangeBounds(range_bounds, range_hash_schemas, schema,
+                                  &bounds_with_hash_schemas));
   RETURN_NOT_OK(EncodeRangeSplits(split_rows, schema, &splits));
-  RETURN_NOT_OK(SplitRangeBounds(schema, std::move(splits), &bounds));
+  RETURN_NOT_OK(SplitRangeBounds(schema, std::move(splits), &bounds_with_hash_schemas));
 
-  // Create a partition per range bound and hash bucket combination.
-  vector<Partition> new_partitions;
-  for (const Partition& base_partition : *partitions) {
-    for (const auto& bound : bounds) {
-      Partition partition = base_partition;
-      partition.partition_key_start_.append(bound.first);
-      partition.partition_key_end_.append(bound.second);
-      new_partitions.push_back(partition);
+  if (!range_hash_schemas.empty()) {
+    // The number of ranges should match the size of range_hash_schemas.
+    DCHECK_EQ(range_hash_schemas.size(), bounds_with_hash_schemas.size());
+    // No split rows should be defined if range_hash_schemas is populated.
+    DCHECK(split_rows.empty());
+    vector<Partition> result_partitions;
+    // Iterate through each bound and its hash schemas to generate hash partitions.
+    for (const auto& bound : bounds_with_hash_schemas) {
+      const auto& current_range_hash_schemas = bound.hash_schemas;
+      vector<Partition> current_bound_hash_partitions;
+      // If current bound's HashBucketSchema is empty, implies use of default table-wide schema.
+      // If not empty, generate hash partitions for all the provided hash schemas in this range.
+      if (current_range_hash_schemas.empty()) {
+        current_bound_hash_partitions = base_hash_partitions;
+      } else {
+        current_bound_hash_partitions = GenerateHashPartitions(current_range_hash_schemas,
+                                                               hash_encoder);
+      }
+      // Adds range part to partition key.
+      for (Partition& partition : current_bound_hash_partitions) {
+        partition.partition_key_start_.append(bound.lower);
+        partition.partition_key_end_.append(bound.upper);
+      }
+      result_partitions.insert(result_partitions.end(),
+                               std::make_move_iterator(current_bound_hash_partitions.begin()),
+                               std::make_move_iterator(current_bound_hash_partitions.end()));
     }
+    *partitions = std::move(result_partitions);
+  } else {
+    // Create a partition per range bound and hash bucket combination.
+    vector<Partition> new_partitions;
+    for (const Partition &base_partition : base_hash_partitions) {
+      for (const auto& bound : bounds_with_hash_schemas) {
+        Partition partition = base_partition;
+        partition.partition_key_start_.append(bound.lower);
+        partition.partition_key_end_.append(bound.upper);
+        new_partitions.push_back(partition);
+      }
+    }
+    *partitions = std::move(new_partitions);
   }
-  partitions->swap(new_partitions);
-
   // Note: the following discussion and logic only takes effect when the table's
   // partition schema includes at least one hash bucket component, and the
   // absolute upper and/or absolute lower range bound is unbounded.
@@ -458,17 +538,37 @@ Status PartitionSchema::PartitionContainsRowImpl(const Partition& partition,
                                                  bool* contains) const {
   CHECK_EQ(partition.hash_buckets().size(), hash_bucket_schemas_.size());
   for (int i = 0; i < hash_bucket_schemas_.size(); i++) {
-    const HashBucketSchema& hash_bucket_schema = hash_bucket_schemas_[i];
-    int32_t bucket;
-    RETURN_NOT_OK(BucketForRow(row, hash_bucket_schema, &bucket));
-
-    if (bucket != partition.hash_buckets()[i]) {
-      *contains = false;
+    RETURN_NOT_OK(HashPartitionContainsRowImpl(partition, row, i, contains));
+    if (!*contains) {
       return Status::OK();
     }
   }
 
+  RETURN_NOT_OK(RangePartitionContainsRowImpl(partition, row, contains));
+
+  return Status::OK();
+}
+
+template<typename Row>
+Status PartitionSchema::HashPartitionContainsRowImpl(const Partition& partition,
+                                                     const Row& row,
+                                                     int hash_idx,
+                                                     bool* contains) const {
+  DCHECK_EQ(partition.hash_buckets().size(), hash_bucket_schemas_.size());
+  const HashBucketSchema& hash_bucket_schema = hash_bucket_schemas_[hash_idx];
+  int32_t bucket = -1;
+  RETURN_NOT_OK(BucketForRow(row, hash_bucket_schema, &bucket));
+  *contains = (partition.hash_buckets()[hash_idx] == bucket);
+  return Status::OK();
+}
+
+template<typename Row>
+Status PartitionSchema::RangePartitionContainsRowImpl(const Partition& partition,
+                                                      const Row& row,
+                                                      bool* contains) const {
   string range_partition_key;
+  // If range partition is not used, column_ids would be empty and
+  // EncodedColumn() would return immediately.
   RETURN_NOT_OK(EncodeColumns(row, range_schema_.column_ids, &range_partition_key));
 
   // If all of the hash buckets match, then the row is contained in the
@@ -477,7 +577,6 @@ Status PartitionSchema::PartitionContainsRowImpl(const Partition& partition,
   *contains = (Slice(range_partition_key).compare(partition.range_key_start()) >= 0)
            && (partition.range_key_end().empty()
                 || Slice(range_partition_key).compare(partition.range_key_end()) < 0);
-
   return Status::OK();
 }
 
@@ -493,11 +592,36 @@ Status PartitionSchema::PartitionContainsRow(const Partition& partition,
   return PartitionContainsRowImpl(partition, row, contains);
 }
 
+Status PartitionSchema::HashPartitionContainsRow(const Partition& partition,
+                                                 const KuduPartialRow& row,
+                                                 int hash_idx,
+                                                 bool* contains) const {
+  return HashPartitionContainsRowImpl(partition, row, hash_idx, contains);
+}
+
+Status PartitionSchema::HashPartitionContainsRow(const Partition& partition,
+                                                 const ConstContiguousRow& row,
+                                                 int hash_idx,
+                                                 bool* contains) const {
+  return HashPartitionContainsRowImpl(partition, row, hash_idx, contains);
+}
+
+Status PartitionSchema::RangePartitionContainsRow(const Partition& partition,
+                                                  const KuduPartialRow& row,
+                                                  bool* contains) const {
+  return RangePartitionContainsRowImpl(partition, row, contains);
+}
+
+Status PartitionSchema::RangePartitionContainsRow(const Partition& partition,
+                                                  const ConstContiguousRow& row,
+                                                  bool* contains) const {
+  return RangePartitionContainsRowImpl(partition, row, contains);
+}
 
 Status PartitionSchema::DecodeRangeKey(Slice* encoded_key,
-                                       KuduPartialRow* row,
+                                       KuduPartialRow* partial_row,
                                        Arena* arena) const {
-  ContiguousRow cont_row(row->schema(), row->row_data_);
+  ContiguousRow cont_row(partial_row->schema(), partial_row->row_data_);
   for (int i = 0; i < range_schema_.column_ids.size(); i++) {
     if (encoded_key->empty()) {
       // This can happen when decoding partition start and end keys, since they
@@ -505,8 +629,8 @@ Status PartitionSchema::DecodeRangeKey(Slice* encoded_key,
       break;
     }
 
-    int32_t column_idx = row->schema()->find_column_by_id(range_schema_.column_ids[i]);
-    const ColumnSchema& column = row->schema()->column(column_idx);
+    int32_t column_idx = partial_row->schema()->find_column_by_id(range_schema_.column_ids[i]);
+    const ColumnSchema& column = partial_row->schema()->column(column_idx);
     const KeyEncoder<faststring>& key_encoder = GetKeyEncoder<faststring>(column.type_info());
     bool is_last = i == (range_schema_.column_ids.size() - 1);
 
@@ -518,7 +642,7 @@ Status PartitionSchema::DecodeRangeKey(Slice* encoded_key,
                           Substitute("Error decoding partition key range component '$0'",
                                      column.name()));
     // Mark the column as set.
-    BitmapSet(row->isset_bitmap_, column_idx);
+    BitmapSet(partial_row->isset_bitmap_, column_idx);
   }
   if (!encoded_key->empty()) {
     return Status::InvalidArgument("unable to fully decode range key",
@@ -585,6 +709,7 @@ namespace {
 string ColumnIdsToColumnNames(const Schema& schema,
                               const vector<ColumnId>& column_ids) {
   vector<string> names;
+  names.reserve(column_ids.size());
   for (const ColumnId& column_id : column_ids) {
     names.push_back(schema.column(schema.find_column_by_id(column_id)).name());
   }
@@ -913,10 +1038,10 @@ Status PartitionSchema::EncodeColumns(const KuduPartialRow& row,
   return Status::OK();
 }
 
-int32_t PartitionSchema::BucketForEncodedColumns(const string& encoded_key,
+int32_t PartitionSchema::BucketForEncodedColumns(const string& encoded_hash_columns,
                                                  const HashBucketSchema& hash_bucket_schema) {
-  uint64_t hash = HashUtil::MurmurHash2_64(encoded_key.data(),
-                                           encoded_key.length(),
+  uint64_t hash = HashUtil::MurmurHash2_64(encoded_hash_columns.data(),
+                                           encoded_hash_columns.length(),
                                            hash_bucket_schema.seed);
   return hash % static_cast<uint64_t>(hash_bucket_schema.num_buckets);
 }
@@ -1273,4 +1398,22 @@ Status PartitionSchema::GetRangeSchemaColumnIndexes(const Schema& schema,
   return Status::OK();
 }
 
+int32_t PartitionSchema::TryGetSingleColumnHashPartitionIndex(const Schema& schema,
+                                                              int32_t col_idx) const {
+  const ColumnId column_id = schema.column_id(col_idx);
+  for (int i = 0; i < hash_partition_schemas().size(); ++i) {
+    auto hash_partition = hash_partition_schemas()[i];
+    if (hash_partition.column_ids.size() == 1 && hash_partition.column_ids[0] == column_id) {
+      return i;
+    }
+  }
+  return -1;
+}
+
+bool PartitionSchema::IsColumnSingleRangeSchema(const Schema& schema, int32_t col_idx) const {
+  const ColumnId column_id = schema.column_id(col_idx);
+  return range_partition_schema().column_ids.size() == 1 &&
+         range_partition_schema().column_ids[0] == column_id;
+}
+
 } // namespace kudu
