@@ -14,8 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
-#ifndef KUDU_COMMON_PARTITION_H
-#define KUDU_COMMON_PARTITION_H
+#pragma once
 
 #include <cstdint>
 #include <string>
@@ -29,13 +28,21 @@
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 
+namespace google {
+namespace protobuf {
+template <typename T> class RepeatedPtrField;
+}  // namespace protobuf
+}  // namespace google
+
 namespace kudu {
 
 class Arena;
 class ConstContiguousRow;
 class KuduPartialRow;
-class PartitionSchemaPB;
 class PartitionPB;
+class PartitionSchemaPB;
+class PartitionSchemaPB_HashBucketSchemaPB;
+template <typename Buffer> class KeyEncoder;
 
 // A Partition describes the set of rows that a Tablet is responsible for
 // serving. Each tablet is assigned a single Partition.
@@ -146,6 +153,23 @@ class PartitionSchema {
     uint32_t seed;
   };
 
+  typedef std::vector<HashBucketSchema> HashBucketSchemas;
+  // Holds each bound's HashBucketSchemas.
+  typedef std::vector<HashBucketSchemas> RangeHashSchema;
+
+  struct RangeWithHashSchemas {
+    std::string lower;
+    std::string upper;
+    HashBucketSchemas hash_schemas;
+  };
+
+  // Extracts HashBucketSchemas from a protobuf repeated field of hash buckets.
+  static Status ExtractHashBucketSchemasFromPB(
+      const Schema& schema,
+      const google::protobuf::RepeatedPtrField<PartitionSchemaPB_HashBucketSchemaPB>&
+          hash_buckets_pb,
+      HashBucketSchemas* hash_bucket_schemas);
+
   // Deserializes a protobuf message into a partition schema.
   static Status FromPB(const PartitionSchemaPB& pb,
                        const Schema& schema,
@@ -168,9 +192,14 @@ class PartitionSchema {
   // of resulting partitions is the product of the number of hash buckets for
   // each hash bucket component, multiplied by
   // (split_rows.size() + max(1, range_bounds.size())).
+  // 'range_hash_schema' contains each range's HashBucketSchemas,
+  // its order corresponds to the bounds in 'range_bounds'.
+  // If 'range_hash_schemas' is empty, the table wide hash schema is used per range.
+  // Size of 'range_hash_schemas' and 'range_bounds' are equal if 'range_hash_schema' isn't empty.
   Status CreatePartitions(const std::vector<KuduPartialRow>& split_rows,
                           const std::vector<std::pair<KuduPartialRow,
                                                       KuduPartialRow>>& range_bounds,
+                          const RangeHashSchema& range_hash_schemas,
                           const Schema& schema,
                           std::vector<Partition>* partitions) const WARN_UNUSED_RESULT;
 
@@ -182,6 +211,24 @@ class PartitionSchema {
                               const ConstContiguousRow& row,
                               bool* contains) const WARN_UNUSED_RESULT;
 
+  // Tests if the hash partition contains the row with given hash_idx.
+  Status HashPartitionContainsRow(const Partition& partition,
+                                  const KuduPartialRow& row,
+                                  int hash_idx,
+                                  bool* contains) const WARN_UNUSED_RESULT;
+  Status HashPartitionContainsRow(const Partition& partition,
+                                  const ConstContiguousRow& row,
+                                  int hash_idx,
+                                  bool* contains) const WARN_UNUSED_RESULT;
+
+  // Tests if the range partition contains the row.
+  Status RangePartitionContainsRow(const Partition& partition,
+                                   const KuduPartialRow& row,
+                                   bool* contains) const WARN_UNUSED_RESULT;
+  Status RangePartitionContainsRow(const Partition& partition,
+                                   const ConstContiguousRow& row,
+                                   bool* contains) const WARN_UNUSED_RESULT;
+
   // Returns a text description of the partition suitable for debug printing.
   //
   // Partitions are considered metadata, so no redaction will happen on the hash
@@ -245,7 +292,7 @@ class PartitionSchema {
 
   // Decodes a range partition key into a partial row, with variable-length
   // fields stored in the arena.
-  Status DecodeRangeKey(Slice* encode_key,
+  Status DecodeRangeKey(Slice* encoded_key,
                         KuduPartialRow* partial_row,
                         Arena* arena) const;
 
@@ -253,7 +300,7 @@ class PartitionSchema {
     return range_schema_;
   }
 
-  const std::vector<HashBucketSchema>& hash_partition_schemas() const {
+  const HashBucketSchemas& hash_partition_schemas() const {
     return hash_bucket_schemas_;
   }
 
@@ -263,6 +310,13 @@ class PartitionSchema {
   Status GetRangeSchemaColumnIndexes(const Schema& schema,
                                      std::vector<int>* range_column_idxs) const;
 
+  // Returns index of given column idx, if it is one of hash key and this hash schema
+  // contains only one column, otherwise returns -1.
+  int32_t TryGetSingleColumnHashPartitionIndex(const Schema& schema, int32_t col_idx) const;
+
+  // Given a column idx, verify that it is the only column of the range partition.
+  bool IsColumnSingleRangeSchema(const Schema& schema, int32_t col_idx) const;
+
  private:
   friend class PartitionPruner;
   FRIEND_TEST(PartitionTest, TestIncrementRangePartitionBounds);
@@ -307,6 +361,19 @@ class PartitionSchema {
                                   const Row& row,
                                   bool* contains) const;
 
+  // Private templated helper for HashPartitionContainsRow.
+  template<typename Row>
+  Status HashPartitionContainsRowImpl(const Partition& partition,
+                                      const Row& row,
+                                      int hash_idx,
+                                      bool* contains) const;
+
+  // Private templated helper for RangePartitionContainsRow.
+  template<typename Row>
+  Status RangePartitionContainsRowImpl(const Partition& partition,
+                                       const Row& row,
+                                       bool* contains) const;
+
   // Private templated helper for EncodeKey.
   template<typename Row>
   Status EncodeKeyImpl(const Row& row, std::string* buf) const;
@@ -323,7 +390,7 @@ class PartitionSchema {
   void AppendRangeDebugStringComponentsOrMin(const KuduPartialRow& row,
                                              std::vector<std::string>* components) const;
 
-  // Returns the stringified hash and range schema componenets of the partition
+  // Returns the stringified hash and range schema components of the partition
   // schema.
   //
   // Partition schemas are considered metadata, so no redaction will happen on
@@ -342,7 +409,7 @@ class PartitionSchema {
   //
   // This should only be called with partition keys created from a row, not with
   // partition keys from a partition.
-  Status DecodeHashBuckets(Slice* partition_key, std::vector<int32_t>* buckets) const;
+  Status DecodeHashBuckets(Slice* encoded_key, std::vector<int32_t>* buckets) const;
 
   // Clears the state of this partition schema.
   void Clear();
@@ -351,6 +418,10 @@ class PartitionSchema {
   // appropriate error code for an invalid partition schema.
   Status Validate(const Schema& schema) const;
 
+  // Generates hash partitions for each combination of hash buckets in hash_schemas.
+  static std::vector<Partition> GenerateHashPartitions(const HashBucketSchemas& hash_schemas,
+                                                       const KeyEncoder<std::string>& hash_encoder);
+
   // Validates the split rows, converts them to partition key form, and inserts
   // them into splits in sorted order.
   Status EncodeRangeSplits(const std::vector<KuduPartialRow>& split_rows,
@@ -358,30 +429,30 @@ class PartitionSchema {
                            std::vector<std::string>* splits) const;
 
   // Validates the range bounds, converts them to partition key form, and
-  // inserts them into encoded_range_partitions in sorted order.
+  // inserts them into 'bounds_with_hash_schemas' in sorted order. The hash schemas
+  // per range are stored within 'range_hash_schemas'. If 'range_hash_schemas' is empty,
+  // it indicates that the table wide hash schema will be used per range.
   Status EncodeRangeBounds(const std::vector<std::pair<KuduPartialRow,
                                                        KuduPartialRow>>& range_bounds,
+                           const RangeHashSchema& range_hash_schemas,
                            const Schema& schema,
-                           std::vector<std::pair<std::string,
-                                                 std::string>>* encoded_range_bounds) const;
+                           std::vector<RangeWithHashSchemas>* bounds_with_hash_schemas) const;
 
-  // Splits the encoded range bounds by the split points. The splits and bounds
-  // must be sorted. If `bounds` is empty, then a single unbounded range is
-  // assumed. If any of the splits falls outside of the bounds then an
-  // InvalidArgument status is returned.
+  // Splits the encoded range bounds by the split points. The splits and bounds within
+  // 'bounds_with_hash_schemas' must be sorted. If `bounds_with_hash_schemas` is empty,
+  // then a single unbounded range is assumed. If any of the splits falls outside
+  // of the bounds, then an InvalidArgument status is returned.
   Status SplitRangeBounds(const Schema& schema,
                           std::vector<std::string> splits,
-                          std::vector<std::pair<std::string, std::string>>* bounds) const;
+                          std::vector<RangeWithHashSchemas>* bounds_with_hash_schemas) const;
 
   // Increments a range partition key, setting 'increment' to true if the
   // increment succeeds, or false if all range partition columns are already the
   // maximum value. Unset columns will be incremented to increment(min_value).
   Status IncrementRangePartitionKey(KuduPartialRow* row, bool* increment) const;
 
-  std::vector<HashBucketSchema> hash_bucket_schemas_;
+  HashBucketSchemas hash_bucket_schemas_;
   RangeSchema range_schema_;
 };
 
 } // namespace kudu
-
-#endif
