@@ -16,6 +16,7 @@
 // under the License.
 #pragma once
 
+#include <atomic>
 #include <cstdint>
 #include <memory>
 #include <ostream>
@@ -31,6 +32,7 @@
 #include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/util/locks.h"
+#include "kudu/util/make_shared.h"
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/status.h"
 
@@ -65,7 +67,9 @@ class PeerProxy;
 // object, and performed on a thread pool (since it may do IO). When a
 // response is received, the peer updates the PeerMessageQueue
 // using PeerMessageQueue::ResponseFromPeer(...) on the same thread pool.
-class Peer : public std::enable_shared_from_this<Peer> {
+class Peer :
+    public std::enable_shared_from_this<Peer>,
+    public enable_make_shared<Peer> {
  public:
   // Initializes a peer and start sending periodic heartbeats.
   Status Init();
@@ -115,7 +119,7 @@ class Peer : public std::enable_shared_from_this<Peer> {
                               std::shared_ptr<rpc::Messenger> messenger,
                               std::shared_ptr<Peer>* peer);
 
- private:
+ protected:
   Peer(RaftPeerPB peer_pb,
        std::string tablet_id,
        std::string leader_uuid,
@@ -124,6 +128,7 @@ class Peer : public std::enable_shared_from_this<Peer> {
        std::unique_ptr<PeerProxy> proxy,
        std::shared_ptr<rpc::Messenger> messenger);
 
+ private:
   void SendNextRequest(bool even_if_queue_empty);
 
   // Signals that a response was received from the peer.
@@ -147,7 +152,7 @@ class Peer : public std::enable_shared_from_this<Peer> {
   void ProcessTabletCopyResponse();
 
   // Signals there was an error sending the request to the peer.
-  void ProcessResponseError(const Status& status);
+  void ProcessResponseErrorUnlocked(const Status& status);
 
   std::string LogPrefixUnlocked() const;
 
@@ -189,11 +194,18 @@ class Peer : public std::enable_shared_from_this<Peer> {
   // Repeating timer responsible for scheduling heartbeats to this peer.
   std::shared_ptr<rpc::PeriodicTimer> heartbeater_;
 
-  // Lock that protects Peer state changes, initialization, etc.
-  mutable simple_spinlock peer_lock_;
-  bool request_pending_ = false;
-  bool closed_ = false;
-  bool has_sent_first_request_ = false;
+  // Lock that protects Peer state changes, initialization, etc. It's necessary
+  // to hold 'peer_lock_' if setting 'request_pending_', 'closed_', and
+  // 'has_sent_first_request_' fields below. To read 'has_sent_first_request_',
+  // it's necessary to hold 'peer_lock_'. To read the 'closed_' and
+  // 'request_pending_' fields, there is no need to hold 'peer_lock_' unless
+  // it's necessary to block the threads which might be setting these fields
+  // concurrently.
+  simple_spinlock peer_lock_;
+
+  std::atomic<bool> request_pending_;
+  std::atomic<bool> closed_;
+  bool has_sent_first_request_;
 };
 
 // A proxy to another peer. Usually a thin wrapper around an rpc proxy but can
