@@ -86,6 +86,10 @@ struct ServerContext {
 
   // Threadpool on which to run Raft tasks.
   ThreadPool* raft_pool;
+
+  // Shared boolean indicating whether Raft consensus should continue sending request messages
+  // even if a peer is considered as failed.
+  const bool* allow_status_msg_for_failed_peer = nullptr;
 };
 
 struct ConsensusOptions {
@@ -142,6 +146,8 @@ class RaftConsensus : public std::enable_shared_from_this<RaftConsensus>,
     EXTERNAL_REQUEST
   };
 
+  typedef std::pair<RaftPeerPB::Role, RaftPeerPB::MemberType> RoleAndMemberType;
+
   ~RaftConsensus();
 
   // Factory method to construct and initialize a RaftConsensus instance.
@@ -317,6 +323,10 @@ class RaftConsensus : public std::enable_shared_from_this<RaftConsensus>,
   // Returns the current Raft role of this instance.
   RaftPeerPB::Role role() const;
 
+  // Returns the current Raft role and member type of this instance.
+  // May return <UNKNOWN_ROLE, UNKNOWN_MEMBER_TYPE> if the information is not available.
+  RoleAndMemberType GetRoleAndMemberType() const;
+
   // Returns the current term.
   int64_t CurrentTerm() const;
 
