@@ -74,6 +74,10 @@ void MiniHms::EnableKuduPlugin(bool enable) {
   enable_kudu_plugin_ = enable;
 }
 
+void MiniHms::AddEnvVar(const string& key, const string& value) {
+  extra_env_vars_.insert({key, value});
+}
+
 void MiniHms::SetDataRoot(string data_root) {
   CHECK(!hms_process_);
   data_root_ = std::move(data_root);
@@ -102,6 +106,7 @@ Status MiniHms::Start() {
     data_root_ = GetTestDataDirectory();
   }
 
+  RETURN_NOT_OK(CreateSecurityProperties());
   RETURN_NOT_OK(CreateHiveSite());
   RETURN_NOT_OK(CreateCoreSite());
   RETURN_NOT_OK(CreateLogConfig());
@@ -127,6 +132,10 @@ Status MiniHms::Start() {
   // directory. We want it in data_root_.
   java_options += Substitute(" -Dderby.stream.error.file=$0/derby.log", data_root_);
 
+  // Add the security properties file generated by `CreateSecurityProperties()`.
+  java_options += Substitute(" -Djava.security.properties=$0/security.properties",
+      data_root_);
+
   if (!krb5_conf_.empty()) {
     java_options += Substitute(" -Djava.security.krb5.conf=$0", krb5_conf_);
   }
@@ -142,6 +151,7 @@ Status MiniHms::Start() {
       // TODO(ghenke): Remove after HADOOP-15966 is available (Hadoop 3.1.3+)
       { "HADOOP_OS_TYPE", "Linux" }
   };
+  env_vars.insert(extra_env_vars_.begin(), extra_env_vars_.end());
 
   // Run the schematool to initialize the database if not yet initialized.
   // Instead of running slow 'schematool -dbType derby -info' to check whether
@@ -205,6 +215,21 @@ string MiniHms::uris() const {
   return Substitute("thrift://127.0.0.1:$0", port_);
 }
 
+Status MiniHms::CreateSecurityProperties() const {
+  // Default Java security settings are restrictive with regard to RSA key
+  // length. Since Kudu masters and tablet servers in MiniKuduCluster use
+  // smaller RSA keys to shorten runtime of tests, it's necessary to override
+  // those default security settings to allow for using relaxed cryptography,
+  // particularly smaller RSA keys.
+  string security_file_contents =
+      "jdk.certpath.disabledAlgorithms = MD2, RC4, MD5\n"
+      "jdk.tls.disabledAlgorithms = SSLv3, RC4, MD5\n);";
+
+  return WriteStringToFile(Env::Default(),
+                           security_file_contents,
+                           JoinPathSegments(data_root_, "security.properties"));
+}
+
 Status MiniHms::CreateHiveSite() const {
 
   const string listeners = Substitute("org.apache.hive.hcatalog.listener.DbNotificationListener$0",
