@@ -16,9 +16,11 @@
 // under the License.
 
 #include <cstdint>
-#include <cstdio>
+#include <functional>
 #include <memory>
+#include <ostream>
 #include <string>
+#include <thread>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -29,7 +31,10 @@
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/gutil/map-util.h"
+#include "kudu/gutil/port.h"
 #include "kudu/gutil/stl_util.h"
+#include "kudu/gutil/strings/join.h"
+#include "kudu/gutil/strings/substitute.h"
 #include "kudu/integration-tests/cluster_itest_util.h"
 #include "kudu/integration-tests/cluster_verifier.h"
 #include "kudu/integration-tests/external_mini_cluster-itest-base.h"
@@ -42,8 +47,8 @@
 #include "kudu/tools/tool_test_util.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/path_util.h"
 #include "kudu/util/net/sockaddr.h"
+#include "kudu/util/path_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
@@ -58,19 +63,23 @@ using kudu::master::ChangeTServerStateResponsePB;
 using kudu::cluster::ExternalDaemon;
 using kudu::cluster::ExternalMiniClusterOptions;
 using kudu::cluster::ExternalTabletServer;
+using kudu::cluster::LocationInfo;
 using kudu::consensus::ConsensusStatePB;
 using kudu::consensus::HealthReportPB;
 using kudu::consensus::IncludeHealthReport;
 using kudu::itest::GetInt64Metric;
 using kudu::master::MasterServiceProxy;
 using kudu::master::TServerStateChangePB;
-using kudu::tools::RunKuduTool;
+using kudu::tools::RunActionPrependStdoutStderr;
+using std::function;
 using std::pair;
 using std::shared_ptr;
 using std::string;
+using std::thread;
 using std::unique_ptr;
 using std::unordered_map;
 using std::vector;
+using strings::Substitute;
 
 namespace kudu {
 namespace itest {
@@ -82,7 +91,7 @@ static const vector<string> kTServerFlags = {
   "--raft_heartbeat_interval_ms=100",
   "--follower_unavailable_considered_failed_sec=2",
   // Disable log GC in case our write workloads lead to eviction because
-  // consensus will consider replicas that are too fare behind unrecoverable
+  // consensus will consider replicas that are too far behind unrecoverable
   // and will evict them regardless of maintenance mode.
   "--enable_log_gc=false",
 };
@@ -334,19 +343,12 @@ TEST_F(MaintenanceModeRF3ITest, TestMaintenanceModeDoesntObstructMove) {
   // While the maintenance mode tserver is still online, move a tablet from it.
   // This should succeed, because maintenance mode will not obstruct manual
   // movement of replicas.
-  {
-    vector<string> move_cmd = {
-      "tablet",
-      "change_config",
-      "move_replica",
+  ASSERT_OK(RunActionPrependStdoutStderr(Substitute(
+      "tablet change_config move_replica $0 $1 $2 $3",
       cluster_->master()->bound_rpc_addr().ToString(),
       mnt_tablet_ids[0],
       maintenance_uuid,
-      added_uuid,
-    };
-    string stdout, stderr;
-    ASSERT_OK(RunKuduTool(move_cmd, &stdout, &stderr));
-  }
+      added_uuid)));
   const TServerDetails* added_details = FindOrDie(ts_map, added_uuid);
   ASSERT_EVENTUALLY([&] {
     vector<string> added_tablet_ids;
@@ -487,5 +489,324 @@ TEST_F(MaintenanceModeRF5ITest, TestBackgroundFailureDuringMaintenanceMode) {
   });
 }
 
+namespace {
+
+// Performs the given tasks in parallel, returning an error if any of them
+// return a non-OK Status.
+Status DoInParallel(vector<function<Status()>> tasks,
+                    const string& task_description) {
+  int num_in_parallel = tasks.size();
+  vector<Status> results(num_in_parallel);
+  vector<thread> threads;
+  for (int i = 0; i < num_in_parallel; i++) {
+    threads.emplace_back([&, i] {
+      Status s = tasks[i]();
+      if (PREDICT_FALSE(!s.ok())) {
+        results[i] = s;
+      }
+    });
+  }
+  for (auto& t : threads) {
+    t.join();
+  }
+  bool has_errors = false;
+  for (int i = 0; i < num_in_parallel; i++) {
+    const auto& s = results[i];
+    if (!s.ok()) {
+      LOG(ERROR) << s.ToString();
+      has_errors = true;
+    }
+  }
+  if (has_errors) {
+    return Status::IllegalState(
+        Substitute("errors while running $0 $1 tasks in parallel",
+                    num_in_parallel, task_description));
+  }
+  return Status::OK();
+}
+
+// Repeats 'cmd' until it succeeds, with the given timeout and interval.
+Status RunUntilSuccess(const string& cmd, int timeout_secs, int repeat_interval_secs) {
+  Status s;
+  const MonoTime deadline = MonoTime::Now() + MonoDelta::FromSeconds(timeout_secs);
+  const MonoDelta retry_interval = MonoDelta::FromSeconds(repeat_interval_secs);
+  while (true) {
+    s = RunActionPrependStdoutStderr(cmd);
+    if (s.ok()) {
+      return Status::OK();
+    }
+    if (MonoTime::Now() + retry_interval > deadline) {
+      return Status::TimedOut(
+          Substitute("Running '$0' did not succeed in $1 seconds: $2",
+                      cmd, timeout_secs, s.ToString()));
+    }
+    SleepFor(retry_interval);
+  }
+  return Status::OK();
+}
+
+// Generates locations such that there are a total of 'num_tservers' tablet
+// servers spread as evenly as possible across 'num_locs' locations.
+ExternalMiniClusterOptions GenerateOpts(int num_tservers, int num_locs) {
+  int tservers_per_loc = num_tservers / num_locs;
+  int tservers_with_one_more = num_tservers % num_locs;
+  CHECK_LT(0, tservers_per_loc);
+  LocationInfo locations;
+  for (int l = 0; l < num_locs; l++) {
+    string loc = Substitute("/L$0", l);
+    EmplaceOrDie(&locations, Substitute("/L$0", l),
+        tservers_with_one_more < l ? tservers_per_loc + 1 : tservers_per_loc);
+  }
+
+  ExternalMiniClusterOptions opts;
+  opts.num_tablet_servers = num_tservers;
+  opts.extra_master_flags = {
+    // Don't bother assigning locations to clients; it's hard to do that
+    // correctly with external mini clusters.
+    "--master_client_location_assignment_enabled=false",
+  };
+  opts.extra_tserver_flags = {
+    // To speed up leadership transfers from quiescing, let's make heartbeats
+    // more frequent.
+    "--raft_heartbeat_interval_ms=100",
+    // Let's emulate long-running scans by setting a low scan batch size.
+    "--scanner_default_batch_size_bytes=100",
+  };
+  opts.location_info = std::move(locations);
+  return opts;
+}
+
+struct RollingRestartTestArgs {
+  // Cluster opts.
+  ExternalMiniClusterOptions opts;
+  // Upper bound on the number of tablet servers to restart at the same time.
+  int batch_size;
+  // Replication to use for the test workload.
+  int num_replicas;
+  // Whether the rolling restart should fail.
+  bool restart_fails;
+};
+
+// Convenience builder for more readable test composition.
+class ArgsBuilder {
+ public:
+  ArgsBuilder& batch_size(int batch_size) {
+    args_.batch_size = batch_size;
+    return *this;
+  }
+  ArgsBuilder& num_locations(int num_locations) {
+    num_locations_ = num_locations;
+    return *this;
+  }
+  ArgsBuilder& num_replicas(int num_replicas) {
+    args_.num_replicas = num_replicas;
+    return *this;
+  }
+  ArgsBuilder& num_tservers(int num_tservers) {
+    num_tservers_ = num_tservers;
+    return *this;
+  }
+  ArgsBuilder& restart_fails(bool fails) {
+    args_.restart_fails = fails;
+    return *this;
+  }
+  RollingRestartTestArgs build() {
+    args_.opts = GenerateOpts(num_tservers_, num_locations_);
+    return args_;
+  }
+ private:
+  RollingRestartTestArgs args_;
+  int num_tservers_ = 0;
+  int num_locations_ = 0;
+};
+
+} // anonymous namespace
+
+class RollingRestartITest : public MaintenanceModeITest,
+                            public ::testing::WithParamInterface<RollingRestartTestArgs> {
+ public:
+  void SetUp() override {
+    SKIP_IF_SLOW_NOT_ALLOWED();
+    NO_FATALS(MaintenanceModeITest::SetUp());
+    const auto& args = GetParam();
+    ExternalMiniClusterOptions opts = args.opts;
+    NO_FATALS(StartClusterWithOpts(std::move(opts)));
+    const auto& addr = cluster_->master(0)->bound_rpc_addr();
+    m_proxy_.reset(new MasterServiceProxy(cluster_->messenger(), addr, addr.host()));
+    NO_FATALS(GenerateTServerMap(&ts_map_and_deleter_));
+  }
+  void TearDown() override {
+    SKIP_IF_SLOW_NOT_ALLOWED();
+    NO_FATALS(MaintenanceModeITest::TearDown());
+  }
+
+  // Create a read-write workload that doesn't use a fault-tolerant scanner.
+  unique_ptr<TestWorkload> CreateFaultIntolerantRWWorkload(int num_replicas) {
+    unique_ptr<TestWorkload> rw_workload(new TestWorkload(cluster_.get()));
+    rw_workload->set_scanner_fault_tolerant(false);
+    rw_workload->set_num_replicas(num_replicas);
+    rw_workload->set_num_read_threads(3);
+    rw_workload->set_num_write_threads(3);
+    rw_workload->set_verify_num_rows(false);
+    return rw_workload;
+  }
+
+  // Returns a list of batches of tablet server UUIDs to restart in parallel,
+  // each of size at most 'batch_size'. Batches are generated within each
+  // location.
+  vector<vector<string>> GetRestartBatches(int batch_size) {
+    unordered_map<string, vector<string>> cur_tservers_by_loc;
+    vector<vector<string>> restart_batches;
+    for (const auto& ts_and_details : ts_map_and_deleter_.first) {
+      const auto& uuid = ts_and_details.first;
+      const auto& loc = ts_and_details.second->location;
+      auto& cur_batch = LookupOrInsert(&cur_tservers_by_loc, loc, {});
+      cur_batch.emplace_back(uuid);
+      // If we've reached our desired batch size for this location, put it in
+      // the return set.
+      if (cur_batch.size() >= batch_size) {
+        restart_batches.emplace_back(std::move(cur_batch));
+        CHECK_EQ(1, cur_tservers_by_loc.erase(loc));
+      }
+    }
+    // Create batches out of the remaining, suboptimally-sized batches.
+    for (auto& ts_and_batch : cur_tservers_by_loc) {
+      restart_batches.emplace_back(std::move(ts_and_batch.second));
+    }
+    return restart_batches;
+  }
+
+  // Takes the list of tablet servers and restarts them in parallel in such a
+  // way that shouldn't affect on-going workloads.
+  Status RollingRestartTServers(const vector<string>& ts_to_restart) {
+    // Begin maintenance mode on the servers so we'll stop assigning replicas
+    // to them, and begin quiescing them so the server itself stops accepting
+    // more work.
+    vector<function<Status()>> setup_rr_tasks;
+    for (const auto& ts_id : ts_to_restart) {
+      setup_rr_tasks.emplace_back([&, ts_id] {
+        RETURN_NOT_OK(RunActionPrependStdoutStderr(
+            Substitute("tserver state enter_maintenance $0 $1",
+                       cluster_->master()->bound_rpc_addr().ToString(), ts_id)));
+        return RunUntilSuccess(
+            Substitute("tserver quiesce start $0 --error_if_not_fully_quiesced",
+                       cluster_->tablet_server_by_uuid(ts_id)->bound_rpc_addr().ToString()),
+            /*timeout_secs*/30, /*repeat_interval_secs*/1);
+      });
+    }
+    RETURN_NOT_OK(DoInParallel(std::move(setup_rr_tasks), "rolling restart setup"));
+
+    // Restart the tservers. Note: we don't do this in parallel because
+    // wrangling multiple processes from different threads is messy.
+    for (const auto& ts_id : ts_to_restart) {
+      ExternalTabletServer* ts = cluster_->tablet_server_by_uuid(ts_id);
+      ts->Shutdown();
+      RETURN_NOT_OK(ts->Restart());
+    }
+
+    // Wait for ksck to become healthy.
+    RETURN_NOT_OK_PREPEND(RunUntilSuccess(
+        Substitute("cluster ksck $0", cluster_->master()->bound_rpc_addr().ToString()),
+        /*timeout_secs*/60, /*repeat_interval_secs*/5),
+        "cluster didn't become healthy");
+
+    // Now clean up persistent state.
+    vector<function<Status()>> cleanup_tasks;
+    for (const auto& ts_id : ts_to_restart) {
+      cleanup_tasks.emplace_back([&, ts_id] {
+        return RunActionPrependStdoutStderr(
+            Substitute("tserver state exit_maintenance $0 $1",
+                       cluster_->master()->bound_rpc_addr().ToString(),
+                       ts_id));
+
+      });
+    }
+    return DoInParallel(std::move(cleanup_tasks), "rolling restart cleanup");
+  }
+
+ protected:
+  MapAndDeleter ts_map_and_deleter_;
+};
+
+TEST_P(RollingRestartITest, TestWorkloads) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+  const auto& args = GetParam();
+  unique_ptr<TestWorkload> rw = CreateFaultIntolerantRWWorkload(args.num_replicas);
+  rw->set_read_timeout_millis(10000);
+  // If we're expecting the rolling restart to fail, e.g. because we can't
+  // fully quiesce our servers, chances are our workload can't complete either
+  // because too many servers are quiescing and none can serve scans or writes.
+  rw->set_read_errors_allowed(args.restart_fails);
+  rw->set_timeout_allowed(args.restart_fails);
+  rw->Setup();
+  rw->Start();
+  vector<vector<string>> restart_batches = GetRestartBatches(args.batch_size);
+  for (const auto& batch : restart_batches) {
+    LOG(INFO) << Substitute("Restarting batch of $0 tservers: $1",
+                            batch.size(), JoinStrings(batch, ","));
+    if (args.restart_fails) {
+      ASSERT_EVENTUALLY([&] {
+        Status s = RollingRestartTServers(batch);
+        ASSERT_FALSE(s.ok());
+      });
+    } else {
+      ASSERT_OK(RollingRestartTServers(batch));
+    }
+  }
+  NO_FATALS(rw->StopAndJoin());
+  if (args.restart_fails) {
+    ASSERT_FALSE(rw->read_errors().empty());
+  }
+}
+
+INSTANTIATE_TEST_CASE_P(RollingRestartArgs, RollingRestartITest, ::testing::Values(
+    // Basic RF=3 case.
+    ArgsBuilder().num_tservers(4)
+                 .num_locations(1)
+                 .batch_size(1)
+                 .num_replicas(3)
+                 .restart_fails(false)
+                 .build(),
+    // Basic RF=5 case. The larger replication factor lets us increase the
+    // restart batch size.
+    ArgsBuilder().num_tservers(6)
+                 .num_locations(1)
+                 .batch_size(2)
+                 .num_replicas(5)
+                 .restart_fails(false)
+                 .build(),
+    // RF=3 case with location awareness. The location awareness lets us
+    // increase the restart batch size.
+    ArgsBuilder().num_tservers(6)
+                 .num_locations(3)
+                 .batch_size(2)
+                 .num_replicas(3)
+                 .restart_fails(false)
+                 .build(),
+    // RF=3 case with location awareness, but with an even larger batch size.
+    ArgsBuilder().num_tservers(9)
+                 .num_locations(3)
+                 .batch_size(3)
+                 .num_replicas(3)
+                 .restart_fails(false)
+                 .build(),
+    // Basic RF=3 case, but with too large a batch size. With too large a batch
+    // size, the tablet servers won't be able to fully relinquish leadership
+    // and quiesce; the restart process should thus fail.
+    ArgsBuilder().num_tservers(4)
+                 .num_locations(1)
+                 .batch_size(4)
+                 .num_replicas(3)
+                 .restart_fails(true)
+                 .build(),
+    // The same goes for the case with a single replica.
+    ArgsBuilder().num_tservers(1)
+                 .num_locations(1)
+                 .batch_size(1)
+                 .num_replicas(1)
+                 .restart_fails(true)
+                 .build()
+));
+
 } // namespace itest
 } // namespace kudu
