@@ -16,6 +16,7 @@
 // under the License.
 
 #include <cstdint>
+#include <functional>
 #include <memory>
 #include <mutex>
 #include <ostream>
@@ -47,8 +48,8 @@
 #include "kudu/master/master.proxy.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
 #include "kudu/rpc/messenger.h"
+#include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc_controller.h"
-#include "kudu/sentry/mini_sentry.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/thrift/client.h"
 #include "kudu/tools/tool_action_common.h"
@@ -91,7 +92,6 @@ using kudu::cluster::ExternalMiniCluster;
 using kudu::cluster::ExternalMiniClusterOptions;
 using kudu::hms::HmsClient;
 using kudu::itest::ListTablets;
-using kudu::itest::SentryMode;
 using kudu::master::ListTablesRequestPB;
 using kudu::master::ListTablesResponsePB;
 using kudu::master::ReplaceTabletRequestPB;
@@ -114,9 +114,9 @@ namespace kudu {
 static const MonoDelta kDefaultAdminTimeout = MonoDelta::FromSeconds(300);
 static const MonoDelta kTransientStateBackoff = MonoDelta::FromMilliseconds(50);
 
-// Parameterized based on HmsMode and whether or not to enable Sentry integration.
+// Parameterized based on HmsMode.
 class MasterStressTest : public ExternalMiniClusterITestBase,
-                         public ::testing::WithParamInterface<pair<HmsMode, SentryMode>> {
+                         public ::testing::WithParamInterface<HmsMode> {
  public:
   MasterStressTest()
     : done_(1),
@@ -146,10 +146,7 @@ class MasterStressTest : public ExternalMiniClusterITestBase,
     opts.start_process_timeout = MonoDelta::FromSeconds(60);
     opts.rpc_negotiation_timeout = MonoDelta::FromSeconds(30);
 
-    opts.hms_mode = std::get<0>(GetParam());
-    bool enable_sentry = (std::get<1>(GetParam()) == SentryMode::ENABLED);
-    opts.enable_sentry = enable_sentry;
-    opts.enable_kerberos = enable_sentry;
+    opts.hms_mode = GetParam();
     // Tune down the notification log poll period in order to speed up catalog convergence.
     opts.extra_master_flags.emplace_back("--hive_metastore_notification_log_poll_period_seconds=1");
 
@@ -208,13 +205,8 @@ class MasterStressTest : public ExternalMiniClusterITestBase,
         new MasterServiceProxy(cluster_->messenger(), addr, addr.host()));
     ASSERT_OK(CreateTabletServerMap(m_proxy, cluster_->messenger(), &ts_map_));
 
-    if (enable_sentry) {
-      itest::SetupAdministratorPrivileges(cluster_->kdc(),
-                                          cluster_->sentry()->address());
-    }
-    if (std::get<0>(GetParam()) == HmsMode::ENABLE_METASTORE_INTEGRATION) {
+    if (GetParam() == HmsMode::ENABLE_METASTORE_INTEGRATION) {
       thrift::ClientOptions hms_opts;
-      hms_opts.enable_kerberos = enable_sentry;
       hms_opts.service_principal = "hive";
       hms_client_.reset(new HmsClient(cluster_->hms()->address(), hms_opts));
       ASSERT_OK(hms_client_->Start());
@@ -533,14 +525,9 @@ class MasterStressTest : public ExternalMiniClusterITestBase,
   std::unordered_map<string, itest::TServerDetails*> ts_map_;
 };
 
-// Run the test with the HMS/Sentry integration enabled and disabled. Sentry integration
-// should be only enabled when HMS integration is enabled.
-INSTANTIATE_TEST_CASE_P(HmsSentryConfigurations, MasterStressTest, ::testing::ValuesIn(
-    vector<pair<HmsMode, SentryMode>> {
-      { HmsMode::NONE, SentryMode::DISABLED },
-      { HmsMode::ENABLE_METASTORE_INTEGRATION, SentryMode::DISABLED },
-      { HmsMode::ENABLE_METASTORE_INTEGRATION, SentryMode::ENABLED },
-    }
+// Run the test with the HMS integration enabled and disabled.
+INSTANTIATE_TEST_CASE_P(HmsConfigurations, MasterStressTest, ::testing::ValuesIn(
+    vector<HmsMode> { HmsMode::NONE, HmsMode::ENABLE_METASTORE_INTEGRATION }
 ));
 
 TEST_P(MasterStressTest, Test) {
