@@ -49,13 +49,11 @@
 #include "kudu/transactions/txn_system_client.h"
 #include "kudu/util/decimal_util.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/net/net_util.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
-using boost::make_optional;
 using boost::none;
 using boost::optional;
 using kudu::client::KuduClient;
@@ -310,12 +308,11 @@ class MasterAuthzITestHarness {
   // Creates db.table and db.second_table.
   virtual Status SetUpTables(const unique_ptr<ExternalMiniCluster>& cluster,
                              const shared_ptr<KuduClient>& client) {
+    static const string kUser = kAdminUser;
     RETURN_NOT_OK(CreateKuduTable(kDatabaseName, kTableName, client));
     RETURN_NOT_OK(CreateKuduTable(kDatabaseName, kSecondTable, client));
-    CheckTable(kDatabaseName, kTableName,
-               make_optional<const string&>(kAdminUser), cluster, client);
-    CheckTable(kDatabaseName, kSecondTable,
-               make_optional<const string&>(kAdminUser), cluster, client);
+    CheckTable(kDatabaseName, kTableName, kUser, cluster, client);
+    CheckTable(kDatabaseName, kSecondTable, kUser, cluster, client);
     return Status::OK();
   }
 
@@ -743,16 +740,12 @@ INSTANTIATE_TEST_CASE_P(AuthzProviders, MasterAuthzITest,
 TEST_P(MasterAuthzITest, TestCreateTransactionStatusTable) {
   // Create a transaction status table and add a range. Both requests should
   // succeed, despite no privileges being granted in Ranger.
-  vector<string> master_addrs;
-  for (const auto& hp : cluster_->master_rpc_addrs()) {
-    master_addrs.emplace_back(hp.ToString());
-  }
   ASSERT_OK(this->cluster_->kdc()->Kinit(kTestUser));
   // Attempting to call system client DDL as a non-admin should result in a
   // NotAuthorized error.
   {
     unique_ptr<TxnSystemClient> non_admin_client;
-    ASSERT_OK(TxnSystemClient::Create(master_addrs, &non_admin_client));
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &non_admin_client));
     Status s = non_admin_client->CreateTxnStatusTable(100);
     ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
     s = non_admin_client->AddTxnStatusTableRange(100, 200);
@@ -761,7 +754,7 @@ TEST_P(MasterAuthzITest, TestCreateTransactionStatusTable) {
   // But as service user, we should have no trouble making the calls.
   ASSERT_OK(this->cluster_->kdc()->Kinit(kAdminUser));
   unique_ptr<TxnSystemClient> txn_sys_client;
-  ASSERT_OK(TxnSystemClient::Create(master_addrs, &txn_sys_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client));
   ASSERT_OK(txn_sys_client->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client->AddTxnStatusTableRange(100, 200));
 }
@@ -862,8 +855,8 @@ TEST_P(MasterAuthzITest, TestTrustedUserAcl) {
     .set_range_partition_columns({"key"})
     .set_owner("another_user")
     .Create());
-  NO_FATALS(this->CheckTable(kDatabaseName, "new_table",
-                             make_optional<const string&>(kImpalaUser)));
+  const string user = kImpalaUser;
+  NO_FATALS(this->CheckTable(kDatabaseName, "new_table", user));
 }
 
 TEST_P(MasterAuthzITest, TestAuthzListTables) {
@@ -1016,7 +1009,7 @@ TEST_P(MasterAuthzOwnerITest, TestMismatchedTable) {
   ASSERT_OK(this->cluster_->CreateClient(nullptr, &client));
   shared_ptr<KuduTable> table;
   ASSERT_OK(client->OpenTable(table_name_a, &table));
-  optional<const string&> table_id_a = make_optional<const string&>(table->id());
+  optional<const string&> table_id_a = table->id();
 
   // Log back as 'test-user'.
   ASSERT_OK(this->cluster_->kdc()->Kinit(kTestUser));
