@@ -57,6 +57,7 @@ using kudu::cluster::ExternalMiniCluster;
 using kudu::cluster::ExternalMiniClusterOptions;
 using kudu::cluster::ScopedResumeExternalDaemon;
 using kudu::itest::GetInt64Metric;
+using kudu::itest::GetClusterId;
 using std::set;
 using std::string;
 using std::unique_ptr;
@@ -72,6 +73,8 @@ namespace client {
 
 const int kNumTabletServerReplicas = 3;
 
+const MonoDelta kOperationTimeout = MonoDelta::FromSeconds(30);
+
 // Parameterized based on HmsMode.
 class MasterFailoverTest : public KuduTest,
                            public ::testing::WithParamInterface<HmsMode> {
@@ -487,5 +490,28 @@ TEST_P(MasterFailoverTest, TestMasterPermanentFailure) {
   }
 }
 
+TEST_P(MasterFailoverTest, TestClusterIdOnFailover) {
+  // Validate and store the initial cluster ID.
+  int leader_idx;
+  ASSERT_OK(cluster_->GetLeaderMasterIndex(&leader_idx));
+  string original_cluster_id;
+  ASSERT_OK(GetClusterId(cluster_->master_proxy(leader_idx), kOperationTimeout,
+      &original_cluster_id));
+  ASSERT_TRUE(!original_cluster_id.empty());
+
+  LOG(INFO) << "Shutdown the leader master";
+  cluster_->master(leader_idx)->Shutdown();
+
+  // Validate the cluster ID of the new leader matches.
+  int new_leader_idx;
+  ASSERT_OK(cluster_->GetLeaderMasterIndex(&new_leader_idx));
+  ASSERT_NE(leader_idx, new_leader_idx);
+  string new_cluster_id;
+  ASSERT_OK(GetClusterId(cluster_->master_proxy(new_leader_idx), kOperationTimeout,
+                         &new_cluster_id));
+  ASSERT_TRUE(!new_cluster_id.empty());
+  ASSERT_EQ(original_cluster_id, new_cluster_id);
+}
+
 } // namespace client
 } // namespace kudu
