@@ -122,9 +122,10 @@ class SecurityUnknownTskTest : public KuduTest {
   }
 
   // Generate custom TSK.
-  Status GenerateTsk(TokenSigningPrivateKeyPB* tsk, int64_t seq_num = 100) {
+  static Status GenerateTsk(TokenSigningPrivateKeyPB* tsk, int64_t seq_num = 100) {
     PrivateKey private_key;
-    RETURN_NOT_OK(GeneratePrivateKey(512, &private_key));
+    int key_size = UseLargeKeys() ? 2048 : 512;
+    RETURN_NOT_OK(GeneratePrivateKey(key_size, &private_key));
     string private_key_str_der;
     RETURN_NOT_OK(private_key.ToString(&private_key_str_der, DataFormat::DER));
     tsk->set_rsa_key_der(private_key_str_der);
