@@ -1016,7 +1016,7 @@ Status GetPartitionForTxnStatusTablet(int64_t start_txn_id, int64_t end_txn_id,
   RETURN_NOT_OK(upper_bound.SetInt64(TxnStatusTablet::kTxnIdColName, end_txn_id));
   vector<Partition> ps;
   RETURN_NOT_OK(pschema.CreatePartitions(/*split_rows=*/{},
-      { std::make_pair(lower_bound, upper_bound) }, schema, &ps));
+      { std::make_pair(lower_bound, upper_bound) }, {}, schema, &ps));
   *partition = ps[0];
   *partition_schema = pschema;
   return Status::OK();
@@ -1052,7 +1052,8 @@ class TxnStatusTabletManagementTest : public TsTabletManagerITest {
   // Creates a tablet with a fixed schema, with the given tablet ID.
   Status CreateTablet(MiniTabletServer* ts,
                       const string& tablet_id,
-                      bool is_txn_status_tablet) {
+                      bool is_txn_status_tablet,
+                      scoped_refptr<TabletReplica>* replica = nullptr) {
     CreateTabletRequestPB req = CreateTxnTabletReq(
         tablet_id, ts->server()->fs_manager()->uuid(), ts->CreateLocalConfig(),
         is_txn_status_tablet);
@@ -1071,19 +1072,48 @@ class TxnStatusTabletManagementTest : public TsTabletManagerITest {
     RETURN_NOT_OK(admin_proxy->CreateTablet(req, &resp, &rpc));
     scoped_refptr<TabletReplica> r;
     CHECK(ts->server()->tablet_manager()->LookupTablet(tablet_id, &r));
-    return r->consensus()->WaitUntilLeaderForTests(kTimeout);
+
+    // Wait for the tablet to be in RUNNING state and its consensus running too.
+    RETURN_NOT_OK(r->WaitUntilConsensusRunning(kTimeout));
+    auto s = r->consensus()->WaitUntilLeaderForTests(kTimeout);
+    if (replica) {
+      *replica = std::move(r);
+    }
+    return s;
   }
 
-  // Creates a transaction status tablet at the given tablet server.
+  // Creates a transaction status tablet at the given tablet server,
+  // waiting for the transaction status data to be loaded from the tablet.
   Status CreateTxnStatusTablet(MiniTabletServer* ts) {
-    return CreateTablet(ts, kTxnStatusTabletId, /*is_txn_status_tablet*/true);
+    scoped_refptr<TabletReplica> r;
+    RETURN_NOT_OK(CreateTablet(
+        ts, kTxnStatusTabletId, /*is_txn_status_tablet*/true, &r));
+    TxnCoordinator* c = CHECK_NOTNULL(CHECK_NOTNULL(r)->txn_coordinator());
+
+    // Wait for the transaction status data to be loaded. The verification below
+    // relies on the internals of the TxnStatusManager's implementation, but it
+    // seems OK for test purposes.
+    const auto deadline = MonoTime::Now() + kTimeout;
+    bool is_data_loaded = false;
+    do {
+      auto txn_id = c->highest_txn_id();
+      if (txn_id >= -1) {
+        is_data_loaded = true;
+        break;
+      }
+      SleepFor(MonoDelta::FromMilliseconds(10));
+    } while (MonoTime::Now() < deadline);
+
+    return is_data_loaded
+        ? Status::OK()
+        : Status::TimedOut("timed out waiting for txn status data to be loaded");
   }
 
   static Status StartTransactions(const ParticipantIdsByTxnId& txns, TxnCoordinator* coordinator) {
     TabletServerErrorPB ts_error;
     for (const auto& txn_id_and_prt_ids : txns) {
       const auto& txn_id = txn_id_and_prt_ids.first;
-      RETURN_NOT_OK(coordinator->BeginTransaction(txn_id, kOwner, &ts_error));
+      RETURN_NOT_OK(coordinator->BeginTransaction(txn_id, kOwner, nullptr, &ts_error));
       for (const auto& prt_id : txn_id_and_prt_ids.second) {
         RETURN_NOT_OK(coordinator->RegisterParticipant(txn_id, prt_id, kOwner, &ts_error));
       }
@@ -1196,23 +1226,24 @@ TEST_F(TxnStatusTabletManagementTest, TestTabletServerProxyCalls) {
   FLAGS_superuser_acl = kSuperUser;
   FLAGS_trusted_user_acl = kTrustedUser;
   NO_FATALS(StartCluster({}));
-  auto* ts = cluster_->mini_tablet_server(0);
+  constexpr const int kTServerIdx = 0;
+  auto* ts = cluster_->mini_tablet_server(kTServerIdx);
   ASSERT_OK(CreateTxnStatusTablet(ts));
   // Put together a sequence of ops that should succeed.
   const vector<CoordinatorOpPB::CoordinatorOpType> kOpSequence = {
     CoordinatorOpPB::BEGIN_TXN,
     CoordinatorOpPB::REGISTER_PARTICIPANT,
+    CoordinatorOpPB::KEEP_TXN_ALIVE,
     CoordinatorOpPB::BEGIN_COMMIT_TXN,
     CoordinatorOpPB::ABORT_TXN,
+    CoordinatorOpPB::GET_TXN_STATUS,
   };
   // Perform the series of ops for the given transaction ID as the given user,
   // erroring out if an unexpected result is received. If 'user' is empty, use
   // the logged-in user (this user is the service user, since we just restarted
   // the server).
   const auto perform_ops = [&] (int64_t txn_id, const string& user, bool expect_success) {
-    unique_ptr<TabletServerAdminServiceProxy> admin_proxy(
-        new TabletServerAdminServiceProxy(client_messenger_, ts->bound_rpc_addr(),
-                                          ts->bound_rpc_addr().host()));
+    auto admin_proxy = cluster_->tserver_admin_proxy(kTServerIdx);
     if (!user.empty()) {
       rpc::UserCredentials user_creds;
       user_creds.set_real_user(user);
@@ -1230,7 +1261,9 @@ TEST_F(TxnStatusTabletManagementTest, TestTabletServerProxyCalls) {
       SCOPED_TRACE(SecureDebugString(resp));
       if (expect_success) {
         ASSERT_FALSE(resp.has_error());
-        ASSERT_FALSE(resp.has_op_result());
+        ASSERT_EQ(op_type == CoordinatorOpPB::BEGIN_TXN ||
+                  op_type == CoordinatorOpPB::GET_TXN_STATUS,
+                  resp.has_op_result());
       } else {
         ASSERT_TRUE(s.IsRemoteError()) << s.ToString();
         ASSERT_STR_CONTAINS(s.ToString(), "Not authorized");
@@ -1250,10 +1283,8 @@ TEST_F(TxnStatusTabletManagementTest, TestTabletServerProxyCalls) {
 TEST_F(TxnStatusTabletManagementTest, TestTabletServerProxyCallErrors) {
   NO_FATALS(StartCluster({}));
   auto* ts = cluster_->mini_tablet_server(0);
+  auto admin_proxy = cluster_->tserver_admin_proxy(0);
   ASSERT_OK(CreateTxnStatusTablet(ts));
-  unique_ptr<TabletServerAdminServiceProxy> admin_proxy(
-      new TabletServerAdminServiceProxy(client_messenger_, ts->bound_rpc_addr(),
-                                        ts->bound_rpc_addr().host()));
   // If the request is missing fields, it should be rejected.
   {
     CoordinateTransactionRequestPB req;
@@ -1321,7 +1352,8 @@ TEST_F(TxnStatusTabletManagementTest, TestTabletServerProxyCallErrors) {
       ASSERT_OK(admin_proxy->CoordinateTransaction(req, &resp, &rpc));
       SCOPED_TRACE(SecureDebugString(resp));
       ASSERT_FALSE(resp.has_error());
-      ASSERT_FALSE(resp.has_op_result());
+      ASSERT_TRUE(resp.has_op_result());
+      ASSERT_EQ(1, resp.op_result().highest_seen_txn_id());
     }
     {
       CoordinateTransactionResponsePB resp;
@@ -1330,6 +1362,7 @@ TEST_F(TxnStatusTabletManagementTest, TestTabletServerProxyCallErrors) {
       SCOPED_TRACE(SecureDebugString(resp));
       ASSERT_FALSE(resp.has_error());
       ASSERT_TRUE(resp.has_op_result());
+      ASSERT_EQ(1, resp.op_result().highest_seen_txn_id());
       Status s = StatusFromPB(resp.op_result().op_error());
       ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
       ASSERT_STR_CONTAINS(s.ToString(), "not higher than the highest ID so far");
