@@ -50,13 +50,13 @@
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/tablet/tablet_replica.h"
+#include "kudu/transactions/transactions.pb.h"
 #include "kudu/transactions/txn_status_tablet.h"
 #include "kudu/transactions/txn_system_client.h"
 #include "kudu/tserver/mini_tablet_server.h"
 #include "kudu/tserver/tablet_server.h"
 #include "kudu/tserver/ts_tablet_manager.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/net/net_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
@@ -65,19 +65,22 @@
 DECLARE_double(leader_failure_max_missed_heartbeat_periods);
 DECLARE_string(superuser_acl);
 DECLARE_string(user_acl);
+DECLARE_uint32(txn_keepalive_interval_ms);
 
-using kudu::client::sp::shared_ptr;
 using kudu::client::AuthenticationCredentialsPB;
 using kudu::client::KuduClient;
 using kudu::client::KuduClientBuilder;
 using kudu::client::KuduTable;
+using kudu::client::sp::shared_ptr;
 using kudu::cluster::InternalMiniCluster;
 using kudu::cluster::InternalMiniClusterOptions;
-using kudu::itest::TabletServerMap;
 using kudu::itest::TServerDetails;
+using kudu::itest::TabletServerMap;
 using kudu::tablet::TabletReplica;
-using kudu::transactions::TxnSystemClient;
+using kudu::transactions::TxnStatePB;
+using kudu::transactions::TxnStatusEntryPB;
 using kudu::transactions::TxnStatusTablet;
+using kudu::transactions::TxnSystemClient;
 using std::map;
 using std::string;
 using std::thread;
@@ -104,11 +107,7 @@ class TxnStatusTableITest : public KuduTest {
     ASSERT_OK(cluster_->Start());
 
     // Create the txn system client with which to communicate with the cluster.
-    vector<string> master_addrs;
-    for (const auto& hp : cluster_->master_rpc_addrs()) {
-      master_addrs.emplace_back(hp.ToString());
-    }
-    ASSERT_OK(TxnSystemClient::Create(master_addrs, &txn_sys_client_));
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client_));
   }
 
   // Ensures that all replicas have the right table type set.
@@ -323,7 +322,7 @@ TEST_F(TxnStatusTableITest, TestCreateTxnStatusTablePartitions) {
   // Now add more partitions and try again.
   ASSERT_OK(txn_sys_client_->AddTxnStatusTableRange(100, 200));
   s = txn_sys_client_->AddTxnStatusTableRange(100, 200);
-  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+  ASSERT_TRUE(s.IsAlreadyPresent()) << s.ToString();
   NO_FATALS(CheckTableTypes({ { TableTypePB::TXN_STATUS_TABLE, 2 } }));
 
   // Ensure we still create transaction status tablets even after the master is
@@ -350,26 +349,60 @@ TEST_F(TxnStatusTableITest, TestTxnStatusTableColocatedWithTables) {
 TEST_F(TxnStatusTableITest, TestSystemClientFindTablets) {
   ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
-  ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser));
+  uint32_t txn_keepalive_ms;
+  ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser, &txn_keepalive_ms));
+  ASSERT_EQ(FLAGS_txn_keepalive_interval_ms, txn_keepalive_ms);
+  ASSERT_OK(txn_sys_client_->AbortTransaction(1, kUser));
 
   // If we write out of range, we should see an error.
-  Status s = txn_sys_client_->BeginTransaction(100, kUser);
-  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  {
+    int64_t highest_seen_txn_id = -1;
+    uint32_t txn_keepalive_ms = 0;
+    auto s = txn_sys_client_->BeginTransaction(
+        100, kUser, &txn_keepalive_ms, &highest_seen_txn_id);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    // The 'highest_seen_txn_id' should be left untouched.
+    ASSERT_EQ(-1, highest_seen_txn_id);
+    // txn_keepalive_ms isn't assigned in case of non-OK status.
+    ASSERT_EQ(0, txn_keepalive_ms);
+    ASSERT_NE(0, FLAGS_txn_keepalive_interval_ms);
+  }
+  {
+    auto s = txn_sys_client_->BeginCommitTransaction(100, kUser);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  }
+  {
+    auto s = txn_sys_client_->AbortTransaction(100, kUser);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  }
 
   // Once we add a new range, we should be able to leverage it.
   ASSERT_OK(txn_sys_client_->AddTxnStatusTableRange(100, 200));
-  ASSERT_OK(txn_sys_client_->BeginTransaction(100, kUser));
+  int64_t highest_seen_txn_id = -1;
+  ASSERT_OK(txn_sys_client_->BeginTransaction(
+      100, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id));
+  ASSERT_EQ(100, highest_seen_txn_id);
+  ASSERT_OK(txn_sys_client_->BeginCommitTransaction(100, kUser));
+  ASSERT_OK(txn_sys_client_->AbortTransaction(100, kUser));
 }
 
 TEST_F(TxnStatusTableITest, TestSystemClientTServerDown) {
   ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
 
+  cluster_->mini_tablet_server(0)->Shutdown();
+
   // When the only server is down, the system client should keep trying until
   // it times out.
-  cluster_->mini_tablet_server(0)->Shutdown();
-  Status s = txn_sys_client_->BeginTransaction(1, kUser, MonoDelta::FromMilliseconds(100));
-  ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
+  {
+    int64_t highest_seen_txn_id = -1;
+    auto s = txn_sys_client_->BeginTransaction(
+        1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id,
+        MonoDelta::FromMilliseconds(100));
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
+    // The 'highest_seen_txn_id' should be left untouched.
+    ASSERT_EQ(-1, highest_seen_txn_id);
+  }
 
   // Now try with a longer timeout and ensure that if the server comes back up,
   // the system client will succeed.
@@ -382,23 +415,43 @@ TEST_F(TxnStatusTableITest, TestSystemClientTServerDown) {
   SCOPED_CLEANUP({
     t.join();
   });
-  ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser, MonoDelta::FromSeconds(3)));
+
+  int64_t highest_seen_txn_id = -1;
+  ASSERT_OK(txn_sys_client_->BeginTransaction(
+      1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id,
+      MonoDelta::FromSeconds(3)));
+  ASSERT_EQ(highest_seen_txn_id, 1);
 }
 
 TEST_F(TxnStatusTableITest, TestSystemClientBeginTransactionErrors) {
   ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
   ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
-  ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser));
+  int64_t highest_seen_txn_id = -1;
+  uint32_t txn_keepalive_ms;
+  ASSERT_OK(txn_sys_client_->BeginTransaction(
+      1, kUser, &txn_keepalive_ms, &highest_seen_txn_id));
+  ASSERT_EQ(1, highest_seen_txn_id);
+  ASSERT_EQ(FLAGS_txn_keepalive_interval_ms, txn_keepalive_ms);
 
   // Trying to start another transaction with a used ID should yield an error.
-  Status s = txn_sys_client_->BeginTransaction(1, kUser);
-  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
-  ASSERT_STR_CONTAINS(s.ToString(), "not higher than the highest ID");
+  {
+    int64_t highest_seen_txn_id = -1;
+    auto s = txn_sys_client_->BeginTransaction(
+        1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+    ASSERT_EQ(1, highest_seen_txn_id);
+    ASSERT_STR_CONTAINS(s.ToString(), "not higher than the highest ID");
+  }
 
   // The same should be true with a different user.
-  s = txn_sys_client_->BeginTransaction(1, "stranger");
-  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
-  ASSERT_STR_CONTAINS(s.ToString(), "not higher than the highest ID");
+  {
+    int64_t highest_seen_txn_id = -1;
+    auto s = txn_sys_client_->BeginTransaction(
+        1, "stranger", nullptr /* txn_keepalive_ms */, &highest_seen_txn_id);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+    ASSERT_EQ(1, highest_seen_txn_id);
+    ASSERT_STR_CONTAINS(s.ToString(), "not higher than the highest ID");
+  }
 }
 
 TEST_F(TxnStatusTableITest, TestSystemClientRegisterParticipantErrors) {
@@ -417,6 +470,123 @@ TEST_F(TxnStatusTableITest, TestSystemClientRegisterParticipantErrors) {
   ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
 }
 
+TEST_F(TxnStatusTableITest, SystemClientCommitAndAbortTransaction) {
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
+  ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
+
+  ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser));
+  ASSERT_OK(txn_sys_client_->BeginCommitTransaction(1, kUser));
+
+  // Calling BeginCommitTransaction() on already committing transaction is OK.
+  ASSERT_OK(txn_sys_client_->BeginCommitTransaction(1, kUser));
+  // It's completely legal to abort a transaction that has entered the commit
+  // phase but hasn't finalized yet.
+  ASSERT_OK(txn_sys_client_->AbortTransaction(1, kUser));
+  // Aborting already aborted transaction is fine (aborting is idempotent).
+  ASSERT_OK(txn_sys_client_->AbortTransaction(1, kUser));
+
+  // Even if the transaction is aborted, an attempt to start another transaction
+  // with already used ID should yield an error.
+  {
+    int64_t highest_seen_txn_id = -1;
+    auto s = txn_sys_client_->BeginTransaction(
+        1, kUser, nullptr /* txn_keepalive_ms */, &highest_seen_txn_id);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+    ASSERT_EQ(1, highest_seen_txn_id);
+    ASSERT_STR_CONTAINS(s.ToString(), "not higher than the highest ID");
+  }
+
+  // An attempt to start committing a non-existent transaction should report
+  // an error.
+  {
+    auto s = txn_sys_client_->BeginCommitTransaction(2, kUser);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 not found");
+  }
+
+  // An attempt to abort a non-existent transaction should report an error.
+  {
+    auto s = txn_sys_client_->AbortTransaction(2, kUser);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 not found");
+  }
+
+  // An attempt to commit already aborted transaction should fail with an error.
+  {
+    ASSERT_OK(txn_sys_client_->BeginTransaction(2, kUser));
+    ASSERT_OK(txn_sys_client_->AbortTransaction(2, kUser));
+    auto s = txn_sys_client_->BeginCommitTransaction(2, kUser);
+    ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 is not open");
+  }
+
+  // Commit/Abort transaction should be called as of the transaction's owner.
+  {
+    ASSERT_OK(txn_sys_client_->BeginTransaction(3, kUser));
+    auto s = txn_sys_client_->BeginCommitTransaction(3, "stranger");
+    ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 3 not owned by stranger");
+    s = txn_sys_client_->AbortTransaction(3, "stranger");
+    ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 3 not owned by stranger");
+  }
+}
+
+TEST_F(TxnStatusTableITest, GetTransactionStatus) {
+  const auto verify_state = [&](TxnStatePB state) {
+    TxnStatusEntryPB txn_status;
+    ASSERT_OK(txn_sys_client_->GetTransactionStatus(1, kUser, &txn_status));
+    ASSERT_TRUE(txn_status.has_user());
+    ASSERT_STREQ(kUser, txn_status.user().c_str());
+    ASSERT_TRUE(txn_status.has_state());
+    ASSERT_EQ(state, txn_status.state());
+  };
+
+  ASSERT_OK(txn_sys_client_->CreateTxnStatusTable(100));
+  ASSERT_OK(txn_sys_client_->OpenTxnStatusTable());
+
+  ASSERT_OK(txn_sys_client_->BeginTransaction(1, kUser));
+  NO_FATALS(verify_state(TxnStatePB::OPEN));
+
+  ASSERT_OK(txn_sys_client_->BeginCommitTransaction(1, kUser));
+  NO_FATALS(verify_state(TxnStatePB::COMMIT_IN_PROGRESS));
+
+  ASSERT_OK(txn_sys_client_->AbortTransaction(1, kUser));
+  NO_FATALS(verify_state(TxnStatePB::ABORTED));
+
+  {
+    auto s = txn_sys_client_->BeginCommitTransaction(1, kUser);
+    ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+    NO_FATALS(verify_state(TxnStatePB::ABORTED));
+  }
+
+  // In the negative scenarios below, check for the expected status code
+  // and make sure nothing is set in the output argument.
+  {
+    TxnStatusEntryPB empty;
+    auto s = txn_sys_client_->GetTransactionStatus(1, "interloper", &empty);
+    ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
+    ASSERT_FALSE(empty.has_user());
+    ASSERT_FALSE(empty.has_state());
+  }
+
+  {
+    TxnStatusEntryPB empty;
+    auto s = txn_sys_client_->GetTransactionStatus(2, kUser, &empty);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_FALSE(empty.has_user());
+    ASSERT_FALSE(empty.has_state());
+  }
+
+  {
+    TxnStatusEntryPB empty;
+    auto s = txn_sys_client_->GetTransactionStatus(2, "", &empty);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_FALSE(empty.has_user());
+    ASSERT_FALSE(empty.has_state());
+  }
+}
+
 // Test that a transaction system client can make concurrent calls to multiple
 // transaction status tablets.
 TEST_F(TxnStatusTableITest, TestSystemClientConcurrentCalls) {
@@ -456,11 +626,7 @@ class MultiServerTxnStatusTableITest : public TxnStatusTableITest {
     opts.num_tablet_servers = 4;
     cluster_.reset(new InternalMiniCluster(env_, std::move(opts)));
     ASSERT_OK(cluster_->Start());
-    vector<string> master_addrs;
-    for (const auto& hp : cluster_->master_rpc_addrs()) {
-      master_addrs.emplace_back(hp.ToString());
-    }
-    ASSERT_OK(TxnSystemClient::Create(master_addrs, &txn_sys_client_));
+    ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_sys_client_));
 
     // Create the initial transaction status table partitions and start an
     // initial transaction.
@@ -528,8 +694,8 @@ TEST_F(MultiServerTxnStatusTableITest, TestSystemClientLeadershipChange) {
     string new_leader_uuid;
     ASSERT_OK(FindLeaderId(tablet_id, &new_leader_uuid));
     ASSERT_NE(new_leader_uuid, orig_leader_uuid);
+    ASSERT_OK(txn_sys_client_->BeginTransaction(2, kUser));
   });
-  ASSERT_OK(txn_sys_client_->BeginTransaction(2, kUser));
 }
 
 TEST_F(MultiServerTxnStatusTableITest, TestSystemClientCrashedNodes) {
