@@ -37,6 +37,7 @@
 #include <sparsehash/dense_hash_map>
 
 #include "kudu/consensus/metadata.pb.h"
+#include "kudu/consensus/raft_consensus.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
@@ -48,7 +49,6 @@
 #include "kudu/util/cow_object.h"
 #include "kudu/util/locks.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/net/net_util.h"
 #include "kudu/util/oid_generator.h"
 #include "kudu/util/random.h"
 #include "kudu/util/rw_mutex.h"
@@ -65,6 +65,7 @@ namespace kudu {
 
 class AuthzTokenTest_TestSingleMasterUnavailable_Test;
 class CreateTableStressTest_TestConcurrentCreateTableAndReloadMetadata_Test;
+class HostPort;
 class MetricEntity;
 class MetricRegistry;
 class MonitoredTask;
@@ -82,7 +83,6 @@ class ServiceUnavailableRetryClientTest_CreateTable_Test;
 } // namespace client
 
 namespace consensus {
-class RaftConsensus;
 class StartTabletCopyRequestPB;
 } // namespace consensus
 
@@ -715,6 +715,8 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
                              TabletReportUpdatesPB* full_report_update,
                              rpc::RpcContext* rpc);
 
+  std::string GetClusterId() const;
+
   // Returns the latest handled Hive Metastore notification log event ID.
   int64_t GetLatestNotificationLogEventId();
 
@@ -773,10 +775,16 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
       const consensus::StartTabletCopyRequestPB* req,
       std::function<void(const Status&, tserver::TabletServerErrorPB::Code)> cb) override;
 
-  // Returns this CatalogManager's role in a consensus configuration. CatalogManager
-  // must be initialized before calling this method.
+  // Returns this CatalogManager's role in a consensus configuration.
+  // CatalogManager is expected to be initialized before calling this method otherwise
+  // UNKNOWN_ROLE is returned.
   consensus::RaftPeerPB::Role Role() const;
 
+  // Returns this CatalogManager's role and member type in a consensus configuration.
+  // CatalogManager is expected to be initialized before calling this method otherwise
+  // <UNKNOWN_ROLE, UNKNOWN_MEMBER_TYPE> is returned.
+  consensus::RaftConsensus::RoleAndMemberType GetRoleAndMemberType() const;
+
   hms::HmsCatalog* hms_catalog() const {
     return hms_catalog_.get();
   }
@@ -799,9 +807,15 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // name is returned.
   static std::string NormalizeTableName(const std::string& table_name);
 
-  const std::vector<HostPort>& master_addresses() const {
-    return master_addresses_;
-  }
+  enum ChangeConfigOp {
+    kAddMaster
+  };
+
+  // Add/remove a master specified by 'hp' and 'uuid' by initiating change config request.
+  // RpContext 'rpc' will be used to respond back to the client asynchronously.
+  // Returns the status of submission of the request.
+  Status InitiateMasterChangeConfig(ChangeConfigOp op, const HostPort& hp,
+                                    const std::string& uuid, rpc::RpcContext* rpc);
 
  private:
   // These tests call ElectedAsLeaderCb() directly.
@@ -872,6 +886,9 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // Currently, it's about having a means to authenticate clients by authn tokens.
   Status PrepareFollower(MonoTime* last_tspk_run);
 
+  // Load the cluster ID for the follower catalog manager.
+  Status PrepareFollowerClusterId();
+
   // Prepare CA-related information for the follower catalog manager. Currently,
   // this includes reading the CA information from the system table, creating
   // TLS server certificate request, signing it with the CA key, and installing
@@ -897,6 +914,11 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // This method is thread-safe.
   Status InitSysCatalogAsync(bool is_first_run);
 
+  // Initialize the cluster ID: load the cluster ID record
+  // from the system table. If a cluster ID record is not present in the
+  // table, generate and store a new one.
+  Status InitClusterId();
+
   // Initialize the IPKI certificate authority: load the CA information record
   // from the system table. If the CA information record is not present in the
   // table, generate and store a new one.
@@ -907,12 +929,19 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   Status InitCertAuthorityWith(std::unique_ptr<security::PrivateKey> key,
                                std::unique_ptr<security::Cert> cert);
 
+  // Load the cluster ID from the system table.
+  // If the cluster ID is not found in the table, return Status::NotFound.
+  Status LoadClusterId(std::string* cluster_id);
+
   // Load the IPKI certficate authority information from the system
   // table: the private key and the certificate. If the CA info entry is not
   // found in the table, return Status::NotFound.
   Status LoadCertAuthorityInfo(std::unique_ptr<security::PrivateKey>* key,
                                std::unique_ptr<security::Cert>* cert);
 
+  // Store cluster ID into the system table.
+  Status StoreClusterId(const std::string& cluster_id);
+
   // Store the IPKI certificate authority information into the system table.
   Status StoreCertAuthorityInfo(const security::PrivateKey& key,
                                 const security::Cert& cert);
@@ -1151,6 +1180,8 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   mutable simple_spinlock state_lock_;
   State state_;
 
+  static const char* ChangeConfigOpToString(ChangeConfigOp type);
+
   // Singleton pool that serializes invocations of ElectedAsLeaderCb().
   std::unique_ptr<ThreadPool> leader_election_pool_;
 
@@ -1174,17 +1205,12 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // reading. Following an election where this master is elected leader, it
   // should acquire this lock for writing before reloading the metadata.
   //
-  // Readers should not acquire this lock directly; use ScopedLeadershipLock
+  // Readers should not acquire this lock directly; use ScopedLeaderSharedLock
   // instead.
   //
   // Always acquire this lock before state_lock_.
   RWMutex leader_lock_;
 
-  // Cached information on master addresses. It's populated in Init() since the
-  // membership of masters' Raft consensus is static (i.e. no new members are
-  // added or any existing removed).
-  std::vector<HostPort> master_addresses_;
-
   // Async operations are accessing some private methods
   // (TODO: this stuff should be deferred and done in the background thread)
   friend class AsyncAlterTable;
@@ -1192,6 +1218,10 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // LRU cache to store responses generated by GetTableLocations().
   std::unique_ptr<TableLocationsCache> table_locations_cache_;
 
+  // Lock protecting cluster_id_.
+  mutable simple_spinlock cluster_id_lock_;
+  std::string cluster_id_;
+
   DISALLOW_COPY_AND_ASSIGN(CatalogManager);
 };
 
