@@ -57,6 +57,7 @@
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/strings/util.h"
 #include "kudu/gutil/walltime.h"
+#include "kudu/integration-tests/cluster_itest_util.h"
 #include "kudu/master/catalog_manager.h"
 #include "kudu/master/master.pb.h"
 #include "kudu/master/master.proxy.h"
@@ -96,6 +97,7 @@
 using boost::none;
 using boost::optional;
 using kudu::consensus::ReplicaManagementInfoPB;
+using kudu::itest::GetClusterId;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
 using kudu::rpc::Messenger;
@@ -115,6 +117,7 @@ using std::vector;
 using strings::Substitute;
 
 DECLARE_bool(catalog_manager_check_ts_count_for_create_table);
+DECLARE_bool(enable_per_range_hash_schemas);
 DECLARE_bool(master_support_authz_tokens);
 DECLARE_bool(mock_table_metrics_for_testing);
 DECLARE_bool(raft_prepare_replacement_before_eviction);
@@ -160,15 +163,24 @@ class MasterTest : public KuduTest {
     KuduTest::TearDown();
   }
 
+  struct HashBucketSchema {
+    vector<string> columns;
+    int32_t num_buckets;
+    uint32_t seed;
+  };
+
   void DoListTables(const ListTablesRequestPB& req, ListTablesResponsePB* resp);
   void DoListAllTables(ListTablesResponsePB* resp);
   Status CreateTable(const string& table_name,
-                     const Schema& schema);
+                     const Schema& schema,
+                     const optional<TableTypePB>& table_type = boost::none);
   Status CreateTable(const string& table_name,
                      const Schema& schema,
                      const vector<KuduPartialRow>& split_rows,
                      const vector<pair<KuduPartialRow, KuduPartialRow>>& bounds,
-                     const optional<string>& owner);
+                     const optional<string>& owner,
+                     const optional<TableTypePB>& table_type = boost::none,
+                     const vector<vector<HashBucketSchema>>& range_hash_schema = {});
 
   shared_ptr<Messenger> client_messenger_;
   unique_ptr<MiniMaster> mini_master_;
@@ -525,27 +537,33 @@ TEST_F(MasterTest, TestRegisterAndHeartbeat) {
 }
 
 Status MasterTest::CreateTable(const string& table_name,
-                               const Schema& schema) {
+                               const Schema& schema,
+                               const optional<TableTypePB>& table_type) {
   KuduPartialRow split1(&schema);
   RETURN_NOT_OK(split1.SetInt32("key", 10));
 
   KuduPartialRow split2(&schema);
   RETURN_NOT_OK(split2.SetInt32("key", 20));
 
-  return CreateTable(table_name, schema, { split1, split2 }, {}, boost::none);
+  return CreateTable(
+      table_name, schema, { split1, split2 }, {}, boost::none, table_type);
 }
 
 Status MasterTest::CreateTable(const string& table_name,
                                const Schema& schema,
                                const vector<KuduPartialRow>& split_rows,
                                const vector<pair<KuduPartialRow, KuduPartialRow>>& bounds,
-                               const optional<string>& owner) {
-
+                               const optional<string>& owner,
+                               const optional<TableTypePB>& table_type,
+                               const vector<vector<HashBucketSchema>>& range_hash_schema) {
   CreateTableRequestPB req;
   CreateTableResponsePB resp;
   RpcController controller;
 
   req.set_name(table_name);
+  if (table_type) {
+    req.set_table_type(*table_type);
+  }
   RETURN_NOT_OK(SchemaToPB(schema, req.mutable_schema()));
   RowOperationsPBEncoder encoder(req.mutable_split_rows_range_bounds());
   for (const KuduPartialRow& row : split_rows) {
@@ -556,6 +574,18 @@ Status MasterTest::CreateTable(const string& table_name,
     encoder.Add(RowOperationsPB::RANGE_UPPER_BOUND, bound.second);
   }
 
+  for (const auto& hash_schemas : range_hash_schema) {
+    auto* hash_schemas_pb = req.add_range_hash_schemas();
+    for (const auto& hash_schema : hash_schemas) {
+      auto* hash_bucket_schema_pb = hash_schemas_pb->add_hash_schemas();
+      for (const string& col_name : hash_schema.columns) {
+        hash_bucket_schema_pb->add_columns()->set_name(col_name);
+      }
+      hash_bucket_schema_pb->set_num_buckets(hash_schema.num_buckets);
+      hash_bucket_schema_pb->set_seed(hash_schema.seed);
+    }
+  }
+
   if (owner) {
     req.set_owner(*owner);
   }
@@ -666,6 +696,109 @@ TEST_F(MasterTest, TestCatalog) {
   }
 }
 
+TEST_F(MasterTest, ListTablesWithTableFilter) {
+  static const char* const kUserTableName = "user_table";
+  static const char* const kSystemTableName = "system_table";
+  const Schema kSchema({ColumnSchema("key", INT32), ColumnSchema("v1", INT8)}, 1);
+
+  // Make sure this test scenario stays valid
+  ASSERT_EQ(TableTypePB_MAX, TableTypePB::TXN_STATUS_TABLE);
+
+  // Given there is not any tablet server in the cluster, there should be no
+  // tables out there yet, so a request with an empty (default) filter should
+  // return empty list of tables.
+  {
+    ListTablesRequestPB req;
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(0, resp.tables_size());
+  }
+
+  // The same reasoning as above, but supply filter with all known table types
+  // in the 'table_type' field.
+  {
+    ListTablesRequestPB req;
+    req.add_type_filter(TableTypePB::DEFAULT_TABLE);
+    req.add_type_filter(TableTypePB::TXN_STATUS_TABLE);
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(0, resp.tables_size());
+  }
+
+  ASSERT_OK(CreateTable(kUserTableName, kSchema));
+
+  // An empty 'filter_type' field is equivalent to setting the 'filter_type'
+  // to [TableTypePB::DEFAULT_TABLE].
+  {
+    ListTablesRequestPB req;
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(1, resp.tables_size());
+    ASSERT_EQ(kUserTableName, resp.tables(0).name());
+  }
+
+  // Specify the table type filter explicitly.
+  {
+    ListTablesRequestPB req;
+    req.add_type_filter(TableTypePB::DEFAULT_TABLE);
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(1, resp.tables_size());
+    ASSERT_EQ(kUserTableName, resp.tables(0).name());
+  }
+
+  // No system tables are present at this point.
+  {
+    ListTablesRequestPB req;
+    req.add_type_filter(TableTypePB::TXN_STATUS_TABLE);
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(0, resp.tables_size());
+  }
+
+  ASSERT_OK(CreateTable(
+      kSystemTableName, kSchema, TableTypePB::TXN_STATUS_TABLE));
+
+  // Only user tables should be listed because the explicitly specified
+  // 'type_filter' is set to [TableTypePB::DEFAULT_TABLE].
+  {
+    ListTablesRequestPB req;
+    req.add_type_filter(TableTypePB::DEFAULT_TABLE);
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(1, resp.tables_size());
+    ASSERT_EQ(kUserTableName, resp.tables(0).name());
+  }
+
+  // Only the txn status system table should be listed because the explicitly
+  // specified 'type_filter' is set to [TableTypePB::TXN_STATUS_TABLE].
+  {
+    ListTablesRequestPB req;
+    req.add_type_filter(TableTypePB::TXN_STATUS_TABLE);
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(1, resp.tables_size());
+    ASSERT_EQ(kSystemTableName, resp.tables(0).name());
+  }
+
+  // Both tables should be output when the filter is set to
+  // [TableTypePB::DEFAULT_TABLE, TableTypePB::TXN_STATUS_TABLE].
+  {
+    ListTablesRequestPB req;
+    req.add_type_filter(TableTypePB::DEFAULT_TABLE);
+    req.add_type_filter(TableTypePB::TXN_STATUS_TABLE);
+    ListTablesResponsePB resp;
+    NO_FATALS(DoListTables(req, &resp));
+    ASSERT_EQ(2, resp.tables_size());
+    unordered_set<string> table_names;
+    table_names.emplace(resp.tables(0).name());
+    table_names.emplace(resp.tables(1).name());
+    ASSERT_EQ(2, table_names.size());
+    ASSERT_TRUE(ContainsKey(table_names, kUserTableName));
+    ASSERT_TRUE(ContainsKey(table_names, kSystemTableName));
+  }
+}
+
 TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
   const char *kTableName = "testtb";
   const Schema kTableSchema({ ColumnSchema("key", INT32), ColumnSchema("val", INT32) }, 1);
@@ -693,6 +826,43 @@ TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
                         "least one range partition column");
   }
 
+  // No split rows and range specific hashing concurrently.
+  {
+    FLAGS_enable_per_range_hash_schemas = true; // enable for testing.
+    KuduPartialRow split1(&kTableSchema);
+    ASSERT_OK(split1.SetInt32("key", 1));
+    KuduPartialRow a_lower(&kTableSchema);
+    KuduPartialRow a_upper(&kTableSchema);
+    ASSERT_OK(a_lower.SetInt32("key", 0));
+    ASSERT_OK(a_upper.SetInt32("key", 100));
+    vector<vector<HashBucketSchema>> range_hash_schema = { vector<HashBucketSchema>() };
+    Status s = CreateTable(kTableName, kTableSchema, { split1 }, { { a_lower, a_upper } },
+                           none, none, range_hash_schema);
+    ASSERT_TRUE(s.IsInvalidArgument());
+    ASSERT_STR_CONTAINS(s.ToString(),
+                        "Both 'split_rows' and 'range_hash_schemas' cannot be "
+                        "populated at the same time.");
+  }
+
+  // The number of range bounds must match the size of user defined hash schemas.
+  {
+    FLAGS_enable_per_range_hash_schemas = true; // enable for testing.
+    KuduPartialRow a_lower(&kTableSchema);
+    KuduPartialRow a_upper(&kTableSchema);
+    ASSERT_OK(a_lower.SetInt32("key", 0));
+    ASSERT_OK(a_upper.SetInt32("key", 100));
+    vector<HashBucketSchema> hash_schemas_4 = { { {"key"}, 4, 0 } };
+    vector<HashBucketSchema> hash_schemas_2 = { { {"val"}, 2, 0 } };
+    vector<vector<HashBucketSchema>> range_hash_schema = {std::move(hash_schemas_2),
+                                                          std::move(hash_schemas_4)};
+    Status s = CreateTable(kTableName, kTableSchema, { }, { { a_lower, a_upper } },
+                           none, none, range_hash_schema);
+    ASSERT_TRUE(s.IsInvalidArgument());
+    ASSERT_STR_CONTAINS(s.ToString(),
+                        "The number of range bounds does not match the number of per "
+                        "range hash schemas.");
+  }
+
   // No non-range columns.
   {
     KuduPartialRow split(&kTableSchema);
@@ -715,8 +885,7 @@ TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
     ASSERT_OK(b_lower.SetInt32("key", 50));
     ASSERT_OK(b_upper.SetInt32("key", 150));
     Status s = CreateTable(kTableName, kTableSchema, { }, { { a_lower, a_upper },
-                                                            { b_lower, b_upper } },
-                                                            none);
+                                                            { b_lower, b_upper } }, none);
     ASSERT_TRUE(s.IsInvalidArgument());
     ASSERT_STR_CONTAINS(s.ToString(), "Invalid argument: overlapping range partition");
   }
@@ -730,8 +899,7 @@ TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
     ASSERT_OK(split.SetInt32("key", 200));
 
     Status s = CreateTable(kTableName, kTableSchema, { split },
-                           { { bound_lower, bound_upper } },
-                           none);
+                           { { bound_lower, bound_upper } }, none);
     ASSERT_TRUE(s.IsInvalidArgument());
     ASSERT_STR_CONTAINS(s.ToString(), "Invalid argument: split out of bounds");
   }
@@ -745,8 +913,7 @@ TEST_F(MasterTest, TestCreateTableCheckRangeInvariants) {
     ASSERT_OK(split.SetInt32("key", -120));
 
     Status s = CreateTable(kTableName, kTableSchema, { split },
-                           { { bound_lower, bound_upper } },
-                           none);
+                           { { bound_lower, bound_upper } }, none);
     ASSERT_TRUE(s.IsInvalidArgument());
     ASSERT_STR_CONTAINS(s.ToString(), "Invalid argument: split out of bounds");
   }
@@ -823,7 +990,7 @@ TEST_F(MasterTest, TestCreateTableOwnerNameTooLong) {
     "xyz01234567899abcdefghijklmnopqrstuvwxyz01234567899abcdefghijklmnopqrstuvwxyz01234567899";
 
   const Schema kTableSchema({ ColumnSchema("key", INT32), ColumnSchema("val", INT32) }, 1);
-  Status s = CreateTable(kTableName, kTableSchema, { }, { }, kOwnerName);
+  Status s = CreateTable(kTableName, kTableSchema, {}, {}, kOwnerName);
   ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "invalid owner name");
 }
@@ -1214,8 +1381,10 @@ TEST_P(ConcurrentGetTableSchemaTest, DirectMethodCall) {
   CatalogManager* cm = mini_master_->master()->catalog_manager();
   const auto* token_signer = supports_authz_
       ? mini_master_->master()->token_signer() : nullptr;
-  const auto username = supports_authz_
-      ? boost::make_optional<const string&>(kUserName) : boost::none;
+  optional<const string&> username;
+  if (supports_authz_) {
+    username = kUserName;
+  }
 
   // Start many threads that hammer the master with GetTableSchema() calls
   // for various tables.
@@ -1243,14 +1412,12 @@ TEST_P(ConcurrentGetTableSchemaTest, DirectMethodCall) {
       }
     });
   }
-  SCOPED_CLEANUP({
-    for (auto& t : caller_threads) {
-      t.join();
-    }
-  });
 
   SleepFor(kRunInterval);
   done.Store(true);
+  for (auto& t : caller_threads) {
+    t.join();
+  }
 
   const auto errors = accumulate(error_counters.begin(), error_counters.end(), 0UL);
   if (errors != 0) {
@@ -1853,8 +2020,11 @@ TEST_F(MasterTest, TestConnectToMaster) {
   ASSERT_EQ(1, resp.ca_cert_der_size()) << "should have one cert";
   EXPECT_GT(resp.ca_cert_der(0).size(), 100) << "CA cert should be at least 100 bytes";
   ASSERT_TRUE(resp.has_authn_token()) << "should return an authn token";
-  // Using 512 bit RSA key and SHA256 digest results in 64 byte signature.
-  EXPECT_EQ(64, resp.authn_token().signature().size());
+  // Using 512 bit RSA key and SHA256 digest results in 64 byte signature. If
+  // large keys are used, we use 2048 bit RSA keys so the signature is 256
+  // bytes.
+  int signature_size = UseLargeKeys() ? 256 : 64;
+  EXPECT_EQ(signature_size, resp.authn_token().signature().size());
   ASSERT_TRUE(resp.authn_token().has_signing_key_seq_num());
   EXPECT_GT(resp.authn_token().signing_key_seq_num(), -1);
 
@@ -1869,6 +2039,13 @@ TEST_F(MasterTest, TestConnectToMaster) {
   // The returned location should be empty because no location mapping command
   // is defined.
   ASSERT_TRUE(resp.client_location().empty());
+
+  // The cluster ID should match the masters cluster ID.
+  string cluster_id;
+  const std::shared_ptr<MasterServiceProxy> master_proxy = std::move(proxy_);
+  ASSERT_OK(GetClusterId(master_proxy, MonoDelta::FromSeconds(30), &cluster_id));
+  ASSERT_TRUE(!cluster_id.empty());
+  ASSERT_EQ(cluster_id, resp.cluster_id());
 }
 
 TEST_F(MasterTest, TestConnectToMasterAndAssignLocation) {
