@@ -55,6 +55,7 @@
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/net/net_util.h"
 #include "kudu/util/net/sockaddr.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/scoped_cleanup.h"
@@ -95,7 +96,17 @@ DEFINE_bool(master_support_authz_tokens, true,
             "testing version compatibility in the client.");
 TAG_FLAG(master_support_authz_tokens, hidden);
 
-using boost::make_optional;
+DEFINE_bool(master_support_change_config, false,
+            "Whether the master supports adding/removing master servers dynamically.");
+TAG_FLAG(master_support_change_config, hidden);
+TAG_FLAG(master_support_change_config, unsafe);
+
+DEFINE_bool(master_support_ignore_operations, true,
+            "Whether the cluster supports support ignore operations.");
+TAG_FLAG(master_support_ignore_operations, hidden);
+TAG_FLAG(master_support_ignore_operations, runtime);
+
+
 using google::protobuf::Message;
 using kudu::consensus::ReplicaManagementInfoPB;
 using kudu::pb_util::SecureDebugString;
@@ -230,6 +241,36 @@ void MasterServiceImpl::ChangeTServerState(const ChangeTServerStateRequestPB* re
   rpc->RespondSuccess();
 }
 
+void MasterServiceImpl::AddMaster(const AddMasterRequestPB* req,
+                                  AddMasterResponsePB* resp,
+                                  rpc::RpcContext* rpc) {
+  if (!FLAGS_master_support_change_config) {
+    rpc->RespondFailure(Status::NotSupported("Adding master is not supported"));
+    return;
+  }
+
+  if (!req->has_rpc_addr()) {
+    rpc->RespondFailure(Status::InvalidArgument("RPC address of master to be added not supplied"));
+    return;
+  }
+
+  CatalogManager::ScopedLeaderSharedLock l(server_->catalog_manager());
+  if (!l.CheckIsInitializedAndIsLeaderOrRespond(resp, rpc)) {
+    return;
+  }
+
+  Status s = server_->AddMaster(HostPortFromPB(req->rpc_addr()), rpc);
+  if (!s.ok()) {
+    LOG(ERROR) << Substitute("Failed adding master $0:$1. $2", req->rpc_addr().host(),
+                             req->rpc_addr().port(), s.ToString());
+    rpc->RespondFailure(s);
+    return;
+  }
+  // ChangeConfig request successfully submitted. Once the ChangeConfig request is complete
+  // the completion callback will respond back with the result to the RPC client.
+  // See completion_cb in CatalogManager::InitiateMasterChangeConfig().
+}
+
 void MasterServiceImpl::TSHeartbeat(const TSHeartbeatRequestPB* req,
                                     TSHeartbeatResponsePB* resp,
                                     rpc::RpcContext* rpc) {
@@ -395,7 +436,7 @@ void MasterServiceImpl::GetTabletLocations(const GetTabletLocationsRequestPB* re
         tablet_id, req->replica_type_filter(),
         locs_pb,
         req->intern_ts_infos_in_response() ? &infos_dict : nullptr,
-        make_optional<const string&>(rpc->remote_user().username()));
+        rpc->remote_user().username());
     if (!s.ok()) {
       resp->mutable_tablet_locations()->RemoveLast();
 
@@ -433,8 +474,8 @@ void MasterServiceImpl::IsCreateTableDone(const IsCreateTableDoneRequestPB* req,
     return;
   }
 
-  Status s = server_->catalog_manager()->IsCreateTableDone(
-      req, resp, make_optional<const string&>(rpc->remote_user().username()));
+  auto s = server_->catalog_manager()->IsCreateTableDone(
+      req, resp, rpc->remote_user().username());
   CheckRespErrorOrSetUnknown(s, resp);
   rpc->RespondSuccess();
 }
@@ -473,8 +514,8 @@ void MasterServiceImpl::IsAlterTableDone(const IsAlterTableDoneRequestPB* req,
     return;
   }
 
-  Status s = server_->catalog_manager()->IsAlterTableDone(
-      req, resp, make_optional<const string&>(rpc->remote_user().username()));
+  auto s = server_->catalog_manager()->IsAlterTableDone(
+      req, resp, rpc->remote_user().username());
   CheckRespErrorOrSetUnknown(s, resp);
   rpc->RespondSuccess();
 }
@@ -487,8 +528,8 @@ void MasterServiceImpl::ListTables(const ListTablesRequestPB* req,
     return;
   }
 
-  Status s = server_->catalog_manager()->ListTables(
-      req, resp, make_optional<const string&>(rpc->remote_user().username()));
+  auto s = server_->catalog_manager()->ListTables(
+      req, resp, rpc->remote_user().username());
   CheckRespErrorOrSetUnknown(s, resp);
   rpc->RespondSuccess();
 }
@@ -501,7 +542,7 @@ void MasterServiceImpl::GetTableStatistics(const GetTableStatisticsRequestPB* re
     return;
   }
   Status s = server_->catalog_manager()->GetTableStatistics(
-      req, resp, make_optional<const string&>(rpc->remote_user().username()));
+      req, resp, rpc->remote_user().username());
   CheckRespErrorOrSetUnknown(s, resp);
   rpc->RespondSuccess();
 }
@@ -523,7 +564,7 @@ void MasterServiceImpl::GetTableLocations(const GetTableLocationsRequestPB* req,
       SleepFor(MonoDelta::FromMilliseconds(FLAGS_master_inject_latency_on_tablet_lookups_ms));
     }
     s = server_->catalog_manager()->GetTableLocations(
-        req, resp, make_optional<const string&>(rpc->remote_user().username()));
+        req, resp, rpc->remote_user().username());
   }
 
   CheckRespErrorOrSetUnknown(s, resp);
@@ -541,16 +582,11 @@ void MasterServiceImpl::GetTableSchema(const GetTableSchemaRequestPB* req,
     }
 
     s = server_->catalog_manager()->GetTableSchema(
-        req, resp, make_optional<const string&>(rpc->remote_user().username()),
+        req, resp, rpc->remote_user().username(),
         FLAGS_master_support_authz_tokens ? server_->token_signer() : nullptr);
   }
 
   CheckRespErrorOrSetUnknown(s, resp);
-  if (resp->has_error()) {
-    // If there was an application error, respond to the RPC.
-    rpc->RespondSuccess();
-    return;
-  }
   rpc->RespondSuccess();
 }
 
@@ -624,7 +660,10 @@ void MasterServiceImpl::GetMasterRegistration(const GetMasterRegistrationRequest
 
   Status s = server_->GetMasterRegistration(resp->mutable_registration());
   CheckRespErrorOrSetUnknown(s, resp);
-  resp->set_role(server_->catalog_manager()->Role());
+  const auto& role_and_member = server_->catalog_manager()->GetRoleAndMemberType();
+  resp->set_role(role_and_member.first);
+  resp->set_member_type(role_and_member.second);
+  resp->set_cluster_id(server_->catalog_manager()->GetClusterId());
   rpc->RespondSuccess();
 }
 
@@ -638,7 +677,14 @@ void MasterServiceImpl::ConnectToMaster(const ConnectToMasterRequestPB* /*req*/,
 
   // Set the info about the other masters, so that the client can verify
   // it has the full set of info.
-  const auto& addresses = server_->catalog_manager()->master_addresses();
+  vector<HostPort> addresses;
+  Status s = server_->GetMasterHostPorts(&addresses);
+  if (!s.ok()) {
+    StatusToPB(s, resp->mutable_error()->mutable_status());
+    resp->mutable_error()->set_code(MasterErrorPB::UNKNOWN_ERROR);
+    rpc->RespondSuccess();
+    return;
+  }
   resp->mutable_master_addrs()->Reserve(addresses.size());
   for (const auto& hp : addresses) {
     *resp->add_master_addrs() = HostPortToPB(hp);
@@ -711,6 +757,10 @@ void MasterServiceImpl::ConnectToMaster(const ConnectToMasterRequestPB* /*req*/,
   // until we have taken over all the associated responsibilities.
   resp->set_role(is_leader ? consensus::RaftPeerPB::LEADER
                            : consensus::RaftPeerPB::FOLLOWER);
+
+  // Add the cluster ID.
+  resp->set_cluster_id(server_->catalog_manager()->GetClusterId());
+
   rpc->RespondSuccess();
 }
 
@@ -751,6 +801,10 @@ bool MasterServiceImpl::SupportsFeature(uint32_t feature) const {
       return FLAGS_master_support_authz_tokens;
     case MasterFeatures::CONNECT_TO_MASTER:
       return FLAGS_master_support_connect_to_master_rpc;
+    case MasterFeatures::DYNAMIC_MULTI_MASTER:
+      return FLAGS_master_support_change_config;
+    case MasterFeatures::IGNORE_OPERATIONS:
+      return FLAGS_master_support_ignore_operations;
     default:
       return false;
   }
