@@ -58,6 +58,7 @@
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tserver/tserver.pb.h"
+#include "kudu/tserver/tserver_admin.proxy.h"
 #include "kudu/tserver/tserver_service.proxy.h"
 #include "kudu/util/async_util.h"
 #include "kudu/util/env.h"
@@ -86,6 +87,7 @@ using kudu::rpc::RpcController;
 using kudu::server::ServerStatusPB;
 using kudu::tserver::ListTabletsRequestPB;
 using kudu::tserver::ListTabletsResponsePB;
+using kudu::tserver::TabletServerAdminServiceProxy;
 using kudu::tserver::TabletServerServiceProxy;
 using std::copy;
 using std::pair;
@@ -112,6 +114,7 @@ static double kMasterCatalogManagerTimeoutSeconds = 60.0;
 
 ExternalMiniClusterOptions::ExternalMiniClusterOptions()
     : num_masters(1),
+      supply_single_master_addr(false),
       num_tablet_servers(1),
       bind_mode(kDefaultBindMode),
       num_data_dirs(1),
@@ -495,7 +498,10 @@ Status ExternalMiniCluster::StartMasters() {
 
   vector<string> flags;
   flags.emplace_back("--rpc_reuseport=true");
-  if (num_masters > 1) {
+  // Setting --master_addresses flag for a single master configuration is now supported but not
+  // mandatory. Not setting the flag helps test existing kudu deployments that don't specify
+  // the --master_addresses flag for single master configuration.
+  if (num_masters > 1 || opts_.supply_single_master_addr) {
     flags.emplace_back(Substitute("--master_addresses=$0",
                                   HostPort::ToCommaSeparatedString(master_rpc_addrs)));
   }
@@ -716,9 +722,11 @@ void ExternalMiniCluster::AssertNoCrashes() {
   ASSERT_EQ(0, num_crashes) << "At least one process crashed";
 }
 
-Status ExternalMiniCluster::WaitForTabletsRunning(ExternalTabletServer* ts,
-                                                  int min_tablet_count,
-                                                  const MonoDelta& timeout) {
+Status ExternalMiniCluster::WaitForTabletsRunning(
+    ExternalTabletServer* ts,
+    int min_tablet_count,
+    const MonoDelta& timeout,
+    vector<TabletIdAndTableName>* tablets_info) {
   TabletServerServiceProxy proxy(messenger_, ts->bound_rpc_addr(), ts->bound_rpc_addr().host());
   ListTabletsRequestPB req;
   ListTabletsResponsePB resp;
@@ -743,6 +751,17 @@ Status ExternalMiniCluster::WaitForTabletsRunning(ExternalTabletServer* ts,
     // 1. All the tablets are running, and
     // 2. We've observed as many tablets as we had expected or more.
     if (all_running && resp.status_and_schema_size() >= min_tablet_count) {
+      if (tablets_info) {
+        tablets_info->clear();
+        const auto num_elems = resp.status_and_schema_size();
+        tablets_info->reserve(num_elems);
+        for (auto i = 0; i < num_elems; ++i) {
+          const auto& elem = resp.status_and_schema(i);
+          tablets_info->emplace_back(
+              TabletIdAndTableName{elem.tablet_status().tablet_id(),
+                                   elem.tablet_status().table_name()});
+        }
+      }
       return Status::OK();
     }
 
@@ -875,6 +894,13 @@ std::shared_ptr<TabletServerServiceProxy> ExternalMiniCluster::tserver_proxy(int
   return std::make_shared<TabletServerServiceProxy>(messenger_, addr, addr.host());
 }
 
+std::shared_ptr<TabletServerAdminServiceProxy> ExternalMiniCluster::tserver_admin_proxy(
+    int idx) const {
+  CHECK_LT(idx, tablet_servers_.size());
+  const auto& addr = CHECK_NOTNULL(tablet_server(idx))->bound_rpc_addr();
+  return std::make_shared<TabletServerAdminServiceProxy>(messenger_, addr, addr.host());
+}
+
 Status ExternalMiniCluster::CreateClient(client::KuduClientBuilder* builder,
                                          client::sp::shared_ptr<client::KuduClient>* client) const {
   client::KuduClientBuilder defaults;
@@ -920,6 +946,20 @@ string ExternalMiniCluster::UuidForTS(int ts_idx) const {
   return tablet_server(ts_idx)->uuid();
 }
 
+Status ExternalMiniCluster::AddMaster(const scoped_refptr<ExternalMaster>& new_master) {
+  const auto& new_master_uuid = new_master->instance_id().permanent_uuid();
+  for (const auto& m : masters_) {
+    if (m->instance_id().permanent_uuid() == new_master_uuid) {
+      CHECK(m->bound_rpc_hostport() == new_master->bound_rpc_hostport());
+      return Status::AlreadyPresent(Substitute(
+          "Master $0, uuid: $1 already present in ExternalMiniCluster",
+          m->bound_rpc_hostport().ToString(), new_master_uuid));
+    }
+  }
+  masters_.emplace_back(new_master);
+  return Status::OK();
+}
+
 //------------------------------------------------------------
 // ExternalDaemon
 //------------------------------------------------------------
@@ -957,20 +997,6 @@ Status ExternalDaemon::StartProcess(const vector<string>& user_flags) {
     // rely on forcefully cutting power to a machine or equivalent.
     "--never_fsync",
 
-    // Generate smaller RSA keys -- generating a 768-bit key is faster
-    // than generating the default 2048-bit key, and we don't care about
-    // strong encryption in tests. Setting it lower (e.g. 512 bits) results
-    // in OpenSSL errors RSA_sign:digest too big for rsa key:rsa_sign.c:122
-    // since we are using strong/high TLS v1.2 cipher suites, so the minimum
-    // size of TLS-related RSA key is 768 bits (due to the usage of
-    // the ECDHE-RSA-AES256-GCM-SHA384 suite).
-    "--ipki_server_key_size=768",
-
-    // The RSA key of 768 bits is too short if OpenSSL security level is set to
-    // 1 or higher (applicable for OpenSSL 1.1.0 and newer). Lowering the
-    // security level to 0 makes possible ot use shorter keys in such cases.
-    "--openssl_security_level_override=0",
-
     // Disable minidumps by default since many tests purposely inject faults.
     "--enable_minidumps=false",
 
@@ -1004,6 +1030,23 @@ Status ExternalDaemon::StartProcess(const vector<string>& user_flags) {
     argv.emplace_back("--logbuflevel=-1");
   }
 
+  // If large keys are not enabled.
+  if (!UseLargeKeys()) {
+    // Generate smaller RSA keys -- generating a 768-bit key is faster
+    // than generating the default 2048-bit key, and we don't care about
+    // strong encryption in tests. Setting it lower (e.g. 512 bits) results
+    // in OpenSSL errors RSA_sign:digest too big for rsa key:rsa_sign.c:122
+    // since we are using strong/high TLS v1.2 cipher suites, so the minimum
+    // size of TLS-related RSA key is 768 bits (due to the usage of
+    // the ECDHE-RSA-AES256-GCM-SHA384 suite).
+    argv.emplace_back("--ipki_server_key_size=768");
+
+    // The RSA key of 768 bits is too short if OpenSSL security level is set to
+    // 1 or higher (applicable for OpenSSL 1.1.0 and newer). Lowering the
+    // security level to 0 makes possible ot use shorter keys in such cases.
+    argv.emplace_back("--openssl_security_level_override=0");
+  }
+
   // Add all the flags coming from the minicluster framework.
   argv.insert(argv.end(), user_flags.begin(), user_flags.end());
 
@@ -1429,19 +1472,21 @@ Status ExternalMaster::WaitForCatalogManager(WaitMode wait_mode) {
 }
 
 const vector<string>& ExternalMaster::GetCommonFlags() {
-  static const vector<string> kFlags = {
-    // See the in-line comment for "--ipki_server_key_size" flag in
-    // ExternalDaemon::StartProcess() method.
-    "--ipki_ca_key_size=768",
+  static vector<string> kFlags;
+  if (!UseLargeKeys()) {
+    kFlags = {
+        // See the in-line comment for "--ipki_server_key_size" flag in
+        // ExternalDaemon::StartProcess() method.
+        "--ipki_ca_key_size=768",
 
-    // As for the TSK keys, 512 bits is the minimum since we are using
-    // SHA256 digest for token signing/verification.
-    "--tsk_num_rsa_bits=512",
-  };
+        // As for the TSK keys, 512 bits is the minimum since we are using
+        // SHA256 digest for token signing/verification.
+        "--tsk_num_rsa_bits=512",
+    };
+  }
   return kFlags;
 }
 
-
 //------------------------------------------------------------
 // ExternalTabletServer
 //------------------------------------------------------------
