@@ -82,6 +82,8 @@ class MessengerBuilder {
   friend class Messenger;
   friend class ReactorThread;
 
+  static const int64_t kRpcNegotiationTimeoutMs;
+
   explicit MessengerBuilder(std::string name);
 
   // Set the length of time we will keep a TCP connection will alive with no traffic.
@@ -243,7 +245,9 @@ class Messenger {
   // Returns an error if no service with this name can be found.
   Status UnregisterService(const std::string& service_name);
 
-  // Unregisters all RPC services.
+  // Unregisters all RPC services. Once called, no new services can be
+  // registered, and attempts to access missing services will result in a
+  // retriable error code.
   void UnregisterAllServices();
 
   // Queue a call for transmission. This will pick the appropriate reactor,
@@ -301,14 +305,21 @@ class Messenger {
     return name_;
   }
 
+  void SetServicesRegistered() {
+    std::lock_guard<percpu_rwlock> guard(lock_);
+    state_ = kServicesRegistered;
+  }
+
   bool closing() const {
     shared_lock<rw_spinlock> l(lock_.get_lock());
-    return closing_;
+    return state_ == kClosing;
   }
 
   scoped_refptr<MetricEntity> metric_entity() const { return metric_entity_; }
 
-  const int64_t rpc_negotiation_timeout_ms() const { return rpc_negotiation_timeout_ms_; }
+  int64_t rpc_negotiation_timeout_ms() const {
+    return rpc_negotiation_timeout_ms_;
+  }
 
   const std::string& sasl_proto_name() const {
     return sasl_proto_name_;
@@ -351,7 +362,23 @@ class Messenger {
   // Protects closing_, acceptor_pools_, rpc_services_.
   mutable percpu_rwlock lock_;
 
-  bool closing_;
+  enum State {
+    // The Messenger has been started; not all services may be registered yet.
+    kStarted,
+
+    // The Messenger is fully up running. All services have been registered and
+    // are accepting requests.
+    // NOTE: Messengers that do not register services never enter this state.
+    kServicesRegistered,
+
+    // All services have been unregistered. No further requests will succeed.
+    // NOTE: Messengers that do not register services never enter this state.
+    kServicesUnregistered,
+
+    // The Messenger is being closed. Its resources may be freed.
+    kClosing,
+  };
+  State state_;
 
   // Whether to require authentication and encryption on the connections managed
   // by this messenger.
