@@ -19,6 +19,8 @@
 
 #include <cstddef>
 #include <cstdint>
+#include <functional>
+#include <limits>
 #include <memory>
 #include <mutex>
 #include <string>
@@ -27,7 +29,6 @@
 #include <vector>
 
 #include <boost/container/vector.hpp>
-#include <boost/function.hpp>
 #include <gflags/gflags.h>
 #include <google/protobuf/message.h>
 
@@ -145,7 +146,8 @@ void OutboundCall::SetRequestPayload(const Message& req,
 
   // Compute total size of sidecar payload so that extra space can be reserved as part of
   // the request body.
-  uint32_t message_size = req.ByteSize();
+  size_t message_size = req.ByteSizeLong();
+  CHECK_LE(message_size, std::numeric_limits<uint32_t>::max());
   sidecar_byte_size_ = 0;
   for (const unique_ptr<RpcSidecar>& car: sidecars_) {
     header_.add_sidecar_offsets(sidecar_byte_size_ + message_size);
