@@ -155,26 +155,25 @@ class FileSubstitutions : public Substituter {
 class MethodSubstitutions : public Substituter {
  public:
   explicit MethodSubstitutions(const MethodDescriptor* method)
-    : method_(method) {
+      : method_(method) {
   }
 
   void InitSubstitutionMap(map<string, string>* map) const override {
-    (*map)["rpc_name"] = method_->name();
-    (*map)["rpc_full_name"] = method_->full_name();
-    (*map)["rpc_full_name_plainchars"] =
+    auto& m = *map;
+    m["rpc_name"] = method_->name();
+    m["rpc_full_name"] = method_->full_name();
+    m["rpc_full_name_plainchars"] =
         StringReplace(method_->full_name(), ".", "_", true);
-    (*map)["request"] =
-        ReplaceNamespaceDelimiters(
-            StripNamespaceIfPossible(method_->service()->full_name(),
-                                     method_->input_type()->full_name()));
-    (*map)["response"] =
-        ReplaceNamespaceDelimiters(
-            StripNamespaceIfPossible(method_->service()->full_name(),
-                                     method_->output_type()->full_name()));
-    (*map)["metric_enum_key"] = Substitute("kMetricIndex$0", method_->name());
+    m["request"] = ReplaceNamespaceDelimiters(
+        StripNamespaceIfPossible(method_->service()->full_name(),
+                                 method_->input_type()->full_name()));
+    m["response"] = ReplaceNamespaceDelimiters(
+        StripNamespaceIfPossible(method_->service()->full_name(),
+                                 method_->output_type()->full_name()));
+    m["metric_enum_key"] = Substitute("kMetricIndex$0", method_->name());
     bool track_result = static_cast<bool>(method_->options().GetExtension(track_rpc_result));
-    (*map)["track_result"] = track_result ? " true" : "false";
-    (*map)["authz_method"] = GetAuthzMethod(*method_).get_value_or("AuthorizeAllowAll");
+    m["track_result"] = track_result ? " true" : "false";
+    m["authz_method"] = GetAuthzMethod(*method_).get_value_or("AuthorizeAllowAll");
   }
 
   // Strips the package from method arguments if they are in the same package as
@@ -209,15 +208,16 @@ class MethodSubstitutions : public Substituter {
 class ServiceSubstitutions : public Substituter {
  public:
   explicit ServiceSubstitutions(const ServiceDescriptor* service)
-    : service_(service)
-  {}
+      : service_(service) {
+  }
 
   void InitSubstitutionMap(map<string, string>* map) const override {
-    (*map)["service_name"] = service_->name();
-    (*map)["full_service_name"] = service_->full_name();
-    (*map)["service_method_count"] = SimpleItoa(service_->method_count());
+    auto& m = *map;
+    m["service_name"] = service_->name();
+    m["full_service_name"] = service_->full_name();
+    m["service_method_count"] = SimpleItoa(service_->method_count());
 
-    // TODO: upgrade to protobuf 2.5.x and attach service comments
+    // TODO(todd): upgrade to protobuf 2.5.x and attach service comments
     // to the generated service classes using the SourceLocation API.
   }
 
@@ -225,7 +225,6 @@ class ServiceSubstitutions : public Substituter {
   const ServiceDescriptor* service_;
 };
 
-
 class SubstitutionContext {
  public:
   // Takes ownership of the substituter.
@@ -256,8 +255,6 @@ class SubstitutionContext {
   vector<unique_ptr<const Substituter>> subs_;
 };
 
-
-
 class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
  public:
   CodeGenerator() { }
@@ -316,31 +313,30 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
                                       SubstitutionContext* subs,
                                       const FileDescriptor* file) {
     Print(printer, *subs,
-      "// THIS FILE IS AUTOGENERATED FROM $path$\n"
-      "\n"
-      "#pragma once\n"
-      "\n"
-      "#include <string>\n"
-      "\n"
-      "#include \"kudu/gutil/ref_counted.h\"\n"
-      "#include \"kudu/rpc/service_if.h\"\n"
-      "\n"
-      "namespace google {\n"
-      "namespace protobuf {\n"
-      "class Message;\n"
-      "} // namespace protobuf\n"
-      "} // namespace google\n"
-      "\n"
-      "namespace kudu {\n"
-      "class MetricEntity;\n"
-      "namespace rpc {\n"
-      "class ResultTracker;\n"
-      "class RpcContext;\n"
-      "} // namespace rpc\n"
-      "} // namespace kudu\n"
-      "\n"
-      "$open_namespace$"
-      "\n");
+        "// THIS FILE IS AUTOGENERATED FROM $path$\n"
+        "\n"
+        "#pragma once\n"
+        "\n"
+        "#include <string>\n"
+        "\n"
+        "#include \"kudu/gutil/ref_counted.h\"\n"
+        "#include \"kudu/rpc/service_if.h\"\n"
+        "\n"
+        "namespace google {\n"
+        "namespace protobuf {\n"
+        "class Message;\n"
+        "} // namespace protobuf\n"
+        "} // namespace google\n"
+        "\n"
+        "namespace kudu {\n"
+        "class MetricEntity;\n"
+        "namespace rpc {\n"
+        "class ResultTracker;\n"
+        "class RpcContext;\n"
+        "} // namespace rpc\n"
+        "} // namespace kudu\n"
+        "\n"
+        "$open_namespace$");
 
     for (int service_idx = 0; service_idx < file->service_count();
          ++service_idx) {
@@ -348,26 +344,30 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
       subs->PushService(service);
 
       Print(printer, *subs,
-        "class $service_name$If : public ::kudu::rpc::GeneratedServiceIf {\n"
-        " public:\n"
-        "  explicit $service_name$If(const scoped_refptr<::kudu::MetricEntity>& entity,"
-            " const scoped_refptr<::kudu::rpc::ResultTracker>& result_tracker);\n"
-        "  virtual ~$service_name$If();\n"
-        "  std::string service_name() const override;\n"
-        "  static std::string static_service_name();\n"
-        "\n"
-        );
+          "\n"
+          "class $service_name$If : public ::kudu::rpc::GeneratedServiceIf {\n"
+          " public:\n"
+          "  static const std::string& static_service_name();\n"
+          "\n"
+          "  $service_name$If(\n"
+          "      const scoped_refptr<::kudu::MetricEntity>& entity,\n"
+          "      const scoped_refptr<::kudu::rpc::ResultTracker>& result_tracker);\n"
+          "  virtual ~$service_name$If();\n"
+          "\n"
+          "  const std::string& service_name() const override;\n");
 
       set<string> authz_methods;
       for (int method_idx = 0; method_idx < service->method_count();
            ++method_idx) {
-        const MethodDescriptor *method = service->method(method_idx);
+        const MethodDescriptor* method = service->method(method_idx);
         subs->PushMethod(method);
 
         Print(printer, *subs,
-        "  virtual void $rpc_name$(const class $request$ *req,\n"
-        "      class $response$ *resp, ::kudu::rpc::RpcContext *context) = 0;\n"
-        );
+            "\n"
+            "  virtual void $rpc_name$(\n"
+            "      const class $request$* req,\n"
+            "      class $response$* resp,\n"
+            "      ::kudu::rpc::RpcContext* context) = 0;\n");
         subs->Pop(); // method
         if (auto m = GetAuthzMethod(*method)) {
           authz_methods.insert(m.get());
@@ -376,50 +376,53 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
 
       if (!authz_methods.empty()) {
         printer->Print(
-        "\n\n"
-        "  // Authorization methods\n"
-        "  // ---------------------\n\n");
+            "\n"
+            "  // Authorization methods\n"
+            "  // ---------------------\n");
       }
       for (const string& m : authz_methods) {
         printer->Print({ {"m", m} },
-        "  virtual bool $m$(const google::protobuf::Message* req,\n"
-        "     google::protobuf::Message* resp, ::kudu::rpc::RpcContext *context) = 0;\n");
+            "\n"
+            "  virtual bool $m$(\n"
+            "      const google::protobuf::Message* req,\n"
+            "      google::protobuf::Message* resp,\n"
+            "      ::kudu::rpc::RpcContext* context) = 0;\n");
       }
 
       Print(printer, *subs,
-        "\n"
-        "};\n"
-      );
+          "\n"
+          " private:\n"
+          "  static const std::string kServiceName_;\n"
+          "};\n");
 
       subs->Pop(); // Service
     }
 
     Print(printer, *subs,
-      "\n"
-      "$close_namespace$"
-      "\n");
+        "\n"
+        "$close_namespace$");
   }
 
   static void GenerateServiceIf(Printer* printer,
                                 SubstitutionContext* subs,
                                 const FileDescriptor* file) {
     Print(printer, *subs,
-      "// THIS FILE IS AUTOGENERATED FROM $path$\n"
-      "\n"
-      "#include <functional>\n"
-      "#include <memory>\n"
-      "#include <unordered_map>\n"
-      "#include <utility>\n"
-      "\n"
-      "#include <google/protobuf/message.h>\n"
-      "\n"
-      "#include \"$path_no_extension$.pb.h\"\n"
-      "#include \"$path_no_extension$.service.h\"\n"
-      "\n"
-      "#include \"kudu/rpc/result_tracker.h\"\n"
-      "#include \"kudu/rpc/service_if.h\"\n"
-      "#include \"kudu/util/metrics.h\"\n"
-      "\n");
+        "// THIS FILE IS AUTOGENERATED FROM $path$\n"
+        "\n"
+        "#include <functional>\n"
+        "#include <memory>\n"
+        "#include <unordered_map>\n"
+        "#include <utility>\n"
+        "\n"
+        "#include <google/protobuf/message.h>\n"
+        "\n"
+        "#include \"$path_no_extension$.pb.h\"\n"
+        "#include \"$path_no_extension$.service.h\"\n"
+        "\n"
+        "#include \"kudu/rpc/result_tracker.h\"\n"
+        "#include \"kudu/rpc/service_if.h\"\n"
+        "#include \"kudu/util/metrics.h\"\n"
+        "\n");
 
     // Define metric prototypes for each method in the service.
     for (int service_idx = 0; service_idx < file->service_count();
@@ -432,20 +435,22 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
         const MethodDescriptor* method = service->method(method_idx);
         subs->PushMethod(method);
         Print(printer, *subs,
-          "METRIC_DEFINE_histogram(server, handler_latency_$rpc_full_name_plainchars$,\n"
-          "  \"$rpc_full_name$ RPC Time\",\n"
-          "  kudu::MetricUnit::kMicroseconds,\n"
-          "  \"Microseconds spent handling $rpc_full_name$() RPC requests\",\n"
-          "  kudu::MetricLevel::kInfo,\n"
-          "  60000000LU, 2);\n"
-          "\n");
+            "METRIC_DEFINE_histogram(server,\n"
+            "    handler_latency_$rpc_full_name_plainchars$,\n"
+            "    \"$rpc_full_name$ RPC Time\",\n"
+            "    kudu::MetricUnit::kMicroseconds,\n"
+            "    \"Microseconds spent handling $rpc_full_name$ RPC requests\",\n"
+            "    kudu::MetricLevel::kInfo,\n"
+            "    60000000LU, 2);\n"
+            "\n");
         Print(printer, *subs,
-          "METRIC_DEFINE_counter(server, queue_overflow_rejections_$rpc_full_name_plainchars$,\n"
-          "  \"$rpc_full_name$ RPC Rejections\",\n"
-          "  kudu::MetricUnit::kRequests,\n"
-          "  \"Number of rejected $rpc_full_name$() requests due to RPC queue overflow\",\n"
-          "  kudu::MetricLevel::kInfo);\n"
-          "\n");
+            "METRIC_DEFINE_counter(server,\n"
+            "    queue_overflow_rejections_$rpc_full_name_plainchars$,\n"
+            "    \"$rpc_full_name$ RPC Rejections\",\n"
+            "    kudu::MetricUnit::kRequests,\n"
+            "    \"Number of rejected $rpc_full_name$ requests due to RPC queue overflow\",\n"
+            "    kudu::MetricLevel::kInfo);\n"
+            "\n");
         subs->Pop(); // method
       }
 
@@ -453,15 +458,14 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
     }
 
     Print(printer, *subs,
-      "using google::protobuf::Message;\n"
-      "using kudu::MetricEntity;\n"
-      "using kudu::rpc::ResultTracker;\n"
-      "using kudu::rpc::RpcContext;\n"
-      "using kudu::rpc::RpcMethodInfo;\n"
-      "using std::unique_ptr;\n"
-      "\n"
-      "$open_namespace$"
-      "\n");
+        "using google::protobuf::Message;\n"
+        "using kudu::MetricEntity;\n"
+        "using kudu::rpc::ResultTracker;\n"
+        "using kudu::rpc::RpcContext;\n"
+        "using kudu::rpc::RpcMethodInfo;\n"
+        "using std::string;\n"
+        "\n"
+        "$open_namespace$");
 
     for (int service_idx = 0; service_idx < file->service_count();
          ++service_idx) {
@@ -469,9 +473,17 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
       subs->PushService(service);
 
       Print(printer, *subs,
-        "$service_name$If::$service_name$If(const scoped_refptr<MetricEntity>& entity,"
-            " const scoped_refptr<ResultTracker>& result_tracker) {\n"
-            "result_tracker_ = result_tracker;\n"
+          "\n"
+          "const string $service_name$If::kServiceName_ = \"$full_service_name$\";\n"
+          "\n"
+          "const string& $service_name$If::static_service_name() {\n"
+          "  return kServiceName_;\n"
+          "}\n"
+          "\n"
+          "$service_name$If::$service_name$If(\n"
+          "    const scoped_refptr<MetricEntity>& entity,\n"
+          "    const scoped_refptr<ResultTracker>& result_tracker)\n"
+          "    : GeneratedServiceIf(result_tracker) {\n"
       );
       for (int method_idx = 0; method_idx < service->method_count();
            ++method_idx) {
@@ -479,82 +491,76 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
         subs->PushMethod(method);
 
         Print(printer, *subs,
-              "  {\n"
-              "    scoped_refptr<RpcMethodInfo> mi(new RpcMethodInfo);\n"
-              "    mi->req_prototype.reset(new $request$);\n"
-              "    mi->resp_prototype.reset(new $response$);\n"
-              "    mi->authz_method = [this](const Message* req, Message* resp,\n"
-              "                              RpcContext* ctx) {\n"
-              "      return this->$authz_method$(static_cast<const $request$*>(req),\n"
-              "                           static_cast<$response$*>(resp),\n"
-              "                           ctx);\n"
-              "    };\n"
-              "    mi->track_result = $track_result$;\n"
-              "    mi->handler_latency_histogram =\n"
-              "        METRIC_handler_latency_$rpc_full_name_plainchars$.Instantiate(entity);\n"
-              "    mi->queue_overflow_rejections =\n"
-              "        METRIC_queue_overflow_rejections_$rpc_full_name_plainchars$.Instantiate(\n"
-              "            entity);\n"
-              "    mi->func = [this](const Message* req, Message* resp, RpcContext* ctx) {\n"
-              "      this->$rpc_name$(static_cast<const $request$*>(req),\n"
-              "                       static_cast<$response$*>(resp),\n"
-              "                       ctx);\n"
-              "    };\n"
-              "    methods_by_name_[\"$rpc_name$\"] = std::move(mi);\n"
-              "  }\n");
+            "  {\n"
+            "    scoped_refptr<RpcMethodInfo> mi(new RpcMethodInfo);\n"
+            "    mi->req_prototype.reset(new $request$);\n"
+            "    mi->resp_prototype.reset(new $response$);\n"
+            "    mi->authz_method = [this](const Message* req, Message* resp,\n"
+            "                              RpcContext* ctx) {\n"
+            "      return this->$authz_method$(\n"
+            "          static_cast<const $request$*>(req),\n"
+            "          static_cast<$response$*>(resp),\n"
+            "          ctx);\n"
+            "    };\n"
+            "    mi->track_result = $track_result$;\n"
+            "    mi->handler_latency_histogram =\n"
+            "        METRIC_handler_latency_$rpc_full_name_plainchars$.Instantiate(entity);\n"
+            "    mi->queue_overflow_rejections =\n"
+            "        METRIC_queue_overflow_rejections_$rpc_full_name_plainchars$.Instantiate("
+            "entity);\n"
+            "    mi->func = [this](const Message* req, Message* resp, RpcContext* ctx) {\n"
+            "      this->$rpc_name$(\n"
+            "          static_cast<const $request$*>(req),\n"
+            "          static_cast<$response$*>(resp),\n"
+            "          ctx);\n"
+            "    };\n"
+            "    methods_by_name_[\"$rpc_name$\"] = std::move(mi);\n"
+            "  }\n");
         subs->Pop(); // method
       }
 
       Print(printer, *subs,
-        "}\n"
-        "\n"
-        "$service_name$If::~$service_name$If() {\n"
-        "}\n"
-        "\n"
-        "std::string $service_name$If::service_name() const {\n"
-        "  return \"$full_service_name$\";\n"
-        "}\n"
-        "std::string $service_name$If::static_service_name() {\n"
-        "  return \"$full_service_name$\";\n"
-        "}\n"
-        "\n"
-      );
+          "}\n"
+          "\n"
+          "$service_name$If::~$service_name$If() {\n"
+          "}\n"
+          "\n"
+          "const string& $service_name$If::service_name() const {\n"
+          "  return kServiceName_;\n"
+          "}\n");
 
       subs->Pop(); // service
     }
 
     Print(printer, *subs,
-      "\n"
-      "$close_namespace$"
-      "\n"
-      );
+        "\n"
+        "$close_namespace$");
   }
 
   static void GenerateProxyHeader(Printer* printer,
                                   SubstitutionContext* subs,
                                   const FileDescriptor* file) {
     Print(printer, *subs,
-      "// THIS FILE IS AUTOGENERATED FROM $path$\n"
-      "\n"
-      "#pragma once\n"
-      "\n"
-      "#include <memory>\n"
-      "#include <string>\n"
-      "\n"
-      "#include \"kudu/rpc/proxy.h\"\n"
-      "#include \"kudu/rpc/response_callback.h\"\n"
-      "#include \"kudu/util/status.h\"\n"
-      "\n"
-      "namespace kudu { class Sockaddr; }\n"
-      "namespace kudu {\n"
-      "namespace rpc {\n"
-      "class Messenger;\n"
-      "class RpcController;\n"
-      "} // namespace rpc\n"
-      "} // namespace kudu\n"
-      "\n"
-      "$open_namespace$"
-      "\n");
+        "// THIS FILE IS AUTOGENERATED FROM $path$\n"
+        "\n"
+        "#pragma once\n"
+        "\n"
+        "#include <memory>\n"
+        "#include <string>\n"
+        "\n"
+        "#include \"kudu/rpc/proxy.h\"\n"
+        "#include \"kudu/rpc/response_callback.h\"\n"
+        "#include \"kudu/util/status.h\"\n"
+        "\n"
+        "namespace kudu { class Sockaddr; }\n"
+        "namespace kudu {\n"
+        "namespace rpc {\n"
+        "class Messenger;\n"
+        "class RpcController;\n"
+        "} // namespace rpc\n"
+        "} // namespace kudu\n"
+        "\n"
+        "$open_namespace$");
 
     for (int service_idx = 0; service_idx < file->service_count();
          ++service_idx) {
@@ -562,14 +568,14 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
       subs->PushService(service);
 
       Print(printer, *subs,
-        "class $service_name$Proxy : public ::kudu::rpc::Proxy {\n"
-        " public:\n"
-        "  $service_name$Proxy(std::shared_ptr<::kudu::rpc::Messenger>\n"
-        "                messenger, const ::kudu::Sockaddr &sockaddr,"
-        "                std::string hostname);\n"
-        "  ~$service_name$Proxy();\n"
-        "\n"
-        );
+          "\n"
+          "class $service_name$Proxy : public ::kudu::rpc::Proxy {\n"
+          " public:\n"
+          "  $service_name$Proxy(\n"
+          "      std::shared_ptr<::kudu::rpc::Messenger> messenger,\n"
+          "      const ::kudu::Sockaddr& sockaddr,\n"
+          "      std::string hostname);\n"
+          "  ~$service_name$Proxy();\n");
 
       for (int method_idx = 0; method_idx < service->method_count();
            ++method_idx) {
@@ -577,92 +583,96 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
         subs->PushMethod(method);
 
         Print(printer, *subs,
-        "\n"
-        "  ::kudu::Status $rpc_name$(const class $request$ &req,\n"
-        "                            class $response$ *resp,\n"
-        "                            ::kudu::rpc::RpcController *controller);\n"
-        "  void $rpc_name$Async(const class $request$ &req,\n"
-        "                       class $response$ *response,\n"
-        "                       ::kudu::rpc::RpcController *controller,\n"
-        "                       const ::kudu::rpc::ResponseCallback &callback);\n"
-        );
+            "\n"
+            "  ::kudu::Status $rpc_name$(\n"
+            "      const class $request$& req,\n"
+            "      class $response$* resp,\n"
+            "      ::kudu::rpc::RpcController* controller);\n"
+            "  void $rpc_name$Async(\n"
+            "      const class $request$& req,\n"
+            "      class $response$* response,\n"
+            "      ::kudu::rpc::RpcController* controller,\n"
+            "      const ::kudu::rpc::ResponseCallback& callback);\n");
         subs->Pop(); // method
       }
-      Print(printer, *subs,
-      "};\n");
+      Print(printer, *subs, "};\n");
       subs->Pop(); // service
     }
     Print(printer, *subs,
-      "\n"
-      "$close_namespace$"
-      "\n"
-    );
+        "\n"
+        "$close_namespace$");
   }
 
   static void GenerateProxy(Printer* printer,
                             SubstitutionContext* subs,
                             const FileDescriptor* file) {
     Print(printer, *subs,
-      "// THIS FILE IS AUTOGENERATED FROM $path$\n"
-      "\n"
-      "#include <string>\n"
-      "#include <utility>\n"
-      "\n"
-      "#include \"$path_no_extension$.pb.h\"\n"
-      "#include \"$path_no_extension$.proxy.h\"\n"
-      "\n"
-      "namespace kudu {\n"
-      "namespace rpc {\n"
-      "class Messenger;\n"
-      "class RpcController;\n"
-      "} // namespace rpc\n"
-      "} // namespace kudu\n"
-      "\n"
-      "$open_namespace$"
-      "\n");
+        "// THIS FILE IS AUTOGENERATED FROM $path$\n"
+        "\n"
+        "#include <string>\n"
+        "#include <utility>\n"
+        "\n"
+        "#include \"$path_no_extension$.pb.h\"\n"
+        "#include \"$path_no_extension$.proxy.h\"\n"
+        "\n"
+        "namespace kudu {\n"
+        "namespace rpc {\n"
+        "class Messenger;\n"
+        "class RpcController;\n"
+        "} // namespace rpc\n"
+        "} // namespace kudu\n"
+        "\n"
+        "$open_namespace$");
 
     for (int service_idx = 0; service_idx < file->service_count();
          ++service_idx) {
       const ServiceDescriptor* service = file->service(service_idx);
       subs->PushService(service);
       Print(printer, *subs,
-        "$service_name$Proxy::$service_name$Proxy(\n"
-        "   std::shared_ptr< ::kudu::rpc::Messenger> messenger,\n"
-        "   const ::kudu::Sockaddr &remote, std::string hostname)\n"
-        "  : Proxy(std::move(messenger), remote, std::move(hostname), \"$full_service_name$\") {\n"
-        "}\n"
-        "\n"
-        "$service_name$Proxy::~$service_name$Proxy() {\n"
-        "}\n"
-        "\n"
-        "\n");
+          "\n"
+          "$service_name$Proxy::$service_name$Proxy(\n"
+          "    std::shared_ptr<::kudu::rpc::Messenger> messenger,\n"
+          "    const ::kudu::Sockaddr& remote,\n"
+          "    std::string hostname)\n"
+          "    : Proxy(std::move(messenger),\n"
+          "            remote,\n"
+          "            std::move(hostname),\n"
+          "            \"$full_service_name$\") {\n"
+          "}\n"
+          "\n"
+          "$service_name$Proxy::~$service_name$Proxy() {\n"
+          "}\n");
       for (int method_idx = 0; method_idx < service->method_count();
            ++method_idx) {
         const MethodDescriptor* method = service->method(method_idx);
         subs->PushMethod(method);
         Print(printer, *subs,
-        "::kudu::Status $service_name$Proxy::$rpc_name$(const $request$ &req, $response$ *resp,\n"
-        "                                     ::kudu::rpc::RpcController *controller) {\n"
-        "  return SyncRequest(\"$rpc_name$\", req, resp, controller);\n"
-        "}\n"
-        "\n"
-        "void $service_name$Proxy::$rpc_name$Async(const $request$ &req,\n"
-        "                     $response$ *resp, ::kudu::rpc::RpcController *controller,\n"
-        "                     const ::kudu::rpc::ResponseCallback &callback) {\n"
-        "  AsyncRequest(\"$rpc_name$\", req, resp, controller, callback);\n"
-        "}\n"
-        "\n");
+            "\n"
+            "::kudu::Status $service_name$Proxy::$rpc_name$(\n"
+            "    const $request$& req,\n"
+            "    $response$* resp,\n"
+            "    ::kudu::rpc::RpcController* controller) {\n"
+            "  return SyncRequest(\"$rpc_name$\", req, resp, controller);\n"
+            "}\n"
+            "\n"
+            "void $service_name$Proxy::$rpc_name$Async(\n"
+            "    const $request$& req,\n"
+            "    $response$* resp,\n"
+            "    ::kudu::rpc::RpcController* controller,\n"
+            "    const ::kudu::rpc::ResponseCallback& callback) {\n"
+            "  AsyncRequest(\"$rpc_name$\", req, resp, controller, callback);\n"
+            "}\n");
         subs->Pop(); // method
       }
 
       subs->Pop(); // service
     }
     Print(printer, *subs,
-      "\n"
-      "$close_namespace$"
-      "\n");
+        "\n"
+        "$close_namespace$");
   }
 };
+
 } // namespace rpc
 } // namespace kudu
 
