@@ -783,10 +783,10 @@ load_stats_and_generate_plots() {
   load_and_generate_plot "${FS_SCANINSERT_DISK}%_scan%" fs-withdisk-scan
 
   load_and_generate_plot "${DENSE_NODE_ITEST}_time%" dense-node-bench-times
-  load_and_generate_plot "${DENSE_NODE_ITEST}_num%containers%" dense-node-bench-containers
-  load_and_generate_plot "${DENSE_NODE_ITEST}_num_blocks%" dense-node-bench-blocks
-  load_and_generate_plot "${DENSE_NODE_ITEST}_num_threads%" dense-node-bench-threads
-  load_and_generate_plot "${DENSE_NODE_ITEST}_num_bytes%" dense-node-bench-bytes
+  load_and_generate_plot "${DENSE_NODE_ITEST}_num%containers%" dense-node-bench-containers "number of log block containers"
+  load_and_generate_plot "${DENSE_NODE_ITEST}_num_blocks%" dense-node-bench-blocks "number of blocks"
+  load_and_generate_plot "${DENSE_NODE_ITEST}_num_threads%" dense-node-bench-threads "threads running"
+  load_and_generate_plot "${DENSE_NODE_ITEST}_num_bytes%" dense-node-bench-bytes "bytes under management"
 
   load_and_generate_plot "${BLOCKING_QUEUE_SYMMETRIC_TEST}%" blocking-queue-symmetric "function call rate (req/sec)"
   load_and_generate_plot "${BLOCKING_QUEUE_NON_SYMMETRIC_TEST}%" blocking-queue-non-symmetric "function call rate (req/sec)"
