@@ -31,9 +31,9 @@
 #include <mutex>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <vector>
 
-#include <boost/algorithm/string/predicate.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -51,6 +51,7 @@
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
+#include "kudu/util/string_case.h"
 #include "kudu/util/user.h"
 
 // Hard code OpenSSL flag values from OpenSSL 1.0.1e[1][2] when compiling
@@ -171,13 +172,13 @@ Status TlsContext::Init() {
   //   https://tools.ietf.org/html/rfc7525#section-3.3
   auto options = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_COMPRESSION;
 
-  if (boost::iequals(tls_min_protocol_, "TLSv1.2")) {
+  if (iequals(tls_min_protocol_, "TLSv1.2")) {
     RETURN_NOT_OK(CheckMaxSupportedTlsVersion(TLS1_2_VERSION, "TLSv1.2"));
     options |= SSL_OP_NO_TLSv1 | SSL_OP_NO_TLSv1_1;
-  } else if (boost::iequals(tls_min_protocol_, "TLSv1.1")) {
+  } else if (iequals(tls_min_protocol_, "TLSv1.1")) {
     RETURN_NOT_OK(CheckMaxSupportedTlsVersion(TLS1_1_VERSION, "TLSv1.1"));
     options |= SSL_OP_NO_TLSv1;
-  } else if (!boost::iequals(tls_min_protocol_, "TLSv1")) {
+  } else if (!iequals(tls_min_protocol_, "TLSv1")) {
     return Status::InvalidArgument("unknown value provided for --rpc_tls_min_protocol flag",
                                    tls_min_protocol_);
   }
@@ -245,9 +246,13 @@ Status TlsContext::VerifyCertChainUnlocked(const Cert& cert) {
   int rc = X509_verify_cert(store_ctx.get());
   if (rc != 1) {
     int err = X509_STORE_CTX_get_error(store_ctx.get());
+
+    // This also clears the errors. It's important to do this before we call
+    // X509NameToString(), as it expects the error stack to be empty and it
+    // calls SCOPED_OPENSSL_NO_PENDING_ERRORS.
+    const auto error_msg = GetOpenSSLErrors();
     if (err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT) {
       // It's OK to provide a self-signed cert.
-      ERR_clear_error(); // in case it left anything on the queue.
       return Status::OK();
     }
 
@@ -260,9 +265,8 @@ Status TlsContext::VerifyCertChainUnlocked(const Cert& cert) {
                                 X509NameToString(X509_get_issuer_name(cur_cert)));
     }
 
-    ERR_clear_error(); // in case it left anything on the queue.
     return Status::RuntimeError(
-        Substitute("could not verify certificate chain$0", cert_details),
+        Substitute("could not verify certificate chain$0: $1", cert_details, error_msg),
         X509_verify_cert_error_string(err));
   }
   return Status::OK();
