@@ -25,7 +25,6 @@
 #include <string>
 #include <vector>
 
-#include <boost/algorithm/string/predicate.hpp>
 #include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
@@ -87,6 +86,7 @@
 #endif
 #include "kudu/util/slice.h"
 #include "kudu/util/spinlock_profiling.h"
+#include "kudu/util/string_case.h"
 #include "kudu/util/thread.h"
 #include "kudu/util/user.h"
 #include "kudu/util/version_info.h"
@@ -101,9 +101,8 @@ DEFINE_int32(max_negotiation_threads, 50, "Maximum number of connection negotiat
 TAG_FLAG(max_negotiation_threads, advanced);
 
 DEFINE_int64(rpc_negotiation_timeout_ms, 3000,
-             "Timeout for negotiating an RPC connection.");
+             "Timeout for negotiating an RPC connection, in milliseconds");
 TAG_FLAG(rpc_negotiation_timeout_ms, advanced);
-TAG_FLAG(rpc_negotiation_timeout_ms, runtime);
 
 DEFINE_bool(webserver_enabled, true, "Whether to enable the web server on this daemon. "
             "NOTE: disabling the web server is also likely to prevent monitoring systems "
@@ -221,6 +220,11 @@ DECLARE_string(keytab_file);
 DECLARE_string(principal);
 
 METRIC_DECLARE_gauge_size(merged_entities_count_of_server);
+METRIC_DEFINE_gauge_int64(server, uptime,
+                          "Server Uptime",
+                          kudu::MetricUnit::kMicroseconds,
+                          "Time interval since the server has started",
+                          kudu::MetricLevel::kInfo);
 
 using kudu::security::RpcAuthentication;
 using kudu::security::RpcEncryption;
@@ -393,6 +397,7 @@ int64_t GetFileCacheCapacity(Env* env) {
 ServerBase::ServerBase(string name, const ServerBaseOptions& options,
                        const string& metric_namespace)
     : name_(std::move(name)),
+      start_time_(MonoTime::Min()),
       minidump_handler_(new MinidumpExceptionHandler()),
       mem_tracker_(CreateMemTrackerForServer()),
       metric_registry_(new MetricRegistry()),
@@ -505,7 +510,6 @@ Status ServerBase::Init() {
 
   // Create the Messenger.
   rpc::MessengerBuilder builder(name_);
-
   builder.set_num_reactors(FLAGS_num_reactor_threads)
          .set_min_negotiation_threads(FLAGS_min_negotiation_threads)
          .set_max_negotiation_threads(FLAGS_max_negotiation_threads)
@@ -536,28 +540,22 @@ Status ServerBase::Init() {
   if (FLAGS_rpc_listen_on_unix_domain_socket) {
     VLOG(1) << "Enabling listening on unix domain socket.";
     Sockaddr addr;
+#if !defined(__APPLE__)
     RETURN_NOT_OK_PREPEND(addr.ParseUnixDomainPath(Substitute("@kudu-$0", fs_manager_->uuid())),
                           "unable to parse provided UNIX socket path");
+#else
+    RETURN_NOT_OK_PREPEND(addr.ParseUnixDomainPath(Substitute("/tmp/kudu-$0", fs_manager_->uuid())),
+                          "unable to parse provided UNIX socket path");
+#endif
     RETURN_NOT_OK_PREPEND(rpc_server_->AddBindAddress(addr),
                           "unable to add configured UNIX socket path to list of bind addresses "
                           "for RPC server");
   }
 
-  RETURN_NOT_OK(rpc_server_->Bind());
-
-  RETURN_NOT_OK_PREPEND(StartMetricsLogging(), "Could not enable metrics logging");
-
-  result_tracker_->StartGCThread();
-  RETURN_NOT_OK(StartExcessLogFileDeleterThread());
-#ifdef TCMALLOC_ENABLED
-  RETURN_NOT_OK(StartTcmallocMemoryGcThread());
-#endif
-
-  return Status::OK();
+  return rpc_server_->Bind();
 }
 
 Status ServerBase::InitAcls() {
-
   string service_user;
   boost::optional<string> keytab_user = security::GetLoggedInUsernameFromKeytab();
   if (keytab_user) {
@@ -665,10 +663,10 @@ Status ServerBase::DumpServerInfo(const string& path,
   ServerStatusPB status;
   RETURN_NOT_OK_PREPEND(GetStatusPB(&status), "could not get server status");
 
-  if (boost::iequals(format, "json")) {
+  if (iequals(format, "json")) {
     string json = JsonWriter::ToJson(status, JsonWriter::PRETTY);
     RETURN_NOT_OK(WriteStringToFile(options_.env, Slice(json), path));
-  } else if (boost::iequals(format, "pb")) {
+  } else if (iequals(format, "pb")) {
     // TODO: Use PB container format?
     RETURN_NOT_OK(pb_util::WritePBToPath(options_.env, path, status,
                                          pb_util::NO_SYNC)); // durability doesn't matter
@@ -785,6 +783,27 @@ Status ServerBase::Start() {
 
   RETURN_NOT_OK(RegisterService(
       unique_ptr<rpc::ServiceIf>(new GenericServiceImpl(this))));
+
+  // Webserver shows the wall clock time when server was started and exposes
+  // the server's uptime along with all other metrics, and the metrics logger
+  // accesses the uptime metric as well, so it's necessary to have corresponding
+  // information ready at this point.
+  start_time_ = MonoTime::Now();
+  start_walltime_ = static_cast<int64_t>(WallTime_Now());
+
+  METRIC_uptime.InstantiateFunctionGauge(
+      metric_entity_,
+      [this]() {return (MonoTime::Now() - this->start_time()).ToMicroseconds();})->
+          AutoDetachToLastValue(&metric_detacher_);
+
+  RETURN_NOT_OK_PREPEND(StartMetricsLogging(), "Could not enable metrics logging");
+
+  result_tracker_->StartGCThread();
+  RETURN_NOT_OK(StartExcessLogFileDeleterThread());
+#ifdef TCMALLOC_ENABLED
+  RETURN_NOT_OK(StartTcmallocMemoryGcThread());
+#endif
+
   RETURN_NOT_OK(rpc_server_->Start());
 
   if (web_server_) {
@@ -801,8 +820,6 @@ Status ServerBase::Start() {
                           "Failed to dump server info to " + options_.dump_info_path);
   }
 
-  start_time_ = WallTime_Now();
-
   return Status::OK();
 }
 
