@@ -17,6 +17,12 @@
 
 #include "kudu/server/webserver.h"
 
+#if defined(KUDU_OPENSSL_REQUIRE_FIPS_HEADER)
+#include <openssl/fips.h>
+#else
+#include <openssl/crypto.h>
+#endif
+
 #include <cstdlib>
 #include <functional>
 #include <iosfwd>
@@ -100,14 +106,16 @@ class WebserverTest : public KuduTest {
     server_.reset(new Webserver(opts));
 
     AddDefaultPathHandlers(server_.get());
-    ASSERT_OK(server_->Start());
+    if (!use_htpasswd() || !FIPS_mode()) {
+      ASSERT_OK(server_->Start());
 
-    vector<Sockaddr> addrs;
-    ASSERT_OK(server_->GetBoundAddresses(&addrs));
-    ASSERT_EQ(addrs.size(), 1);
-    ASSERT_TRUE(addrs[0].IsWildcard());
-    ASSERT_OK(addr_.ParseString("127.0.0.1", addrs[0].port()));
-    url_ = Substitute("http://$0", addr_.ToString());
+      vector<Sockaddr> addrs;
+      ASSERT_OK(server_->GetBoundAddresses(&addrs));
+      ASSERT_EQ(addrs.size(), 1);
+      ASSERT_TRUE(addrs[0].IsWildcard());
+      ASSERT_OK(addr_.ParseString("127.0.0.1", addrs[0].port()));
+      url_ = Substitute("http://$0", addr_.ToString());
+    }
   }
 
   void RunTestOptions() {
@@ -115,7 +123,7 @@ class WebserverTest : public KuduTest {
     curl_.set_return_headers(true);
     ASSERT_OK(curl_.FetchURL(url_, &buf_));
     ASSERT_STR_CONTAINS(buf_.ToString(),
-                        "Allow: GET, POST, HEAD, OPTIONS, PROPFIND, MKCOL");
+                        "Allow: GET, POST, HEAD, OPTIONS");
   }
 
  protected:
@@ -147,16 +155,31 @@ class PasswdWebserverTest : public WebserverTest {
 // Send a HTTP request with no username and password. It should reject
 // the request as the .htpasswd is presented to webserver.
 TEST_F(PasswdWebserverTest, TestPasswdMissing) {
+  if (FIPS_mode()) {
+    return;
+  }
   Status status = curl_.FetchURL(url_, &buf_);
   ASSERT_EQ("Remote error: HTTP 401", status.ToString());
 }
 
 TEST_F(PasswdWebserverTest, TestPasswdPresent) {
+  if (FIPS_mode()) {
+    return;
+  }
   ASSERT_OK(curl_.set_auth(CurlAuthType::DIGEST, security::kTestAuthUsername,
                            security::kTestAuthPassword));
   ASSERT_OK(curl_.FetchURL(addr_.ToString(), &buf_));
 }
 
+TEST_F(PasswdWebserverTest, TestCrashInFIPSMode) {
+  if (!FIPS_mode()) {
+    return;
+  }
+
+  Status s = server_->Start();
+  ASSERT_TRUE(s.IsIllegalState());
+  ASSERT_STR_CONTAINS("Digest authentication in FIPS approved mode", s.ToString());
+}
 
 class SpnegoWebserverTest : public WebserverTest {
  protected:
@@ -221,7 +244,21 @@ TEST_F(SpnegoWebserverTest, TestUnauthenticatedBadKeytab) {
   Status s = DoSpnegoCurl();
   EXPECT_EQ(s.ToString(), "Remote error: HTTP 401");
   EXPECT_EQ(kNotAuthn, last_authenticated_spn_);
-  EXPECT_STR_CONTAINS(buf_.ToString(), "GSS failure");
+  // The essence here is to get HTTP 401 error status in the server's response.
+  // There might be different messages returned from webserver because of
+  //   * different messages from GSSAPI on Linux and macOS (actually depends
+  //     on the version of SASL library, see kGssapiPattern in CleanSaslError())
+  //   * different GSSAPI failure paths depending on libkrb5 and libcurl
+  //     libraries: with the randomized keytab, the server fails with SASL step
+  //     with Status::Incomplete() on macOS. On Linux, the server fails the
+  //     SASL step with Status::NotAuthorized(). Instead of finding some
+  //     universal way of screwing up the keytab to get the same behavior on
+  //     both macOS and Linux, it's easier to rely on the fact that the required
+  //     HTTP error code is received and acknowledge for various error messages.
+  ASSERT_STR_MATCHES(buf_.ToString(),
+                     "(Unspecified GSS failure|"
+                     "GSSAPI Error: Miscellaneous failure|"
+                     "Must authenticate with SPNEGO)");
 }
 
 TEST_F(SpnegoWebserverTest, TestUnauthenticatedNoClientAuth) {
