@@ -37,6 +37,8 @@
 #include "kudu/consensus/log_anchor_registry.h"
 #include "kudu/gutil/atomicops.h"
 #include "kudu/gutil/macros.h"
+#include "kudu/gutil/ref_counted.h"
+#include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/concurrent_btree.h"
 #include "kudu/tablet/rowset.h"
 #include "kudu/tablet/rowset_metadata.h"
@@ -89,6 +91,7 @@ class CompactionInput;
 class MemRowSet;
 class Mutation;
 class OperationResultPB;
+class TxnMetadata;
 
 // The value stored in the CBTree for a single row.
 class MRSRow {
@@ -210,11 +213,20 @@ class MemRowSet : public RowSet,
   class Iterator;
 
   static Status Create(int64_t id,
-                       const Schema &schema,
+                       const Schema& schema,
                        log::LogAnchorRegistry* log_anchor_registry,
                        std::shared_ptr<MemTracker> parent_tracker,
                        std::shared_ptr<MemRowSet>* mrs);
 
+  // Create() variant for a MRS that get inserted to by a single transaction of
+  // the given transaction ID and metadata.
+  static Status Create(int64_t id,
+                       const Schema& schema,
+                       int64_t txn_id,
+                       scoped_refptr<TxnMetadata> txn_metadata,
+                       log::LogAnchorRegistry* log_anchor_registry,
+                       std::shared_ptr<MemTracker> parent_tracker,
+                       std::shared_ptr<MemRowSet>* mrs);
   ~MemRowSet();
 
   // Insert a new row into the memrowset.
@@ -231,7 +243,6 @@ class MemRowSet : public RowSet,
                 const ConstContiguousRow& row,
                 const consensus::OpId& op_id);
 
-
   // Update or delete an existing row in the memrowset.
   //
   // Returns Status::NotFound if the row doesn't exist.
@@ -348,6 +359,19 @@ class MemRowSet : public RowSet,
     return id_;
   }
 
+  // Transaction ID that inserted the rows of this MRS. 'none' if the rows in
+  // this MRS were not inserted as a part of a transaction.
+  const boost::optional<int64_t>& txn_id() const {
+    return txn_id_;
+  }
+
+  // Transaction metadata of the transaction that inserted the rows of this
+  // MRS. 'nullptr' if the rows in this MRS were not inserted as a part of a
+  // transaction.
+  const scoped_refptr<TxnMetadata>& txn_metadata() const {
+    return txn_metadata_;
+  }
+
   std::shared_ptr<RowSetMetadata> metadata() override {
     return std::shared_ptr<RowSetMetadata>(
         reinterpret_cast<RowSetMetadata *>(NULL));
@@ -360,7 +384,8 @@ class MemRowSet : public RowSet,
   virtual Status DebugDump(std::vector<std::string> *lines) override;
 
   std::string ToString() const override {
-    return "memrowset";
+    return strings::Substitute("memrowset$0",
+        txn_id_ ? strings::Substitute("(txn_id=$0)", *txn_id_) : "");
   }
 
   // Mark the memrowset as frozen. See CBTree::Freeze()
@@ -377,6 +402,10 @@ class MemRowSet : public RowSet,
 
   size_t DeltaMemStoreSize() const override { return 0; }
 
+  bool DeltaMemStoreInfo(size_t* /*size_bytes*/, MonoTime* /*creation_time*/) const override {
+    return false;
+  }
+
   bool DeltaMemStoreEmpty() const override { return true; }
 
   int64_t MinUnflushedLogIndex() const override {
@@ -427,7 +456,9 @@ class MemRowSet : public RowSet,
 
  protected:
   MemRowSet(int64_t id,
-            const Schema &schema,
+            const Schema& schema,
+            boost::optional<int64_t> txn_id,
+            scoped_refptr<TxnMetadata> txn_metadata,
             log::LogAnchorRegistry* log_anchor_registry,
             std::shared_ptr<MemTracker> parent_tracker);
 
@@ -443,8 +474,12 @@ class MemRowSet : public RowSet,
   typedef btree::CBTree<MSBTreeTraits> MSBTree;
 
   int64_t id_;
-
   const Schema schema_;
+
+  // The transaction ID that inserted into this MemRowSet, and its corresponding metadata.
+  boost::optional<int64_t> txn_id_;
+  scoped_refptr<TxnMetadata> txn_metadata_;
+
   std::shared_ptr<MemoryTrackingBufferAllocator> allocator_;
   std::shared_ptr<ThreadSafeMemoryTrackingArena> arena_;
 
@@ -522,7 +557,8 @@ class MemRowSet::Iterator : public RowwiseIterator {
     return MRSRow(memrowset_.get(), mrsrow_data);
   }
 
-  // Copy the current MRSRow to the 'dst_row' provided using the iterator projection schema.
+  // Copy the current MRSRow to the 'dst_row' provided using the iterator
+  // projection schema. Used in compactions.
   Status GetCurrentRow(RowBlockRow* dst_row,
                        Arena* row_arena,
                        Mutation** redo_head,
