@@ -18,6 +18,7 @@
 #include "kudu/tablet/ops/participant_op.h"
 
 #include <memory>
+#include <ostream>
 
 #include <glog/logging.h>
 #include <google/protobuf/arena.h>
@@ -27,12 +28,16 @@
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/opid.pb.h"
 #include "kudu/consensus/raft_consensus.h"
+#include "kudu/consensus/time_manager.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/rpc_header.pb.h"
+#include "kudu/tablet/mvcc.h"
 #include "kudu/tablet/ops/op.h"
+#include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet_replica.h"
 #include "kudu/tablet/txn_participant.h"
+#include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/status.h"
@@ -45,6 +50,7 @@ using kudu::consensus::OpId;
 using kudu::pb_util::SecureShortDebugString;
 using kudu::tablet::TabletReplica;
 using kudu::tserver::ParticipantOpPB;
+using kudu::tserver::TabletServerErrorPB;
 using std::string;
 using std::unique_ptr;
 using strings::Substitute;
@@ -88,53 +94,51 @@ string ParticipantOpState::ToString() const {
       ParticipantOpPB::ParticipantOpType_Name(request_->op().type()));
 }
 
-Status ParticipantOpState::ValidateOp() const {
+Status ParticipantOpState::ValidateOp() {
   const auto& op = request()->op();
   DCHECK(txn_);
+  TabletServerErrorPB::Code code = TabletServerErrorPB::UNKNOWN_ERROR;
+  Status s;
   switch (op.type()) {
     case ParticipantOpPB::BEGIN_TXN:
-      return txn_->ValidateBeginTransaction();
-    case ParticipantOpPB::BEGIN_COMMIT:
-      return txn_->ValidateBeginCommit();
+      s = txn_->ValidateBeginTransaction(&code);
+      break;
+    case ParticipantOpPB::BEGIN_COMMIT: {
+      Timestamp begin_commit_ts;
+      s = txn_->ValidateBeginCommit(&code, &begin_commit_ts);
+      if (PREDICT_FALSE(begin_commit_ts != Timestamp::kInvalidTimestamp)) {
+        DCHECK(s.IsIllegalState()) << s.ToString();
+        response_->set_timestamp(begin_commit_ts.value());
+      }
+      break;
+    }
     case ParticipantOpPB::FINALIZE_COMMIT:
-      return txn_->ValidateFinalize();
+      s = txn_->ValidateFinalize(&code);
+      break;
     case ParticipantOpPB::ABORT_TXN:
-      return txn_->ValidateAbort();
-    case ParticipantOpPB::UNKNOWN:
-      return Status::InvalidArgument("unknown op type");
+      s = txn_->ValidateAbort(&code);
+      break;
+    default:
+      s = Status::InvalidArgument("unknown op type");
+      break;
+  }
+  if (PREDICT_FALSE(!s.ok())) {
+    completion_callback()->set_error(s, code);
+    return s;
   }
   return Status::OK();
 }
 
-Status ParticipantOpState::PerformOp(const OpId& op_id) {
-  const auto& op = request()->op();
-  Txn* txn = txn_.get();
-  Status s;
-  switch (op.type()) {
-    // NOTE: these can currently never fail because we are only updating
-    // metadata. When we begin validating write ops before committing, we'll
-    // need to populate the response with errors.
-    case ParticipantOpPB::BEGIN_TXN: {
-      txn->BeginTransaction(op_id);
-      break;
-    }
-    case ParticipantOpPB::BEGIN_COMMIT: {
-      txn->BeginCommit(op_id);
-      break;
-    }
-    case ParticipantOpPB::FINALIZE_COMMIT: {
-      txn->FinalizeCommit(op_id, op.finalized_commit_timestamp());
-      break;
-    }
-    case ParticipantOpPB::ABORT_TXN: {
-      txn->AbortTransaction(op_id);
-      break;
-    }
-    case ParticipantOpPB::UNKNOWN: {
-      return Status::InvalidArgument("unknown op type");
-    }
-  }
-  return Status::OK();
+void ParticipantOpState::SetMvccOp(unique_ptr<ScopedOp> mvcc_op) {
+  DCHECK_EQ(ParticipantOpPB::BEGIN_COMMIT, request()->op().type());
+  DCHECK(nullptr == begin_commit_mvcc_op_);
+  begin_commit_mvcc_op_ = std::move(mvcc_op);
+}
+
+void ParticipantOpState::ReleaseMvccOpToTxn() {
+  DCHECK_EQ(ParticipantOpPB::BEGIN_COMMIT, request()->op().type());
+  DCHECK(begin_commit_mvcc_op_);
+  txn_->SetCommitOp(std::move(begin_commit_mvcc_op_));
 }
 
 void ParticipantOp::NewReplicateMsg(unique_ptr<ReplicateMsg>* replicate_msg) {
@@ -151,6 +155,15 @@ Status ParticipantOp::Prepare() {
   TRACE("PREPARE: Starting.");
   state_->AcquireTxnAndLock();
   RETURN_NOT_OK(state_->ValidateOp());
+
+  // Before we assign a timestamp, bump the clock so further ops get assigned
+  // higher timestamps (including this one).
+  if (state_->request()->op().type() == ParticipantOpPB::FINALIZE_COMMIT &&
+      type() == consensus::LEADER) {
+    DCHECK(!state_->consensus_round()->replicate_msg()->has_timestamp());
+    RETURN_NOT_OK(state_->tablet_replica()->time_manager()->UpdateClockAndLastAssignedTimestamp(
+        state_->commit_timestamp()));
+  }
   TRACE("PREPARE: Finished.");
   return Status::OK();
 }
@@ -159,14 +172,64 @@ Status ParticipantOp::Start() {
   DCHECK(!state_->has_timestamp());
   DCHECK(state_->consensus_round()->replicate_msg()->has_timestamp());
   state_->set_timestamp(Timestamp(state_->consensus_round()->replicate_msg()->timestamp()));
+  if (state_->request()->op().type() == ParticipantOpPB::BEGIN_COMMIT) {
+    // When beginning to commit, register an MVCC op so scanners at later
+    // timestamps wait for the commit to complete.
+    state_->tablet_replica()->tablet()->StartOp(state_.get());
+  }
   TRACE("START. Timestamp: $0", clock::HybridClock::GetPhysicalValueMicros(state_->timestamp()));
   return Status::OK();
 }
 
+Status ParticipantOpState::PerformOp(const consensus::OpId& op_id, Tablet* tablet) {
+  const auto& op = request()->op();
+  const auto& op_type = request()->op().type();
+  Status s;
+  switch (op_type) {
+    // NOTE: these can currently never fail because we are only updating
+    // metadata. When we begin validating write ops before committing, we'll
+    // need to populate the response with errors.
+    case ParticipantOpPB::BEGIN_TXN: {
+      tablet->BeginTransaction(txn_.get(), op_id);
+      break;
+    }
+    case ParticipantOpPB::BEGIN_COMMIT: {
+      tablet->BeginCommit(txn_.get(), begin_commit_mvcc_op_->timestamp(), op_id);
+      ReleaseMvccOpToTxn();
+      break;
+    }
+    case ParticipantOpPB::FINALIZE_COMMIT: {
+      DCHECK(op.has_finalized_commit_timestamp());
+      const auto& commit_ts = op.finalized_commit_timestamp();
+      tablet->CommitTransaction(txn_.get(), Timestamp(commit_ts), op_id);
+      // NOTE: we may not have a commit op if we are bootstrapping and we GCed
+      // the BEGIN_COMMIT op before flushing the finalized commit timestamp.
+      if (txn_->commit_op()) {
+        txn_->commit_op()->FinishApplying();
+      }
+      break;
+    }
+    case ParticipantOpPB::ABORT_TXN: {
+      tablet->AbortTransaction(txn_.get(), op_id);
+      // NOTE: we may not have a commit op if we are aborting before beginning
+      // to commit.
+      if (txn_->commit_op()) {
+        txn_->commit_op()->Abort();
+      }
+      break;
+    }
+    case ParticipantOpPB::UNKNOWN: {
+      return Status::InvalidArgument("unknown op type");
+    }
+  }
+  return Status::OK();
+}
+
 Status ParticipantOp::Apply(CommitMsg** commit_msg) {
   TRACE_EVENT0("op", "ParticipantOp::Apply");
   TRACE("APPLY: Starting.");
-  CHECK_OK(state_->PerformOp(state()->op_id()));
+  state_->tablet_replica()->tablet()->StartApplying(state_.get());
+  CHECK_OK(state_->PerformOp(state()->op_id(), state_->tablet_replica()->tablet()));
   *commit_msg = google::protobuf::Arena::CreateMessage<CommitMsg>(state_->pb_arena());
   (*commit_msg)->set_op_type(OperationType::PARTICIPANT_OP);
   TRACE("APPLY: Finished.");
@@ -177,14 +240,19 @@ void ParticipantOp::Finish(OpResult result) {
   auto txn_id = state_->request()->op().txn_id();
   state_->ReleaseTxn();
   TxnParticipant* txn_participant = state_->txn_participant_;
+
+  // If the transaction is complete, get rid of the in-flight Txn.
+  txn_participant->ClearIfComplete(txn_id);
+
   if (PREDICT_FALSE(result == Op::ABORTED)) {
+    // NOTE: The only way we end up with an init failure is if we ran a
+    // BEGIN_TXN op but aborted mid-way, leaving the Txn in the kInitialized
+    // state and no further ops attempting to drive the state change to kOpen.
     txn_participant->ClearIfInitFailed(txn_id);
     TRACE("FINISH: Op aborted");
     return;
   }
   DCHECK_EQ(result, Op::APPLIED);
-  // TODO(awong): when implementing transaction cleanup on participants, clean
-  // up finalized and aborted transactions here.
   TRACE("FINISH: Op applied");
 }
 
