@@ -53,6 +53,7 @@
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tablet/tablet_metrics.h"
 #include "kudu/tablet/tablet_replica.h"
+#include "kudu/tablet/txn_participant.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/flag_tags.h"
@@ -103,6 +104,7 @@ void AddWritePrivilegesForRowOperations(const RowOperationsPB::Type& op_type,
                                         WritePrivileges* privileges) {
   switch (op_type) {
     case RowOperationsPB::INSERT:
+    case RowOperationsPB::INSERT_IGNORE:
       InsertIfNotPresent(privileges, WritePrivilegeType::INSERT);
       break;
     case RowOperationsPB::UPSERT:
@@ -110,9 +112,11 @@ void AddWritePrivilegesForRowOperations(const RowOperationsPB::Type& op_type,
       InsertIfNotPresent(privileges, WritePrivilegeType::UPDATE);
       break;
     case RowOperationsPB::UPDATE:
+    case RowOperationsPB::UPDATE_IGNORE:
       InsertIfNotPresent(privileges, WritePrivilegeType::UPDATE);
       break;
     case RowOperationsPB::DELETE:
+    case RowOperationsPB::DELETE_IGNORE:
       InsertIfNotPresent(privileges, WritePrivilegeType::DELETE);
       break;
     default:
@@ -167,12 +171,35 @@ Status WriteOp::Prepare() {
 
   Tablet* tablet = state()->tablet_replica()->tablet();
 
-  Status s = tablet->DecodeWriteOperations(&client_schema, state());
+  // Before taking any other locks, acquire the transaction state lock and
+  // ensure it is open.
+  Status s;
+  if (state_->request()->has_txn_id()) {
+    s = tablet->AcquireTxnLock(state_->request()->txn_id(), state());
+    if (!s.ok()) {
+      state()->completion_callback()->set_error(s, TabletServerErrorPB::TXN_ILLEGAL_STATE);
+      return s;
+    }
+  }
+
+  s = tablet->DecodeWriteOperations(&client_schema, state());
   if (!s.ok()) {
     // TODO(unknown): is MISMATCHED_SCHEMA always right here? probably not.
     state()->completion_callback()->set_error(s, TabletServerErrorPB::MISMATCHED_SCHEMA);
     return s;
   }
+  // Only after decoding rows, check that only supported operations make it
+  // through.
+  if (state_->request()->has_txn_id()) {
+    for (const auto& op : state_->row_ops()) {
+      const auto& op_type = op->decoded_op.type;
+      if (op_type != RowOperationsPB::INSERT &&
+          op_type != RowOperationsPB::INSERT_IGNORE) {
+        state()->completion_callback()->set_error(s, TabletServerErrorPB::INVALID_MUTATION);
+        return Status::NotSupported("transactions may only insert");
+      }
+    }
+  }
 
   // Authorize the request if needed.
   const auto& authz_context = state()->authz_context();
@@ -271,7 +298,9 @@ void WriteOp::Finish(OpResult result) {
     metrics->insert_ignore_errors->IncrementBy(state_->metrics().insert_ignore_errors);
     metrics->rows_upserted->IncrementBy(state_->metrics().successful_upserts);
     metrics->rows_updated->IncrementBy(state_->metrics().successful_updates);
+    metrics->update_ignore_errors->IncrementBy(state_->metrics().update_ignore_errors);
     metrics->rows_deleted->IncrementBy(state_->metrics().successful_deletes);
+    metrics->delete_ignore_errors->IncrementBy(state_->metrics().delete_ignore_errors);
 
     if (type() == consensus::LEADER) {
       if (state()->external_consistency_mode() == COMMIT_WAIT) {
@@ -323,7 +352,7 @@ WriteOpState::WriteOpState(TabletReplica* tablet_replica,
   }
 }
 
-void WriteOpState::SetMvccTx(unique_ptr<ScopedOp> mvcc_op) {
+void WriteOpState::SetMvccOp(unique_ptr<ScopedOp> mvcc_op) {
   DCHECK(!mvcc_op_) << "Mvcc op already started/set.";
   mvcc_op_ = std::move(mvcc_op);
 }
@@ -335,6 +364,11 @@ void WriteOpState::set_tablet_components(
   tablet_components_ = components;
 }
 
+void WriteOpState::set_txn_rowsets(const scoped_refptr<TxnRowSets>& rowsets) {
+  DCHECK(!txn_rowsets_) << "Already set";
+  txn_rowsets_ = rowsets;
+}
+
 void WriteOpState::AcquireSchemaLock(rw_semaphore* schema_lock) {
   TRACE("Acquiring schema lock in shared mode");
   shared_lock<rw_semaphore> temp(*schema_lock);
@@ -342,6 +376,19 @@ void WriteOpState::AcquireSchemaLock(rw_semaphore* schema_lock) {
   TRACE("Acquired schema lock");
 }
 
+Status WriteOpState::AcquireTxnLockCheckOpen(scoped_refptr<Txn> txn) {
+  shared_lock<rw_semaphore> temp;
+  txn->AcquireReadLock(&temp);
+  const auto txn_state = txn->state();
+  if (PREDICT_FALSE(txn_state != kOpen)) {
+    return Status::InvalidArgument(Substitute("txn $0 is not open: $1",
+        txn->txn_id(), TxnStateToString(txn_state)));
+  }
+  txn_lock_.swap(temp);
+  txn_ = std::move(txn);
+  return Status::OK();
+}
+
 void WriteOpState::ReleaseSchemaLock() {
   shared_lock<rw_semaphore> temp;
   schema_lock_.swap(temp);
@@ -420,6 +467,7 @@ void WriteOpState::UpdateMetricsForOp(const RowOp& op) {
   }
   switch (op.decoded_op.type) {
     case RowOperationsPB::INSERT:
+      DCHECK(!op.error_ignored);
       op_metrics_.successful_inserts++;
       break;
     case RowOperationsPB::INSERT_IGNORE:
@@ -430,14 +478,31 @@ void WriteOpState::UpdateMetricsForOp(const RowOp& op) {
       }
       break;
     case RowOperationsPB::UPSERT:
+      DCHECK(!op.error_ignored);
       op_metrics_.successful_upserts++;
       break;
     case RowOperationsPB::UPDATE:
+      DCHECK(!op.error_ignored);
       op_metrics_.successful_updates++;
       break;
+    case RowOperationsPB::UPDATE_IGNORE:
+      if (op.error_ignored) {
+        op_metrics_.update_ignore_errors++;
+      } else {
+        op_metrics_.successful_updates++;
+      }
+      break;
     case RowOperationsPB::DELETE:
+      DCHECK(!op.error_ignored);
       op_metrics_.successful_deletes++;
       break;
+    case RowOperationsPB::DELETE_IGNORE:
+      if (op.error_ignored) {
+        op_metrics_.delete_ignore_errors++;
+      } else {
+        op_metrics_.successful_deletes++;
+      }
+      break;
     case RowOperationsPB::UNKNOWN:
     case RowOperationsPB::SPLIT_ROW:
     case RowOperationsPB::RANGE_LOWER_BOUND:
@@ -462,7 +527,20 @@ void WriteOpState::AcquireRowLocks(LockManager* lock_manager) {
   rows_lock_ = ScopedRowLock(lock_manager, this, keys, LockManager::LOCK_EXCLUSIVE);
 }
 
-void WriteOpState::ReleaseRowLocks() { rows_lock_.Release(); }
+void WriteOpState::ReleaseRowLocks() {
+  rows_lock_.Release();
+}
+
+void WriteOpState::ReleaseTxnLock() {
+  shared_lock<rw_semaphore> temp;
+  txn_lock_.swap(temp);
+  // It's possible we took a reference to a transaction that failed to start
+  // (e.g. because we changed leadership before the BEGIN_TXN could complete).
+  // If that's the case, clear its state in the participant.
+  txn_.reset();
+  tablet_replica()->tablet()->txn_participant()->ClearIfInitFailed(txn_->txn_id());
+  TRACE("Released schema lock");
+}
 
 WriteOpState::~WriteOpState() {
   Reset();
