@@ -25,6 +25,9 @@
 #include <mutex>
 #include <ostream>
 #include <string>
+#include <unordered_map>
+#include <unordered_set>
+#include <utility>
 #include <vector>
 
 #include <glog/logging.h>
@@ -54,6 +57,7 @@
 #include "kudu/util/status.h"
 
 namespace kudu {
+
 class AlterTableTest;
 class ConstContiguousRow;
 class EncodedKey;
@@ -66,6 +70,10 @@ class Timestamp;
 struct IterWithBounds;
 struct IteratorStats;
 
+namespace consensus {
+class OpId;
+}  // namespace consensus
+
 namespace clock {
 class Clock;
 } // namespace clock
@@ -80,12 +88,15 @@ class AlterSchemaOpState;
 class CompactionPolicy;
 class HistoryGcOpts;
 class MemRowSet;
+class ParticipantOpState;
 class RowSetTree;
 class RowSetsInCompaction;
+class TxnMetadata;
 class WriteOpState;
 struct RowOp;
 struct TabletComponents;
 struct TabletMetrics;
+struct TxnRowSets;
 
 class Tablet {
  public:
@@ -110,9 +121,14 @@ class Tablet {
 
   ~Tablet();
 
-  // Open the tablet.
+  // Open the tablet, initializing transactions for 'in_flight_txn_ids', and
+  // MRSs for 'txn_ids_with_mrs'. The created MRSs will be uncommitted -- it is
+  // up to the caller to determine whether they should be committed after
+  // finishing bootstrapping.
+  //
   // Upon completion, the tablet enters the kBootstrapping state.
-  Status Open();
+  Status Open(const std::unordered_set<int64_t>& in_flight_txn_ids = std::unordered_set<int64_t>{},
+              const std::unordered_set<int64_t>& txn_ids_with_mrs = std::unordered_set<int64_t>{});
 
   // Mark that the tablet has finished bootstrapping.
   // This transitions from kBootstrapping to kOpen state.
@@ -146,11 +162,16 @@ class Tablet {
   // This also sets the row op's RowSetKeyProbe.
   Status AcquireRowLocks(WriteOpState* op_state);
 
+  // Acquire a shared lock on the given transaction, to ensure the
+  // transaction's state doesn't change while the given write is in flight.
+  Status AcquireTxnLock(int64_t txn_id, WriteOpState* op_state);
+
   // Starts an MVCC op which must have a pre-assigned timestamp.
   //
   // TODO(todd): rename this to something like "FinishPrepare" or "StartApply", since
   // it's not the first thing in an op!
   void StartOp(WriteOpState* op_state);
+  void StartOp(ParticipantOpState* op_state);
 
   // Like the above but actually assigns the timestamp. Only used for tests that
   // don't boot a tablet server.
@@ -158,6 +179,7 @@ class Tablet {
 
   // Signal that the given op is about to Apply.
   void StartApplying(WriteOpState* op_state);
+  void StartApplying(ParticipantOpState* op_state);
 
   // Apply all of the row operations associated with this op.
   Status ApplyRowOperations(WriteOpState* op_state) WARN_UNUSED_RESULT;
@@ -169,6 +191,34 @@ class Tablet {
                            RowOp* row_op,
                            ProbeStats* stats) WARN_UNUSED_RESULT;
 
+  // Begins the transaction, recording its presence in the tablet metadata.
+  // Upon calling this, 'op_id' will be anchored until the metadata is flushed,
+  // using 'txn' as the anchor owner.
+  void BeginTransaction(Txn* txn, const consensus::OpId& op_id);
+
+  // Indicates that the transaction has started to commit, recording the
+  // timestamp used by the MVCC op to demarcate the end of the transaction in
+  // the tablet metadata. Upon calling this, 'op_id' will be anchored until
+  // the metadata is flushed, using 'txn' as the anchor owner.
+  void BeginCommit(Txn* txn, Timestamp mvcc_op_ts, const consensus::OpId& op_id);
+
+  // Commits the transaction, recording its commit timestamp in the tablet metadata.
+  // Upon calling this, 'op_id' will be anchored until the metadata is flushed,
+  // using 'txn' as the anchor owner.
+  void CommitTransaction(Txn* txn, Timestamp commit_ts, const consensus::OpId& op_id);
+
+  // Merges the uncommitted transaction rowsets associated with the given
+  // 'txn_id' with the committed rowsets.
+  void CommitTxnRowSets(int64_t txn_id);
+
+  // Aborts the transaction, recording the abort in the tablet metadata.
+  // Upon calling this, 'op_id' will be anchored until the metadata is flushed,
+  // using 'txn' as the anchor owner.
+  void AbortTransaction(Txn* txn, const consensus::OpId& op_id);
+
+  // Creates new rowsets for the given transaction.
+  void CreateTxnRowSets(int64_t txn_id, scoped_refptr<TxnMetadata> txn_meta);
+
   // Create a new row iterator which yields the rows as of the current MVCC
   // state of this tablet.
   // The returned iterator is not initialized.
@@ -268,11 +318,6 @@ class Tablet {
   // Same as MemRowSetEmpty(), but for the DMS.
   bool DeltaMemRowSetEmpty() const;
 
-  // Fills in the in-memory size and replay size in bytes for the DMS with the
-  // highest retention.
-  void GetInfoForBestDMSToFlush(const ReplaySizeMap& replay_size_map,
-                                int64_t* mem_size, int64_t* replay_size) const;
-
   // Flushes the DMS with the highest retention.
   Status FlushBestDMS(const ReplaySizeMap &replay_size_map) const;
 
@@ -476,6 +521,15 @@ class Tablet {
   // This method is not thread safe and should only be called from a single thread at once.
   double CollectAndUpdateWorkloadStats(MaintenanceOp::PerfImprovementOpType type);
 
+  // Returns the best DMS to flush, based on its memory size and retained
+  // bytes. Also returns the earliest creation time of a DMS seen. Note that
+  // 'mem_size' and 'replay_size' correspond to the same DMS but
+  // 'earliest_dms_time' may not.
+  std::shared_ptr<RowSet> FindBestDMSToFlush(const ReplaySizeMap& replay_size_map,
+                                             int64_t* mem_size = nullptr,
+                                             int64_t* replay_size = nullptr,
+                                             MonoTime* earliest_dms_time = nullptr) const;
+
  private:
   friend class kudu::AlterTableTest;
   friend class Iterator;
@@ -487,6 +541,7 @@ class Tablet {
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRangeWithOneRowSet);
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRangeWithNonOverlappingRowSets);
   FRIEND_TEST(TestTabletStringKey, TestSplitKeyRangeWithMinimumValueRowSet);
+  FRIEND_TEST(TxnParticipantTest, TestFlushMultipleMRSs);
 
   // Lifecycle states that a Tablet can be in. Legal state transitions for a
   // Tablet object:
@@ -561,10 +616,10 @@ class Tablet {
                                 RowOp* op,
                                 ProbeStats* stats);
 
-  // Same as above, but for UPDATE.
+  // Same as above, but for UPDATE, UPDATE_IGNORE, DELETE, or DELETE_IGNORE operations.
   Status MutateRowUnlocked(const fs::IOContext* io_context,
                            WriteOpState *op_state,
-                           RowOp* mutate,
+                           RowOp* op,
                            ProbeStats* stats);
 
   // In the case of an UPSERT against a duplicate row, converts the UPSERT
@@ -604,13 +659,15 @@ class Tablet {
 
   // Performs a merge compaction or a flush.
   Status DoMergeCompactionOrFlush(const RowSetsInCompaction &input,
-                                  int64_t mrs_being_flushed);
+                                  int64_t mrs_being_flushed,
+                                  const std::vector<TxnInfoBeingFlushed>& txns_being_flushed);
 
   // Handle the case in which a compaction or flush yielded no output rows.
   // In this case, we just need to remove the rowsets in 'rowsets' from the
   // metadata and flush it.
   Status HandleEmptyCompactionOrFlush(const RowSetVector& rowsets,
-                                      int mrs_being_flushed);
+                                      int mrs_being_flushed,
+                                      const std::vector<TxnInfoBeingFlushed>& txns_being_flushed);
 
   // Updates the average rowset height metric. Acquires the tablet's
   // compact_select_lock_.
@@ -618,7 +675,8 @@ class Tablet {
 
   Status FlushMetadata(const RowSetVector& to_remove,
                        const RowSetMetadataVector& to_add,
-                       int64_t mrs_being_flushed);
+                       int64_t mrs_being_flushed,
+                       const std::vector<TxnInfoBeingFlushed>& txns_being_flushed);
 
   static void ModifyRowSetTree(const RowSetTree& old_tree,
                                const RowSetVector& rowsets_to_remove,
@@ -645,17 +703,13 @@ class Tablet {
     *comps = components_;
   }
 
-  // Create a new MemRowSet, replacing the current one.
-  // The 'old_ms' pointer will be set to the current MemRowSet set before the replacement.
-  // If the MemRowSet is not empty it will be added to the 'compaction' input
-  // and the MemRowSet compaction lock will be taken to prevent the inclusion
-  // in any concurrent compactions.
-  Status ReplaceMemRowSetUnlocked(RowSetsInCompaction *compaction,
-                                  std::shared_ptr<MemRowSet> *old_ms);
-
-  // TODO: Document me.
-  Status FlushInternal(const RowSetsInCompaction& input,
-                       const std::shared_ptr<MemRowSet>& old_ms);
+  // Create a new MemRowSet, replacing the current committed one(s).
+  // 'old_mrss' will be populated to the current committed MemRowSet(s) set
+  // before the replacement. If any MemRowSet is not empty it will be added to
+  // the 'compaction' input and the MemRowSets' compaction locks will be taken
+  // to prevent the inclusion in any concurrent compactions.
+  Status ReplaceMemRowSetsUnlocked(RowSetsInCompaction* compaction,
+                                   std::vector<std::shared_ptr<MemRowSet>>* old_mrss);
 
   // Convert the specified read client schema (without IDs) to a server schema (with IDs)
   // This method is used by NewRowIterator().
@@ -664,9 +718,6 @@ class Tablet {
 
   Status CheckRowInTablet(const ConstContiguousRow& row) const;
 
-  // Helper method to find the rowset that has the DMS with the highest retention.
-  std::shared_ptr<RowSet> FindBestDMSToFlush(const ReplaySizeMap& replay_size_map) const;
-
   // Helper method to find how many bytes need to be replayed to restore in-memory
   // state from this index.
   static int64_t GetReplaySizeForIndex(int64_t min_log_index,
@@ -720,17 +771,16 @@ class Tablet {
   // is active, a writer comes along, then all future short readers will be blocked.
   mutable rw_spinlock component_lock_;
 
-  // The current components of the tablet. These should always be read
-  // or swapped under the component_lock.
+  // The components of the tablet whose base data has been committed. These
+  // should always be read or swapped under the component_lock.
   scoped_refptr<TabletComponents> components_;
 
+  // Uncommitted transaction state.
+  std::unordered_map<int64_t, scoped_refptr<TxnRowSets>> uncommitted_rowsets_by_txn_id_;
+
   scoped_refptr<log::LogAnchorRegistry> log_anchor_registry_;
   TabletMemTrackers mem_trackers_;
 
-  // Maintains the set of in-flight transactions, and any WAL anchors
-  // associated with them.
-  TxnParticipant txn_participant_;
-
   scoped_refptr<MetricEntity> metric_entity_;
   std::unique_ptr<TabletMetrics> metrics_;
 
@@ -742,6 +792,13 @@ class Tablet {
   clock::Clock* clock_;
 
   MvccManager mvcc_;
+
+  // Maintains the set of in-flight transactions, and any WAL anchors
+  // associated with them.
+  // NOTE: the participant may retain MVCC ops, so define it after the
+  // MvccManager, to ensure those ops get destructed before the MvccManager.
+  TxnParticipant txn_participant_;
+
   LockManager lock_manager_;
 
   std::unique_ptr<CompactionPolicy> compaction_policy_;
@@ -866,11 +923,29 @@ class Tablet::Iterator : public RowwiseIterator {
 // change.
 struct TabletComponents : public RefCountedThreadSafe<TabletComponents> {
   TabletComponents(std::shared_ptr<MemRowSet> mrs,
+                   std::vector<std::shared_ptr<MemRowSet>> txn_mrss,
                    std::shared_ptr<RowSetTree> rs_tree);
+  // The "main" MemRowSet that catches inserts to the tablet that are not a
+  // part of any transaction.
   const std::shared_ptr<MemRowSet> memrowset;
+
+  // MemRowSets whose insertion heads were inserted as a part of a transaction.
+  const std::vector<std::shared_ptr<MemRowSet>> txn_memrowsets;
+
+  // The persisted RowSets that comprise the rows of a tablet.
   const std::shared_ptr<RowSetTree> rowsets;
 };
 
+// Encapsulates data inserted as a part of a transaction that has not yet been
+// committed.
+// TODO(awong): when we support flushing transactional MRSs before committing,
+// track uncommitted disk rowsets here.
+struct TxnRowSets : public RefCountedThreadSafe<TxnRowSets> {
+  explicit TxnRowSets(std::shared_ptr<MemRowSet> mrs)
+      : memrowset(std::move(mrs)) {}
+  const std::shared_ptr<MemRowSet> memrowset;
+};
+
 } // namespace tablet
 } // namespace kudu
 
