@@ -25,6 +25,7 @@
 #include <ostream>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -130,6 +131,7 @@ using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
 using kudu::rpc::ResultTracker;
 using kudu::tserver::AlterSchemaRequestPB;
+using kudu::tserver::ParticipantOpPB;
 using kudu::tserver::WriteRequestPB;
 using kudu::tserver::WriteResponsePB;
 using std::map;
@@ -184,6 +186,23 @@ class FlushedStoresSnapshot {
 // rest of the consensus configuration, or it can start serving the data itself, after it
 // has been appointed LEADER of that particular consensus configuration.
 //
+// The high-level steps to replay the WAL are as follows:
+//  for each segment in the WAL:
+//    for each entry in the segment:
+//      - If the entry is a replicate message, keep track of it and write it to
+//        the new WAL, since we may find a corresponding commit message later.
+//      - If the entry is a commit message, determine whether or not we have a
+//        corresponding replicate message in the WAL, and if so, replay the op,
+//        skipping operations whose mem-stores have been persisted to disk. If
+//        no replicate message is found, we can skip replaying the op since its
+//        mutation has been persisted to disk.
+//  for each commit message with no corresponding replicate message:
+//    - Validate that the mutated stores are non-active.
+//  for each replicate message with no corresponding commit message:
+//    - Return the replicate message as an "orphaned replicate".
+//    - When the RaftConsensus instance starts up, orphaned replicates are
+//      initialized as follower ops.
+//
 // NOTE: this does not handle pulling data from other replicas in the cluster. That
 // is handled by the 'TabletCopy' classes, which copy blocks and metadata locally
 // before invoking this local bootstrap functionality to start the tablet.
@@ -426,8 +445,9 @@ class TabletBootstrap {
     // Number of REPLICATE messages for which a matching COMMIT was found.
     int ops_committed;
 
-    // Number inserts/mutations seen and ignored. Note inserts_ignored does not refer
-    // to the INSERT_IGNORE operation. It refers to inserts ignored during log replay.
+    // Number inserts/mutations seen and ignored. Note inserts_ignored and mutations_ignored
+    // do not refer to the INSERT_IGNORE, UPDATE_IGNORE, and DELETE_IGNORE operations.
+    // They refer to inserts and mutations ignored during log replay.
     int inserts_seen, inserts_ignored;
     int mutations_seen, mutations_ignored;
 
@@ -439,6 +459,14 @@ class TabletBootstrap {
   // Snapshot of which stores were flushed prior to restart.
   FlushedStoresSnapshot flushed_stores_;
 
+  // Transactions that were persisted as being in-flight (neither finalized or
+  // aborted) and completed prior to restart.
+  std::unordered_set<int64_t> in_flight_txn_ids_;
+  std::unordered_set<int64_t> terminal_txn_ids_;
+
+  // Transactions (committed or not) that have active MemRowSets.
+  std::unordered_set<int64_t> mrs_txn_ids_;
+
   DISALLOW_COPY_AND_ASSIGN(TabletBootstrap);
 };
 
@@ -587,6 +615,7 @@ Status TabletBootstrap::RunBootstrap(shared_ptr<Tablet>* rebuilt_tablet,
   }
 
   RETURN_NOT_OK(flushed_stores_.InitFrom(*tablet_meta_.get()));
+  tablet_meta_->GetTxnIds(&in_flight_txn_ids_, &terminal_txn_ids_, &mrs_txn_ids_);
 
   bool has_blocks;
   RETURN_NOT_OK(OpenTablet(&has_blocks));
@@ -646,7 +675,7 @@ Status TabletBootstrap::OpenTablet(bool* has_blocks) {
   // doing nothing for now except opening a tablet locally.
   {
     SCOPED_LOG_SLOW_EXECUTION_PREFIX(INFO, 100, LogPrefix(), "opening tablet");
-    RETURN_NOT_OK(tablet->Open());
+    RETURN_NOT_OK(tablet->Open(in_flight_txn_ids_, mrs_txn_ids_));
   }
   *has_blocks = tablet->num_rowsets() != 0;
   tablet_ = std::move(tablet);
@@ -1528,16 +1557,55 @@ Status TabletBootstrap::PlayChangeConfigRequest(const IOContext* /*io_context*/,
 Status TabletBootstrap::PlayTxnParticipantOpRequest(const IOContext* /*io_context*/,
                                                     ReplicateMsg* replicate_msg,
                                                     const CommitMsg& commit_msg) {
+  // When replaying participant ops:
+  // - If we've persisted completed transactions (i.e. aborted or committed),
+  //   we can skip replaying all participant ops for that transaction.
+  // - If we otherwise have persisted that a transaction exists, it was
+  //   persisted on-disk as being in-flight before restarting, we should have
+  //   opened a transaction before starting to replay, and we should replay all
+  //   participant ops for that transaction.
+  // - If we have no record of a transaction persisted, it was not persisted
+  //   on-disk as being in-flight, and we must replay all participant ops.
   ParticipantOpState op_state(tablet_replica_.get(),
                               tablet_->txn_participant(),
                               &replicate_msg->participant_request());
-  op_state.AcquireTxnAndLock();
-  SCOPED_CLEANUP({
-    op_state.ReleaseTxn();
-  });
-  // NOTE: don't bother validating the current state of the op. Presumably that
-  // happened the first time this op was written.
-  RETURN_NOT_OK(op_state.PerformOp(replicate_msg->id()));
+  const auto& op_type = op_state.request()->op().type();
+  const auto& txn_id = op_state.txn_id();
+  if (ContainsKey(terminal_txn_ids_, op_state.txn_id())) {
+    // Even if we're skipping over this participant op because its metadata
+    // state has been persisted, we still need to commit its rowsets if active,
+    // since we may have had to rebuild a committed MRS.
+    if (op_type == ParticipantOpPB::FINALIZE_COMMIT &&
+        ContainsKey(mrs_txn_ids_, txn_id)) {
+      tablet_->CommitTxnRowSets(txn_id);
+    }
+    return AppendCommitMsg(commit_msg);
+  }
+  bool persisted_in_flight = ContainsKey(in_flight_txn_ids_, op_state.txn_id());
+  if ((persisted_in_flight && op_type != ParticipantOpPB::BEGIN_TXN) ||
+      !persisted_in_flight) {
+    // If we're about to create a new MRS, add this transaction ID to the set
+    // that have active MRSs.
+    if (op_type == ParticipantOpPB::BEGIN_TXN) {
+      EmplaceOrDie(&mrs_txn_ids_, txn_id);
+    }
+    op_state.mutable_op_id()->CopyFrom(replicate_msg->id());
+    op_state.set_timestamp(Timestamp(replicate_msg->timestamp()));
+    op_state.AcquireTxnAndLock();
+    SCOPED_CLEANUP({
+      op_state.ReleaseTxn();
+    });
+    // Start an MVCC op to track the commit if we're beginning to commit.
+    tablet_->StartOp(&op_state);
+
+    // Start applying the commit's MVCC op if we're finalizing the commit.
+    // PerformOp() will take care of finishing applying the op.
+    tablet_->StartApplying(&op_state);
+
+    // NOTE: don't bother validating the current state of the op. Presumably that
+    // happened the first time this op was written.
+    RETURN_NOT_OK(op_state.PerformOp(replicate_msg->id(), tablet_.get()));
+  }
   return AppendCommitMsg(commit_msg);
 }
 
@@ -1591,7 +1659,9 @@ Status TabletBootstrap::ApplyOperations(const IOContext* io_context,
         break;
       }
       case RowOperationsPB::UPDATE:
-      case RowOperationsPB::DELETE: {
+      case RowOperationsPB::UPDATE_IGNORE:
+      case RowOperationsPB::DELETE:
+      case RowOperationsPB::DELETE_IGNORE: {
         stats_.mutations_seen++;
         if (op->has_result()) {
           stats_.mutations_ignored++;
@@ -1664,6 +1734,14 @@ Status TabletBootstrap::FilterOperation(const OperationResultPB& op_result,
   // output targets was active.
   int num_active_stores = 0;
   for (const MemStoreTargetPB& mutated_store : op_result.mutated_stores()) {
+    if (mutated_store.has_rs_txn_id()) {
+      // TODO(awong): once we begin flushing before commit, we'll need to have
+      // this account for a last_flushed_mrs_id per transaction.
+      if (ContainsKey(mrs_txn_ids_, mutated_store.rs_txn_id())) {
+        num_active_stores++;
+      }
+      continue;
+    }
     if (flushed_stores_.IsMemStoreActive(mutated_store)) {
       num_active_stores++;
     }
