@@ -23,13 +23,18 @@
 #include <ostream>
 #include <string>
 #include <type_traits>
+#include <unordered_map>
+#include <utility>
 
 #include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
+#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/schema.h"
+#include "kudu/common/timestamp.h"
 #include "kudu/common/wire_protocol.h"
+#include "kudu/consensus/log_anchor_registry.h"
 #include "kudu/consensus/opid.pb.h"
 #include "kudu/consensus/opid_util.h"
 #include "kudu/fs/block_id.h"
@@ -43,6 +48,8 @@
 #include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/rowset_metadata.h"
+#include "kudu/tablet/txn_metadata.h"
+#include "kudu/tablet/txn_participant.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/env.h"
 #include "kudu/util/flag_tags.h"
@@ -64,11 +71,14 @@ using kudu::consensus::MinimumOpId;
 using kudu::consensus::OpId;
 using kudu::fs::BlockManager;
 using kudu::fs::BlockDeletionTransaction;
+using kudu::log::MinLogIndexAnchorer;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
 using std::memory_order_relaxed;
 using std::shared_ptr;
 using std::string;
+using std::unordered_map;
+using std::unordered_set;
 using std::unique_ptr;
 using std::vector;
 using strings::Substitute;
@@ -488,6 +498,23 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
     if (superblock.has_table_type() && superblock.table_type() != TableTypePB::DEFAULT_TABLE) {
       table_type_ = superblock.table_type();
     }
+
+    std::unordered_map<int64_t, scoped_refptr<TxnMetadata>> txn_metas;
+    for (const auto& txn_id_and_metadata : superblock.txn_metadata()) {
+      const auto& txn_meta = txn_id_and_metadata.second;
+      EmplaceOrDie(&txn_metas, txn_id_and_metadata.first,
+          new TxnMetadata(
+              txn_meta.has_aborted() && txn_meta.aborted(),
+              txn_meta.has_commit_mvcc_op_timestamp() ?
+                  boost::make_optional(Timestamp(txn_meta.commit_mvcc_op_timestamp())) :
+                  boost::none,
+              txn_meta.has_commit_timestamp() ?
+                  boost::make_optional(Timestamp(txn_meta.commit_timestamp())) :
+                  boost::none,
+              txn_meta.has_flushed_committed_mrs() && txn_meta.flushed_committed_mrs()
+          ));
+    }
+    txn_metadata_by_txn_id_ = std::move(txn_metas);
   }
 
   // Now is a good time to clean up any orphaned blocks that may have been
@@ -501,10 +528,11 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
 
 Status TabletMetadata::UpdateAndFlush(const RowSetMetadataIds& to_remove,
                                       const RowSetMetadataVector& to_add,
-                                      int64_t last_durable_mrs_id) {
+                                      int64_t last_durable_mrs_id,
+                                      const vector<TxnInfoBeingFlushed>& txns_being_flushed) {
   {
     std::lock_guard<LockType> l(data_lock_);
-    RETURN_NOT_OK(UpdateUnlocked(to_remove, to_add, last_durable_mrs_id));
+    RETURN_NOT_OK(UpdateUnlocked(to_remove, to_add, last_durable_mrs_id, txns_being_flushed));
   }
   return Flush();
 }
@@ -573,6 +601,7 @@ Status TabletMetadata::Flush() {
   MutexLock l_flush(flush_lock_);
   BlockIdContainer orphaned;
   TabletSuperBlockPB pb;
+  vector<unique_ptr<MinLogIndexAnchorer>> anchors_needing_flush;
   {
     std::lock_guard<LockType> l(data_lock_);
     CHECK_GE(num_flush_pins_, 0);
@@ -592,12 +621,17 @@ Status TabletMetadata::Flush() {
     // want to accidentally delete those blocks before that next metadata update
     // is persisted. See KUDU-701 for details.
     orphaned.assign(orphaned_blocks_.begin(), orphaned_blocks_.end());
+    anchors_needing_flush = std::move(anchors_needing_flush_);
   }
   pre_flush_callback_();
   RETURN_NOT_OK(ReplaceSuperBlockUnlocked(pb));
   TRACE("Metadata flushed");
   l_flush.Unlock();
 
+  // Now that we've flushed, we can unanchor our WALs by destructing our
+  // anchors.
+  anchors_needing_flush.clear();
+
   // Now that the superblock is written, try to delete the orphaned blocks.
   //
   // If we crash just before the deletion, we'll retry when reloading from
@@ -610,13 +644,18 @@ Status TabletMetadata::Flush() {
 Status TabletMetadata::UpdateUnlocked(
     const RowSetMetadataIds& to_remove,
     const RowSetMetadataVector& to_add,
-    int64_t last_durable_mrs_id) {
+    int64_t last_durable_mrs_id,
+    const vector<TxnInfoBeingFlushed>& txns_being_flushed) {
   DCHECK(data_lock_.is_locked());
   CHECK_NE(state_, kNotLoadedYet);
   if (last_durable_mrs_id != kNoMrsFlushed) {
     DCHECK_GE(last_durable_mrs_id, last_durable_mrs_id_);
     last_durable_mrs_id_ = last_durable_mrs_id;
   }
+  for (const auto& txn_id : txns_being_flushed) {
+    auto txn_meta = FindOrDie(txn_metadata_by_txn_id_, txn_id);
+    txn_meta->set_flushed_committed_mrs_unlocked();
+  }
 
   RowSetMetadataVector new_rowsets = rowsets_;
   auto it = new_rowsets.begin();
@@ -706,6 +745,26 @@ Status TabletMetadata::ToSuperBlockUnlocked(TabletSuperBlockPB* super_block,
     meta->ToProtobuf(pb.add_rowsets());
   }
 
+  for (const auto& txn_id_and_metadata : txn_metadata_by_txn_id_) {
+    TxnMetadataPB meta_pb;
+    const auto& txn_meta = txn_id_and_metadata.second;
+    const auto& commit_ts = txn_meta->commit_timestamp();
+    if (commit_ts) {
+      meta_pb.set_commit_timestamp(commit_ts->value());
+    }
+    const auto& commit_mvcc_op_ts = txn_meta->commit_mvcc_op_timestamp();
+    if (commit_mvcc_op_ts) {
+      meta_pb.set_commit_mvcc_op_timestamp(commit_mvcc_op_ts->value());
+    }
+    if (txn_meta->aborted()) {
+      meta_pb.set_aborted(true);
+    }
+    if (txn_meta->flushed_committed_mrs_unlocked()) {
+      meta_pb.set_flushed_committed_mrs(true);
+    }
+    InsertOrDie(pb.mutable_txn_metadata(), txn_id_and_metadata.first, meta_pb);
+  }
+
   DCHECK(schema_->has_column_ids());
   RETURN_NOT_OK_PREPEND(SchemaToPB(*schema_, pb.mutable_schema()),
                         "Couldn't serialize schema into superblock");
@@ -754,6 +813,101 @@ Status TabletMetadata::CreateRowSet(shared_ptr<RowSetMetadata>* rowset) {
   return Status::OK();
 }
 
+void TabletMetadata::AddTxnMetadata(int64_t txn_id, unique_ptr<MinLogIndexAnchorer> log_anchor) {
+  std::lock_guard<LockType> l(data_lock_);
+  EmplaceOrDie(&txn_metadata_by_txn_id_, txn_id, new TxnMetadata());
+  anchors_needing_flush_.emplace_back(std::move(log_anchor));
+}
+
+void TabletMetadata::BeginCommitTransaction(int64_t txn_id, Timestamp mvcc_op_timestamp,
+                                            unique_ptr<MinLogIndexAnchorer> log_anchor) {
+  std::lock_guard<LockType> l(data_lock_);
+  auto txn_metadata = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
+  CHECK(txn_metadata);
+  // NOTE: we may already have an MVCC op timestamp if we are bootstrapping and
+  // the timestamp was persisted already, in which case, we don't need to
+  // anchor the WAL to ensure the timestamp's persistence in metadata.
+  if (!txn_metadata->commit_mvcc_op_timestamp()) {
+    txn_metadata->set_commit_mvcc_op_timestamp(mvcc_op_timestamp);
+    anchors_needing_flush_.emplace_back(std::move(log_anchor));
+  }
+}
+
+void TabletMetadata::AddCommitTimestamp(int64_t txn_id, Timestamp commit_timestamp,
+                                        unique_ptr<MinLogIndexAnchorer> log_anchor) {
+  std::lock_guard<LockType> l(data_lock_);
+  auto txn_metadata = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
+  CHECK(txn_metadata);
+  txn_metadata->set_commit_timestamp(commit_timestamp);
+  anchors_needing_flush_.emplace_back(std::move(log_anchor));
+}
+
+void TabletMetadata::AbortTransaction(int64_t txn_id, unique_ptr<MinLogIndexAnchorer> log_anchor) {
+  std::lock_guard<LockType> l(data_lock_);
+  auto txn_metadata = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
+  CHECK(txn_metadata);
+  txn_metadata->set_aborted();
+  anchors_needing_flush_.emplace_back(std::move(log_anchor));
+}
+
+bool TabletMetadata::HasTxnMetadata(int64_t txn_id, TxnState* state) {
+  std::lock_guard<LockType> l(data_lock_);
+  auto txn_meta = FindPtrOrNull(txn_metadata_by_txn_id_, txn_id);
+  if (txn_meta) {
+    if (!state) return true;
+    if (txn_meta->commit_timestamp()) {
+      *state = kCommitted;
+    } else if (txn_meta->aborted()) {
+      *state = kAborted;
+    } else if (txn_meta->commit_mvcc_op_timestamp()) {
+      *state = kCommitInProgress;
+    } else {
+      *state = kOpen;
+    }
+    return true;
+  }
+  return false;
+}
+
+void TabletMetadata::GetTxnIds(unordered_set<int64_t>* in_flight_txn_ids,
+                               unordered_set<int64_t>* terminal_txn_ids,
+                               unordered_set<int64_t>* txn_ids_with_mrs) {
+  std::unordered_set<int64_t> in_flights;
+  std::unordered_set<int64_t> terminals;
+  std::unordered_set<int64_t> needs_mrs;
+  std::lock_guard<LockType> l(data_lock_);
+  for (const auto& txn_id_and_metadata : txn_metadata_by_txn_id_) {
+    const auto& txn_id = txn_id_and_metadata.first;
+    const auto& txn_meta = txn_id_and_metadata.second;
+    if (txn_meta->commit_timestamp() || txn_meta->aborted()) {
+      if (terminal_txn_ids) {
+        EmplaceOrDie(&terminals, txn_id);
+      }
+    } else {
+      EmplaceOrDie(&in_flights, txn_id);
+    }
+    // If we have not flushed the MRS after committing, the bootstrap process
+    // will need to create an MRS for it, even if the transaction is committed.
+    if (txn_ids_with_mrs &&
+        !txn_meta->flushed_committed_mrs_unlocked() &&
+        !txn_meta->aborted()) {
+      EmplaceOrDie(&needs_mrs, txn_id);
+    }
+  }
+  *in_flight_txn_ids = std::move(in_flights);
+  if (terminal_txn_ids) {
+    *terminal_txn_ids = std::move(terminals);
+  }
+  if (txn_ids_with_mrs) {
+    *txn_ids_with_mrs = std::move(needs_mrs);
+  }
+}
+
+unordered_map<int64_t, scoped_refptr<TxnMetadata>> TabletMetadata::GetTxnMetadata() const {
+  std::lock_guard<LockType> l(data_lock_);
+  return txn_metadata_by_txn_id_;
+}
+
 const RowSetMetadata *TabletMetadata::GetRowSetForTests(int64_t id) const {
   for (const shared_ptr<RowSetMetadata>& rowset_meta : rowsets_) {
     if (rowset_meta->id() == id) {
