@@ -20,6 +20,7 @@
 #include <cstdint>
 #include <memory>
 #include <string>
+#include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
@@ -43,18 +44,31 @@ namespace kudu {
 class BlockIdPB;
 class FsManager;
 class Schema;
+class Timestamp;
 
 namespace consensus {
 class OpId;
-}
+} // namespace consensus
+
+namespace log {
+class MinLogIndexAnchorer;
+} // namespace log
 
 namespace tablet {
 
 class RowSetMetadata;
+class TxnMetadata;
+enum TxnState : int8_t;
 
 typedef std::vector<std::shared_ptr<RowSetMetadata> > RowSetMetadataVector;
 typedef std::unordered_set<int64_t> RowSetMetadataIds;
 
+// A transaction ID whose MRS is being flushed.
+// TODO(awong): If we begin supporting flushing before committing, we should
+// extend this to include the MRS ID so we can define some
+// 'last_durable_mrs_id' per transaction.
+typedef int64_t TxnInfoBeingFlushed;
+
 extern const int64_t kNoDurableMemStore;
 
 // Manages the "blocks tracking" for the specified tablet.
@@ -183,7 +197,8 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   static const int64_t kNoMrsFlushed = -1;
   Status UpdateAndFlush(const RowSetMetadataIds& to_remove,
                         const RowSetMetadataVector& to_add,
-                        int64_t last_durable_mrs_id);
+                        int64_t last_durable_mrs_id,
+                        const std::vector<TxnInfoBeingFlushed>& txns_being_flushed = {});
 
   // Adds the blocks referenced by 'block_ids' to 'orphaned_blocks_'.
   //
@@ -230,6 +245,42 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // calls to do so.
   Status CreateRowSet(std::shared_ptr<RowSetMetadata>* rowset);
 
+  // Records the fact that the given transaction ID has been started, adopting
+  // the anchor until the metadata is flushed. The transaction must not already
+  // have metadata associated with it.
+  void AddTxnMetadata(int64_t txn_id, std::unique_ptr<log::MinLogIndexAnchorer> log_anchor);
+
+  // Records the fact that the given transaction has started committing, with
+  // the given timestamp as the end of its open window, adopting the anchor
+  // until the metadata is flushed. The transaction must already have metadata
+  // associated with it.
+  void BeginCommitTransaction(int64_t txn_id, Timestamp mvcc_op_timestamp,
+                              std::unique_ptr<log::MinLogIndexAnchorer> log_anchor);
+
+  // Records the fact that the transaction was committed at the given
+  // timestamp, adopting the anchor until the metadata is flushed. The
+  // transaction must already have metadata associated with it.
+  //
+  // TODO(awong): implement a way to remove timstamps once all relevant
+  // transactional mutations have been merged with the rest of the tablet.
+  void AddCommitTimestamp(int64_t txn_id, Timestamp commit_timestamp,
+                          std::unique_ptr<log::MinLogIndexAnchorer> log_anchor);
+
+  // Adds an abort bit to the transaction metadata, adopting the anchor until
+  // the metadata is flushed. The transaction must already have metadata
+  // associated with it.
+  void AbortTransaction(int64_t txn_id, std::unique_ptr<log::MinLogIndexAnchorer> log_anchor);
+
+  // Returns whether a given transaction has metadata, and if requested, what
+  // state the transaction is in.
+  bool HasTxnMetadata(int64_t txn_id, TxnState* state = nullptr);
+
+  // Returns the transaction IDs that were persisted as being in-flight,
+  // terminal (committed or aborted), and having un-flushed MRSs.
+  void GetTxnIds(std::unordered_set<int64_t>* in_flight_txn_ids,
+                 std::unordered_set<int64_t>* terminal_txn_ids = nullptr,
+                 std::unordered_set<int64_t>* txn_ids_with_mrs = nullptr);
+
   const RowSetMetadataVector& rowsets() const { return rowsets_; }
 
   FsManager *fs_manager() const { return fs_manager_; }
@@ -272,6 +323,8 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
 
   RowSetMetadata *GetRowSetForTests(int64_t id);
 
+  std::unordered_map<int64_t, scoped_refptr<TxnMetadata>> GetTxnMetadata() const;
+
   // Return standard "T xxx P yyy" log prefix.
   std::string LogPrefix() const;
 
@@ -328,7 +381,8 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // Requires 'data_lock_'.
   Status UpdateUnlocked(const RowSetMetadataIds& to_remove,
                         const RowSetMetadataVector& to_add,
-                        int64_t last_durable_mrs_id);
+                        int64_t last_durable_mrs_id,
+                        const std::vector<TxnInfoBeingFlushed>& txns_being_flushed);
 
   // Requires 'data_lock_'.
   Status ToSuperBlockUnlocked(TabletSuperBlockPB* super_block,
@@ -372,6 +426,13 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
 
   int64_t last_durable_mrs_id_;
 
+  // Log anchors that are waiting on a metadata flush to be unanchored.
+  std::vector<std::unique_ptr<log::MinLogIndexAnchorer>> anchors_needing_flush_;
+
+  // Commit timestamps for transactions whose mutations have not yet been
+  // merged with main state.
+  std::unordered_map<int64_t, scoped_refptr<TxnMetadata>> txn_metadata_by_txn_id_;;
+
   // The current schema version. This is owned by this class.
   // We don't use unique_ptr so that we can do an atomic swap.
   Schema* schema_;
