@@ -36,45 +36,36 @@
 #include "kudu/common/wire_protocol.h"
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/consensus.pb.h"
-#include "kudu/consensus/consensus_meta.h"
-#include "kudu/consensus/consensus_meta_manager.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log_anchor_registry.h"
 #include "kudu/consensus/log_reader.h"
 #include "kudu/consensus/log_util.h"
-#include "kudu/consensus/metadata.pb.h"
 #include "kudu/consensus/opid.pb.h"
 #include "kudu/consensus/opid_util.h"
 #include "kudu/consensus/raft_consensus.h"
 #include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/rpc/messenger.h"
-#include "kudu/rpc/result_tracker.h"
-#include "kudu/tablet/tablet-harness.h"
-#include "kudu/tablet/tablet-test-util.h"
+#include "kudu/tablet/ops/alter_schema_op.h"
+#include "kudu/tablet/ops/op.h"
+#include "kudu/tablet/ops/op_driver.h"
+#include "kudu/tablet/ops/op_tracker.h"
+#include "kudu/tablet/ops/write_op.h"
 #include "kudu/tablet/tablet.h"
-#include "kudu/tablet/tablet_bootstrap.h"
 #include "kudu/tablet/tablet_metadata.h"
+#include "kudu/tablet/tablet_replica-test-base.h"
 #include "kudu/tablet/tablet_replica_mm_ops.h"
-#include "kudu/tablet/transactions/alter_schema_transaction.h"
-#include "kudu/tablet/transactions/transaction.h"
-#include "kudu/tablet/transactions/transaction_driver.h"
-#include "kudu/tablet/transactions/transaction_tracker.h"
-#include "kudu/tablet/transactions/write_transaction.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/tserver/tserver_admin.pb.h"
 #include "kudu/util/countdown_latch.h"
 #include "kudu/util/maintenance_manager.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/net/dns_resolver.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/random.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
-#include "kudu/util/threadpool.h"
 
 DECLARE_bool(enable_maintenance_manager);
 DECLARE_int32(flush_threshold_mb);
@@ -86,19 +77,12 @@ METRIC_DECLARE_gauge_uint64(live_row_count);
 
 using kudu::consensus::CommitMsg;
 using kudu::consensus::ConsensusBootstrapInfo;
-using kudu::consensus::ConsensusMetadata;
-using kudu::consensus::ConsensusMetadataManager;
 using kudu::consensus::OpId;
 using kudu::consensus::RECEIVED_OPID;
-using kudu::consensus::RaftConfigPB;
 using kudu::consensus::RaftConsensus;
-using kudu::consensus::RaftPeerPB;
 using kudu::log::Log;
-using kudu::log::LogOptions;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
-using kudu::rpc::Messenger;
-using kudu::rpc::ResultTracker;
 using kudu::tserver::AlterSchemaRequestPB;
 using kudu::tserver::AlterSchemaResponsePB;
 using kudu::tserver::WriteRequestPB;
@@ -108,147 +92,19 @@ using std::string;
 using std::unique_ptr;
 
 namespace kudu {
+
 namespace tablet {
 
 static Schema GetTestSchema() {
   return Schema({ ColumnSchema("key", INT32) }, 1);
 }
 
-class TabletReplicaTest : public KuduTabletTest {
+class TabletReplicaTest : public TabletReplicaTestBase {
  public:
   TabletReplicaTest()
-      : KuduTabletTest(GetTestSchema(),
-                       TabletHarness::Options::ClockType::HYBRID_CLOCK),
+      : TabletReplicaTestBase(GetTestSchema()),
         insert_counter_(0),
-        delete_counter_(0),
-        dns_resolver_(new DnsResolver) {
-  }
-
-  void SetUpReplica(bool new_replica = true) {
-    ASSERT_TRUE(tablet_replica_.get() == nullptr);
-
-    RaftConfigPB config;
-    config.set_opid_index(consensus::kInvalidOpIdIndex);
-
-    RaftPeerPB* config_peer = config.add_peers();
-    config_peer->set_permanent_uuid(tablet()->metadata()->fs_manager()->uuid());
-    config_peer->mutable_last_known_addr()->set_host("0.0.0.0");
-    config_peer->mutable_last_known_addr()->set_port(0);
-    config_peer->set_member_type(RaftPeerPB::VOTER);
-
-
-    if (new_replica) {
-      ASSERT_OK(cmeta_manager_->Create(tablet()->tablet_id(), config, consensus::kMinimumTerm));
-    }
-
-    // "Bootstrap" and start the TabletReplica.
-    const auto& tablet_id = tablet()->tablet_id();
-    tablet_replica_.reset(
-      new TabletReplica(tablet()->shared_metadata(),
-                        cmeta_manager_,
-                        *config_peer,
-                        apply_pool_.get(),
-                        [this, tablet_id](const string& reason) {
-                          this->TabletReplicaStateChangedCallback(tablet_id, reason);
-                        }));
-    ASSERT_OK(tablet_replica_->Init({ /*quiescing*/nullptr,
-                                      /*num_leaders*/nullptr,
-                                      raft_pool_.get() }));
-    // Make TabletReplica use the same LogAnchorRegistry as the Tablet created by the harness.
-    // TODO(mpercy): Refactor TabletHarness to allow taking a
-    // LogAnchorRegistry, while also providing TabletMetadata for consumption
-    // by TabletReplica before Tablet is instantiated.
-    tablet_replica_->log_anchor_registry_ = tablet()->log_anchor_registry_;
-  }
-
-  virtual void SetUp() override {
-    KuduTabletTest::SetUp();
-
-    ASSERT_OK(ThreadPoolBuilder("prepare").Build(&prepare_pool_));
-    ASSERT_OK(ThreadPoolBuilder("apply").Build(&apply_pool_));
-    ASSERT_OK(ThreadPoolBuilder("raft").Build(&raft_pool_));
-
-    rpc::MessengerBuilder builder(CURRENT_TEST_NAME());
-    ASSERT_OK(builder.Build(&messenger_));
-
-    cmeta_manager_.reset(new ConsensusMetadataManager(fs_manager()));
-
-    metric_entity_ = METRIC_ENTITY_tablet.Instantiate(&metric_registry_, "test-tablet");
-    NO_FATALS(SetUpReplica());
-  }
-
-  Status StartReplica(const ConsensusBootstrapInfo& info) {
-    scoped_refptr<Log> log;
-    RETURN_NOT_OK(Log::Open(LogOptions(),
-                            fs_manager(),
-                            /*file_cache*/nullptr,
-                            tablet()->tablet_id(),
-                            *tablet()->schema(),
-                            tablet()->metadata()->schema_version(),
-                            metric_entity_.get(),
-                            &log));
-    tablet_replica_->SetBootstrapping();
-    return tablet_replica_->Start(info,
-                                  tablet(),
-                                  clock(),
-                                  messenger_,
-                                  scoped_refptr<ResultTracker>(),
-                                  log,
-                                  prepare_pool_.get(),
-                                  dns_resolver_.get());
-  }
-
-  Status StartReplicaAndWaitUntilLeader(const ConsensusBootstrapInfo& info) {
-    RETURN_NOT_OK(StartReplica(info));
-    const MonoDelta kTimeout = MonoDelta::FromSeconds(10);
-    return tablet_replica_->consensus()->WaitUntilLeaderForTests(kTimeout);
-  }
-
-  void TabletReplicaStateChangedCallback(const string& tablet_id, const string& reason) {
-    LOG(INFO) << "Tablet replica state changed for tablet " << tablet_id << ". Reason: " << reason;
-  }
-
-  virtual void TearDown() override {
-    tablet_replica_->Shutdown();
-    prepare_pool_->Shutdown();
-    apply_pool_->Shutdown();
-    KuduTabletTest::TearDown();
-  }
-
-  void RestartReplica() {
-    tablet_replica_->Shutdown();
-    tablet_replica_.reset();
-    NO_FATALS(SetUpReplica(/*new_replica=*/ false));
-    scoped_refptr<ConsensusMetadata> cmeta;
-    ASSERT_OK(cmeta_manager_->Load(tablet_replica_->tablet_id(), &cmeta));
-    shared_ptr<Tablet> tablet;
-    scoped_refptr<Log> log;
-    ConsensusBootstrapInfo bootstrap_info;
-
-    tablet_replica_->SetBootstrapping();
-    ASSERT_OK(BootstrapTablet(tablet_replica_->tablet_metadata(),
-                              cmeta->CommittedConfig(),
-                              clock(),
-                              /*mem_tracker*/nullptr,
-                              /*result_tracker*/nullptr,
-                              &metric_registry_,
-                              /*file_cache*/nullptr,
-                              tablet_replica_,
-                              tablet_replica_->log_anchor_registry(),
-                              &tablet,
-                              &log,
-                              &bootstrap_info));
-    ASSERT_OK(tablet_replica_->Start(bootstrap_info,
-                                     tablet,
-                                     clock(),
-                                     messenger_,
-                                     scoped_refptr<ResultTracker>(),
-                                     log,
-                                     prepare_pool_.get(),
-                                     dns_resolver_.get()));
-    // Wait for the replica to be usable.
-    const MonoDelta kTimeout = MonoDelta::FromSeconds(30);
-    ASSERT_OK(tablet_replica_->consensus()->WaitUntilLeaderForTests(kTimeout));
+        delete_counter_(0) {
   }
 
  protected:
@@ -284,24 +140,6 @@ class TabletReplicaTest : public KuduTabletTest {
     return Status::OK();
   }
 
-  Status ExecuteWrite(TabletReplica* replica, const WriteRequestPB& req) {
-    unique_ptr<WriteResponsePB> resp(new WriteResponsePB());
-    unique_ptr<WriteTransactionState> tx_state(new WriteTransactionState(replica,
-                                                                         &req,
-                                                                         nullptr, // No RequestIdPB
-                                                                         resp.get()));
-
-    CountDownLatch rpc_latch(1);
-    tx_state->set_completion_callback(unique_ptr<TransactionCompletionCallback>(
-        new LatchTransactionCompletionCallback<WriteResponsePB>(&rpc_latch, resp.get())));
-
-    RETURN_NOT_OK(replica->SubmitWrite(std::move(tx_state)));
-    rpc_latch.Wait();
-    CHECK(!resp->has_error())
-        << "\nReq:\n" << SecureDebugString(req) << "Resp:\n" << SecureDebugString(*resp);
-    return Status::OK();
-  }
-
   Status UpdateSchema(const SchemaPB& schema, int schema_version) {
     AlterSchemaRequestPB alter;
     alter.set_dest_uuid(tablet()->metadata()->fs_manager()->uuid());
@@ -313,12 +151,12 @@ class TabletReplicaTest : public KuduTabletTest {
 
   Status ExecuteAlter(TabletReplica* replica, const AlterSchemaRequestPB& req) {
     unique_ptr<AlterSchemaResponsePB> resp(new AlterSchemaResponsePB());
-    unique_ptr<AlterSchemaTransactionState> tx_state(
-        new AlterSchemaTransactionState(replica, &req, resp.get()));
+    unique_ptr<AlterSchemaOpState> op_state(
+        new AlterSchemaOpState(replica, &req, resp.get()));
     CountDownLatch rpc_latch(1);
-    tx_state->set_completion_callback(unique_ptr<TransactionCompletionCallback>(
-          new LatchTransactionCompletionCallback<AlterSchemaResponsePB>(&rpc_latch, resp.get())));
-    RETURN_NOT_OK(replica->SubmitAlterSchema(std::move(tx_state)));
+    op_state->set_completion_callback(unique_ptr<OpCompletionCallback>(
+          new LatchOpCompletionCallback<AlterSchemaResponsePB>(&rpc_latch, resp.get())));
+    RETURN_NOT_OK(replica->SubmitAlterSchema(std::move(op_state)));
     rpc_latch.Wait();
     CHECK(!resp->has_error())
         << "\nReq:\n" << SecureDebugString(req) << "Resp:\n" << SecureDebugString(*resp);
@@ -364,12 +202,12 @@ class TabletReplicaTest : public KuduTabletTest {
 
   // Assert that there are no log anchors held on the tablet replica.
   //
-  // NOTE: when a transaction finishes and notifies the completion callback, it still is
-  // registered with the transaction tracker for a very short time before being
+  // NOTE: when an op finishes and notifies the completion callback, it still is
+  // registered with the op tracker for a very short time before being
   // destructed. So, this should always be called with an ASSERT_EVENTUALLY wrapper.
   void AssertNoLogAnchors() {
     // Make sure that there are no registered anchors in the registry
-    ASSERT_EQ(0, tablet_replica_->log_anchor_registry()->GetAnchorCountForTests());
+    ASSERT_EQ(0, tablet_replica()->log_anchor_registry()->GetAnchorCountForTests());
   }
 
   // Assert that the Log GC() anchor is earlier than the latest OpId in the Log.
@@ -388,43 +226,31 @@ class TabletReplicaTest : public KuduTabletTest {
 
   int32_t insert_counter_;
   int32_t delete_counter_;
-  MetricRegistry metric_registry_;
-  scoped_refptr<MetricEntity> metric_entity_;
-  shared_ptr<Messenger> messenger_;
-  unique_ptr<ThreadPool> prepare_pool_;
-  unique_ptr<ThreadPool> apply_pool_;
-  unique_ptr<ThreadPool> raft_pool_;
-  unique_ptr<DnsResolver> dns_resolver_;
-
-  scoped_refptr<ConsensusMetadataManager> cmeta_manager_;
-
-  // Must be destroyed before thread pools.
-  scoped_refptr<TabletReplica> tablet_replica_;
 };
 
-// A Transaction that waits on the apply_continue latch inside of Apply().
-class DelayedApplyTransaction : public WriteTransaction {
+// A Op that waits on the apply_continue latch inside of Apply().
+class DelayedApplyOp : public WriteOp {
  public:
-  DelayedApplyTransaction(CountDownLatch* apply_started,
-                          CountDownLatch* apply_continue,
-                          unique_ptr<WriteTransactionState> state)
-      : WriteTransaction(std::move(state), consensus::LEADER),
+  DelayedApplyOp(CountDownLatch* apply_started,
+                 CountDownLatch* apply_continue,
+                 unique_ptr<WriteOpState> state)
+      : WriteOp(std::move(state), consensus::LEADER),
         apply_started_(DCHECK_NOTNULL(apply_started)),
         apply_continue_(DCHECK_NOTNULL(apply_continue)) {
   }
 
-  virtual Status Apply(unique_ptr<CommitMsg>* commit_msg) override {
+  virtual Status Apply(CommitMsg** commit_msg) override {
     apply_started_->CountDown();
     LOG(INFO) << "Delaying apply...";
     apply_continue_->Wait();
     LOG(INFO) << "Apply proceeding";
-    return WriteTransaction::Apply(commit_msg);
+    return WriteOp::Apply(commit_msg);
   }
 
  private:
   CountDownLatch* apply_started_;
   CountDownLatch* apply_continue_;
-  DISALLOW_COPY_AND_ASSIGN(DelayedApplyTransaction);
+  DISALLOW_COPY_AND_ASSIGN(DelayedApplyOp);
 };
 
 // Ensure that Log::GC() doesn't delete logs when the MRS has an anchor.
@@ -547,8 +373,8 @@ TEST_F(TabletReplicaTest, TestDMSAnchorPreventsLogGC) {
   ASSERT_EQ(2, segments.size());
 }
 
-// Ensure that Log::GC() doesn't compact logs with OpIds of active transactions.
-TEST_F(TabletReplicaTest, TestActiveTransactionPreventsLogGC) {
+// Ensure that Log::GC() doesn't compact logs with OpIds of active ops.
+TEST_F(TabletReplicaTest, TestActiveOpPreventsLogGC) {
   ConsensusBootstrapInfo info;
   ASSERT_OK(StartReplicaAndWaitUntilLeader(info));
 
@@ -572,9 +398,9 @@ TEST_F(TabletReplicaTest, TestActiveTransactionPreventsLogGC) {
   // Verify no anchors after Flush().
   ASSERT_EVENTUALLY([&]{ AssertNoLogAnchors(); });
 
-  // Now create a long-lived Transaction that hangs during Apply().
-  // Allow other transactions to go through. Logs should be populated, but the
-  // long-lived Transaction should prevent the log from being deleted since it
+  // Now create a long-lived op that hangs during Apply().
+  // Allow other ops to go through. Logs should be populated, but the
+  // long-lived op should prevent the log from being deleted since it
   // is in-flight.
   CountDownLatch rpc_latch(1);
   CountDownLatch apply_started(1);
@@ -584,33 +410,33 @@ TEST_F(TabletReplicaTest, TestActiveTransactionPreventsLogGC) {
   {
     // Long-running mutation.
     ASSERT_OK(GenerateSequentialDeleteRequest(req.get()));
-    unique_ptr<WriteTransactionState> tx_state(new WriteTransactionState(tablet_replica_.get(),
-                                                                         req.get(),
-                                                                         nullptr, // No RequestIdPB
-                                                                         resp.get()));
+    unique_ptr<WriteOpState> op_state(new WriteOpState(tablet_replica_.get(),
+                                                       req.get(),
+                                                       nullptr, // No RequestIdPB
+                                                       resp.get()));
 
-    tx_state->set_completion_callback(unique_ptr<TransactionCompletionCallback>(
-        new LatchTransactionCompletionCallback<WriteResponsePB>(&rpc_latch, resp.get())));
+    op_state->set_completion_callback(unique_ptr<OpCompletionCallback>(
+        new LatchOpCompletionCallback<WriteResponsePB>(&rpc_latch, resp.get())));
 
-    unique_ptr<DelayedApplyTransaction> transaction(
-        new DelayedApplyTransaction(&apply_started,
-                                    &apply_continue,
-                                    std::move(tx_state)));
+    unique_ptr<DelayedApplyOp> op(
+        new DelayedApplyOp(&apply_started,
+                           &apply_continue,
+                           std::move(op_state)));
 
-    scoped_refptr<TransactionDriver> driver;
-    ASSERT_OK(tablet_replica_->NewLeaderTransactionDriver(std::move(transaction),
-                                                          &driver));
+    scoped_refptr<OpDriver> driver;
+    ASSERT_OK(tablet_replica_->NewLeaderOpDriver(std::move(op),
+                                                 &driver));
 
     ASSERT_OK(driver->ExecuteAsync());
     apply_started.Wait();
     ASSERT_TRUE(driver->GetOpId().IsInitialized())
-      << "By the time a transaction is applied, it should have an Opid";
+      << "By the time an op is applied, it should have an Opid";
     // The apply will hang until we CountDown() the continue latch.
     // Now, roll the log. Below, we execute a few more insertions with rolling.
     ASSERT_OK(log->AllocateSegmentAndRollOverForTests());
   }
 
-  ASSERT_EQ(1, tablet_replica_->txn_tracker_.GetNumPendingForTests());
+  ASSERT_EQ(1, tablet_replica_->op_tracker_.GetNumPendingForTests());
   // The log anchor is currently equal to the latest OpId written to the Log
   // because we are delaying the Commit message with the CountDownLatch.
 
@@ -622,7 +448,7 @@ TEST_F(TabletReplicaTest, TestActiveTransactionPreventsLogGC) {
   ASSERT_EQ(2, segments.size());
 
   // We use mutations here, since an MRS Flush() quiesces the tablet, and we
-  // want to ensure the only thing "anchoring" is the TransactionTracker.
+  // want to ensure the only thing "anchoring" is the OpTracker.
   ASSERT_OK(ExecuteDeletesAndRollLogs(3));
   log->reader()->GetSegmentsSnapshot(&segments);
   ASSERT_EQ(5, segments.size());
@@ -631,25 +457,25 @@ TEST_F(TabletReplicaTest, TestActiveTransactionPreventsLogGC) {
 
   ASSERT_EVENTUALLY([&]{
       AssertNoLogAnchors();
-      ASSERT_EQ(1, tablet_replica_->txn_tracker_.GetNumPendingForTests());
+      ASSERT_EQ(1, tablet_replica_->op_tracker_.GetNumPendingForTests());
     });
 
   NO_FATALS(AssertLogAnchorEarlierThanLogLatest());
 
-  // Try to GC(), nothing should be deleted due to the in-flight transaction.
+  // Try to GC(), nothing should be deleted due to the in-flight op.
   retention = tablet_replica_->GetRetentionIndexes();
   ASSERT_OK(log->GC(retention, &num_gced));
   ASSERT_EQ(0, num_gced);
   log->reader()->GetSegmentsSnapshot(&segments);
   ASSERT_EQ(5, segments.size());
 
-  // Now we release the transaction and wait for everything to complete.
+  // Now we release the op and wait for everything to complete.
   // We fully quiesce and flush, which should release all anchors.
-  ASSERT_EQ(1, tablet_replica_->txn_tracker_.GetNumPendingForTests());
+  ASSERT_EQ(1, tablet_replica_->op_tracker_.GetNumPendingForTests());
   apply_continue.CountDown();
   rpc_latch.Wait();
-  tablet_replica_->txn_tracker_.WaitForAllToFinish();
-  ASSERT_EQ(0, tablet_replica_->txn_tracker_.GetNumPendingForTests());
+  tablet_replica_->op_tracker_.WaitForAllToFinish();
+  ASSERT_EQ(0, tablet_replica_->op_tracker_.GetNumPendingForTests());
   tablet_replica_->tablet()->FlushBiggestDMS();
   ASSERT_EVENTUALLY([&]{ AssertNoLogAnchors(); });
 
@@ -673,16 +499,18 @@ TEST_F(TabletReplicaTest, TestFlushOpsPerfImprovements) {
 
   MaintenanceOpStats stats;
 
-  // Just on the threshold and not enough time has passed for a time-based flush.
+  // Just on the threshold and not enough time has passed for a time-based flush,
+  // we'll expect improvement equal to '1'.
   stats.set_ram_anchored(64 * 1024 * 1024);
   FlushOpPerfImprovementPolicy::SetPerfImprovementForFlush(&stats, 1);
-  ASSERT_EQ(0.0, stats.perf_improvement());
+  ASSERT_EQ(1.0, stats.perf_improvement());
   stats.Clear();
 
-  // Just on the threshold and enough time has passed, we'll have a low improvement.
-  stats.set_ram_anchored(64 * 1024 * 1024);
+  // Below the threshold and enough time has passed, we'll have a low improvement.
+  stats.set_ram_anchored(2 * 1024 * 1024);
   FlushOpPerfImprovementPolicy::SetPerfImprovementForFlush(&stats, 3 * 60 * 1000);
-  ASSERT_GT(stats.perf_improvement(), 0.01);
+  ASSERT_LT(0.01, stats.perf_improvement());
+  ASSERT_GT(0.1, stats.perf_improvement());
   stats.Clear();
 
   // Over the threshold, we expect improvement equal to the excess MB.
@@ -692,11 +520,19 @@ TEST_F(TabletReplicaTest, TestFlushOpsPerfImprovements) {
   stats.Clear();
 
   // Below the threshold but have been there a long time, closing in to 1.0.
-  stats.set_ram_anchored(30 * 1024 * 1024);
+  stats.set_ram_anchored(1);
   FlushOpPerfImprovementPolicy::SetPerfImprovementForFlush(&stats, 60 * 50 * 1000);
   ASSERT_LT(0.7, stats.perf_improvement());
   ASSERT_GT(1.0, stats.perf_improvement());
   stats.Clear();
+
+  // Approaching threshold, enough time has passed but haven't been there a long time,
+  // closing in to 1.0.
+  stats.set_ram_anchored(63 * 1024 * 1024);
+  FlushOpPerfImprovementPolicy::SetPerfImprovementForFlush(&stats, 3 * 60 * 1000);
+  ASSERT_LT(0.9, stats.perf_improvement());
+  ASSERT_GT(1.0, stats.perf_improvement());
+  stats.Clear();
 }
 
 // Test that the schema of a tablet will be rolled forward upon replaying an
@@ -723,7 +559,7 @@ TEST_F(TabletReplicaTest, TestRollLogSegmentSchemaOnAlter) {
   };
   // Upon restarting, our log segment header schema should have "new_col".
   NO_FATALS(write());
-  NO_FATALS(RestartReplica());
+  ASSERT_OK(RestartReplica());
 
   // Get rid of the alter in the WALs.
   NO_FATALS(write());
@@ -736,7 +572,7 @@ TEST_F(TabletReplicaTest, TestRollLogSegmentSchemaOnAlter) {
   // didn't have "new_col", bootstrapping would fail, complaining about a
   // mismatch between the segment header schema and the write request schema.
   NO_FATALS(write());
-  NO_FATALS(RestartReplica());
+  ASSERT_OK(RestartReplica());
 }
 
 // Regression test for KUDU-2690, wherein a alter schema request that failed
@@ -775,7 +611,7 @@ TEST_F(TabletReplicaTest, Kudu2690Test) {
   // Before KUDU-2960 was fixed, bootstrapping would fail, complaining that the
   // write requests contained a column that was not in the log segment header's
   // schema.
-  NO_FATALS(RestartReplica());
+  ASSERT_OK(RestartReplica());
 }
 
 TEST_F(TabletReplicaTest, TestLiveRowCountMetric) {
@@ -833,7 +669,7 @@ TEST_F(TabletReplicaTest, TestRestartAfterGCDeletedRowsets) {
   ASSERT_EQ(0, live_row_count->value());
 
   // Restart and ensure we can rebuild our DMS okay.
-  NO_FATALS(RestartReplica());
+  ASSERT_OK(RestartReplica());
   tablet = tablet_replica_->tablet();
   ASSERT_EQ(1, tablet->num_rowsets());
   live_row_count = METRIC_live_row_count.InstantiateFunctionGauge(
@@ -842,7 +678,7 @@ TEST_F(TabletReplicaTest, TestRestartAfterGCDeletedRowsets) {
 
   // Now do that again but with deltafiles.
   ASSERT_OK(tablet->FlushBiggestDMS());
-  NO_FATALS(RestartReplica());
+  ASSERT_OK(RestartReplica());
   tablet = tablet_replica_->tablet();
   ASSERT_EQ(1, tablet->num_rowsets());
 
