@@ -29,10 +29,10 @@
 #include <stack>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <boost/algorithm/string/predicate.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 // IWYU pragma: no_include <yaml-cpp/node/impl.h>
@@ -41,6 +41,7 @@
 #include "kudu/client/client-internal.h"  // IWYU pragma: keep
 #include "kudu/client/client.h"
 #include "kudu/client/master_proxy_rpc.h"
+#include "kudu/client/replica_controller-internal.h"
 #include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/common/common.pb.h"
 #include "kudu/common/row_operations.h"
@@ -59,6 +60,7 @@
 #include "kudu/gutil/strings/stringpiece.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/strings/util.h"
+#include "kudu/master/master.h"
 #include "kudu/master/master.proxy.h" // IWYU pragma: keep
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/response_callback.h"
@@ -74,6 +76,7 @@
 #include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/async_util.h"
 #include "kudu/util/env.h"
+#include "kudu/util/flag_validators.h"
 #include "kudu/util/jsonwriter.h"
 #include "kudu/util/mem_tracker.pb.h"
 #include "kudu/util/memory/arena.h"
@@ -83,6 +86,7 @@
 #include "kudu/util/path_util.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/status.h"
+#include "kudu/util/string_case.h"
 #include "kudu/util/yamlreader.h"
 
 DEFINE_bool(force, false, "If true, allows the set_flag command to set a flag "
@@ -138,9 +142,27 @@ static bool ValidateNumThreads(const char* flag_name, int32_t flag_value) {
 }
 DEFINE_validator(num_threads, &ValidateNumThreads);
 
+DEFINE_int64(negotiation_timeout_ms, 3000,
+             "Timeout for negotiating an RPC connection to a Kudu server, "
+             "in milliseconds");
+
+bool ValidateTimeoutSettings() {
+  if (FLAGS_timeout_ms < FLAGS_negotiation_timeout_ms) {
+    LOG(ERROR) << strings::Substitute(
+        "RPC timeout set by --timeout_ms should be not less than connection "
+        "negotiation timeout set by --negotiation_timeout_ms; "
+        "current settings are $0 and $1 correspondingly",
+        FLAGS_timeout_ms, FLAGS_negotiation_timeout_ms);
+    return false;
+  }
+  return true;
+}
+GROUP_FLAG_VALIDATOR(timeout_flags, ValidateTimeoutSettings);
+
 using kudu::client::KuduClient;
 using kudu::client::KuduClientBuilder;
 using kudu::client::internal::AsyncLeaderMasterRpc;
+using kudu::client::internal::ReplicaController;
 using kudu::consensus::ConsensusServiceProxy; // NOLINT
 using kudu::consensus::ReplicateMsg;
 using kudu::log::LogEntryPB;
@@ -198,6 +220,16 @@ const char* const kTabletIdsCsvArg = "tablet_ids";
 const char* const kTabletIdsCsvArgDesc =
     "Comma-separated list of Tablet Identifiers";
 
+const char* const kMasterAddressArg = "master_address";
+const char* const kMasterAddressDesc = "Address of a Kudu Master of form "
+    "'hostname:port'. Port may be omitted if the Master is bound to the "
+    "default port.";
+
+const char* const kTServerAddressArg = "tserver_address";
+const char* const kTServerAddressDesc = "Address of a Kudu Tablet Server of "
+    "form 'hostname:port'. Port may be omitted if the Tablet Server is bound "
+    "to the default port.";
+
 namespace {
 
 enum PrintEntryType {
@@ -314,19 +346,92 @@ bool GetClusterName(const string& master_addresses_str, string* cluster_name) {
   return false;
 }
 
+// Retrieve flags from a remote server.
+//
+// If 'address' does not contain a port, 'default_port' is used instead.
+//
+// 'all_flags' controls whether all flags are returned, or only flags which are
+// explicitly set.
+//
+// 'flag_tags' is a comma-separated list of tags used to restrict which flags
+// are returned. An empty value matches all tags.
+Status GetServerFlags(const string& address,
+                      uint16_t default_port,
+                      bool all_flags,
+                      const string& flags_to_get,
+                      const string& flag_tags,
+                      vector<server::GetFlagsResponsePB_Flag>* flags) {
+  unique_ptr<GenericServiceProxy> proxy;
+  RETURN_NOT_OK(BuildProxy(address, default_port, &proxy));
+
+  GetFlagsRequestPB req;
+  GetFlagsResponsePB resp;
+  RpcController rpc;
+  rpc.set_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
+
+  req.set_all_flags(all_flags);
+  for (StringPiece tag : strings::Split(flag_tags, ",", strings::SkipEmpty())) {
+    req.add_tags(tag.as_string());
+  }
+  for (StringPiece flag: strings::Split(flags_to_get, ",", strings::SkipEmpty())) {
+    req.add_flags(flag.as_string());
+  }
+
+  RETURN_NOT_OK(proxy->GetFlags(req, &resp, &rpc));
+
+  flags->clear();
+  std::move(resp.flags().begin(), resp.flags().end(), std::back_inserter(*flags));
+  return Status::OK();
+}
+
 } // anonymous namespace
 
+RpcActionBuilder::RpcActionBuilder(std::string name, ActionRunner runner)
+    : ActionBuilder(std::move(name), std::move(runner)) {
+}
+
+unique_ptr<Action> RpcActionBuilder::Build() {
+  AddOptionalParameter("negotiation_timeout_ms");
+  AddOptionalParameter("timeout_ms");
+  return ActionBuilder::Build();
+}
+
+ClusterActionBuilder::ClusterActionBuilder(std::string name, ActionRunner runner)
+    : RpcActionBuilder(std::move(name), std::move(runner)) {
+  AddRequiredParameter({ kMasterAddressesArg, kMasterAddressesArgDesc });
+}
+
+MasterActionBuilder::MasterActionBuilder(std::string name, ActionRunner runner)
+    : RpcActionBuilder(std::move(name), std::move(runner)) {
+  AddRequiredParameter({ kMasterAddressArg, kMasterAddressDesc });
+}
+
+TServerActionBuilder::TServerActionBuilder(std::string name, ActionRunner runner)
+    : RpcActionBuilder(std::move(name), std::move(runner)) {
+  AddRequiredParameter({ kTServerAddressArg, kTServerAddressDesc });
+}
+
+Status BuildMessenger(std::string name, shared_ptr<Messenger>* messenger) {
+  shared_ptr<Messenger> m;
+  MessengerBuilder b(std::move(name));
+  b.set_rpc_negotiation_timeout_ms(FLAGS_negotiation_timeout_ms);
+  auto s = b.Build(&m);
+  if (s.ok()) {
+    *messenger = std::move(m);
+  }
+  return s;
+}
+
 template<class ProxyClass>
 Status BuildProxy(const string& address,
                   uint16_t default_port,
                   unique_ptr<ProxyClass>* proxy) {
   HostPort hp;
   RETURN_NOT_OK(hp.ParseString(address, default_port));
-  shared_ptr<Messenger> messenger;
-  RETURN_NOT_OK(MessengerBuilder("tool").Build(&messenger));
-
   vector<Sockaddr> resolved;
   RETURN_NOT_OK(hp.ResolveAddresses(&resolved));
+  shared_ptr<Messenger> messenger;
+  RETURN_NOT_OK(BuildMessenger("tool", &messenger));
 
   proxy->reset(new ProxyClass(messenger, resolved[0], hp.host()));
   return Status::OK();
@@ -407,35 +512,6 @@ Status PrintSegment(const scoped_refptr<ReadableLogSegment>& segment) {
   return Status::OK();
 }
 
-Status GetServerFlags(const string& address,
-                      uint16_t default_port,
-                      bool all_flags,
-                      const string& flags_to_get,
-                      const string& flag_tags,
-                      vector<server::GetFlagsResponsePB_Flag>* flags) {
-  unique_ptr<GenericServiceProxy> proxy;
-  RETURN_NOT_OK(BuildProxy(address, default_port, &proxy));
-
-  GetFlagsRequestPB req;
-  GetFlagsResponsePB resp;
-  RpcController rpc;
-  rpc.set_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
-
-  req.set_all_flags(all_flags);
-  for (StringPiece tag : strings::Split(flag_tags, ",", strings::SkipEmpty())) {
-    req.add_tags(tag.as_string());
-  }
-  for (StringPiece flag: strings::Split(flags_to_get, ",", strings::SkipEmpty())) {
-    req.add_flags(flag.as_string());
-  }
-
-  RETURN_NOT_OK(proxy->GetFlags(req, &resp, &rpc));
-
-  flags->clear();
-  std::move(resp.flags().begin(), resp.flags().end(), std::back_inserter(*flags));
-  return Status::OK();
-}
-
 Status PrintServerFlags(const string& address, uint16_t default_port) {
   vector<server::GetFlagsResponsePB_Flag> flags;
   RETURN_NOT_OK(GetServerFlags(address, default_port, FLAGS_all_flags,
@@ -496,14 +572,31 @@ bool MatchesAnyPattern(const vector<string>& patterns, const string& str) {
   return false;
 }
 
+Status CreateKuduClient(const vector<string>& master_addresses,
+                        client::sp::shared_ptr<KuduClient>* client,
+                        bool can_see_all_replicas) {
+  auto rpc_timeout = MonoDelta::FromMilliseconds(FLAGS_timeout_ms);
+  auto negotiation_timeout = MonoDelta::FromMilliseconds(
+      FLAGS_negotiation_timeout_ms);
+  KuduClientBuilder b;
+  if (can_see_all_replicas) {
+    ReplicaController::SetVisibility(&b, ReplicaController::Visibility::ALL);
+  }
+  return b
+      .master_server_addrs(master_addresses)
+      .default_rpc_timeout(rpc_timeout)
+      .default_admin_operation_timeout(rpc_timeout)
+      .connection_negotiation_timeout(negotiation_timeout)
+      .Build(client);
+}
+
 Status CreateKuduClient(const RunnerContext& context,
                         const char* master_addresses_arg,
                         client::sp::shared_ptr<KuduClient>* client) {
   vector<string> master_addresses;
-  RETURN_NOT_OK(ParseMasterAddresses(context, master_addresses_arg, &master_addresses));
-  return KuduClientBuilder()
-             .master_server_addrs(master_addresses)
-             .Build(client);
+  RETURN_NOT_OK(ParseMasterAddresses(context, master_addresses_arg,
+                                     &master_addresses));
+  return CreateKuduClient(master_addresses, client);
 }
 
 Status CreateKuduClient(const RunnerContext& context,
@@ -566,6 +659,25 @@ Status ParseMasterAddresses(
   return ParseMasterAddresses(context, kMasterAddressesArg, master_addresses);
 }
 
+Status MasterAddressesToSet(
+    const string& master_addresses_arg,
+    UnorderedHostPortSet* res) {
+  res->clear();
+  vector<HostPort> hp_vector;
+  RETURN_NOT_OK(HostPort::ParseStrings(master_addresses_arg, master::Master::kDefaultPort,
+      &hp_vector));
+  *res = UnorderedHostPortSet(hp_vector.begin(), hp_vector.end());
+
+  // If we deduplicated some masters addresses, log something about it.
+  if (res->size() < hp_vector.size()) {
+    vector<HostPort> addr_list(res->begin(), res->end());
+    LOG(INFO) << "deduplicated master addresses: "
+              << HostPort::ToCommaSeparatedString(addr_list);
+  }
+
+  return Status::OK();
+}
+
 Status PrintServerStatus(const string& address, uint16_t default_port) {
   ServerStatusPB status;
   RETURN_NOT_OK(GetServerStatus(address, default_port, &status));
@@ -600,13 +712,13 @@ Status DumpMemTrackers(const string& address, uint16_t default_port) {
   rpc.set_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
   RETURN_NOT_OK(proxy->DumpMemTrackers(req, &resp, &rpc));
 
-  if (boost::iequals(FLAGS_memtracker_output, "json")) {
+  if (iequals(FLAGS_memtracker_output, "json")) {
     cout << JsonWriter::ToJson(resp.root_tracker(), JsonWriter::Mode::PRETTY)
          << endl;
-  } else if (boost::iequals(FLAGS_memtracker_output, "json_compact")) {
+  } else if (iequals(FLAGS_memtracker_output, "json_compact")) {
     cout << JsonWriter::ToJson(resp.root_tracker(), JsonWriter::Mode::COMPACT)
          << endl;
-  } else if (boost::iequals(FLAGS_memtracker_output, "table")) {
+  } else if (iequals(FLAGS_memtracker_output, "table")) {
     DataTable table({ "id", "parent_id", "limit",
                       "current consumption", "peak_consumption" });
     const auto& root = resp.root_tracker();
@@ -763,15 +875,15 @@ void DataTable::AddColumn(string name, vector<string> column) {
 }
 
 Status DataTable::PrintTo(ostream& out) const {
-  if (boost::iequals(FLAGS_format, "pretty")) {
+  if (iequals(FLAGS_format, "pretty")) {
     PrettyPrintTable(column_names_, columns_, out);
-  } else if (boost::iequals(FLAGS_format, "space")) {
+  } else if (iequals(FLAGS_format, "space")) {
     PrintTable(columns_, " ", out);
-  } else if (boost::iequals(FLAGS_format, "tsv")) {
+  } else if (iequals(FLAGS_format, "tsv")) {
     PrintTable(columns_, "	", out);
-  } else if (boost::iequals(FLAGS_format, "csv")) {
+  } else if (iequals(FLAGS_format, "csv")) {
     PrintTable(columns_, ",", out);
-  } else if (boost::iequals(FLAGS_format, "json")) {
+  } else if (iequals(FLAGS_format, "json")) {
     JsonPrintTable(column_names_, columns_, out);
   } else {
     return Status::InvalidArgument("unknown format (--format)", FLAGS_format);
@@ -779,21 +891,28 @@ Status DataTable::PrintTo(ostream& out) const {
   return Status::OK();
 }
 
-LeaderMasterProxy::LeaderMasterProxy(client::sp::shared_ptr<KuduClient> client) :
-  client_(std::move(client)) {
+LeaderMasterProxy::LeaderMasterProxy(client::sp::shared_ptr<KuduClient> client)
+    : client_(std::move(client)) {
 }
 
-Status LeaderMasterProxy::Init(const vector<string>& master_addrs, const MonoDelta& timeout) {
-  return KuduClientBuilder().master_server_addrs(master_addrs)
-                            .default_rpc_timeout(timeout)
-                            .default_admin_operation_timeout(timeout)
-                            .Build(&client_);
+Status LeaderMasterProxy::Init(const vector<string>& master_addrs,
+                               const MonoDelta& timeout,
+                               const MonoDelta& connection_negotiation_timeout) {
+  return KuduClientBuilder()
+      .master_server_addrs(master_addrs)
+      .default_rpc_timeout(timeout)
+      .default_admin_operation_timeout(timeout)
+      .connection_negotiation_timeout(connection_negotiation_timeout)
+      .Build(&client_);
 }
 
 Status LeaderMasterProxy::Init(const RunnerContext& context) {
   vector<string> master_addresses;
   RETURN_NOT_OK(ParseMasterAddresses(context, &master_addresses));
-  return Init(master_addresses, MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
+  return Init(
+     master_addresses,
+     MonoDelta::FromMilliseconds(FLAGS_timeout_ms),
+     MonoDelta::FromMilliseconds(FLAGS_negotiation_timeout_ms));
 }
 
 template<typename Req, typename Resp>
