@@ -27,6 +27,8 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/rpc/response_callback.h"
+#include "kudu/tools/tool_action.h"
+#include "kudu/util/net/net_util.h"
 #include "kudu/util/status.h"
 
 namespace kudu {
@@ -42,6 +44,7 @@ class MasterServiceProxy;
 } // namespace master
 
 namespace rpc {
+class Messenger;
 class RpcController;
 } // namespace rpc
 
@@ -50,14 +53,11 @@ class ReadableLogSegment;
 } // namespace log
 
 namespace server {
-class GetFlagsResponsePB_Flag;
 class ServerStatusPB;
 } // namespace server
 
 namespace tools {
 
-struct RunnerContext;
-
 // Constants for parameters and descriptions.
 extern const char* const kMasterAddressesArg;
 extern const char* const kMasterAddressesArgDesc;
@@ -69,8 +69,45 @@ extern const char* const kTabletIdArgDesc;
 extern const char* const kTabletIdsCsvArg;
 extern const char* const kTabletIdsCsvArgDesc;
 
+extern const char* const kMasterAddressArg;
+extern const char* const kMasterAddressDesc;
+
+extern const char* const kTServerAddressArg;
+extern const char* const kTServerAddressDesc;
+
+// Builder for actions involving RPC communications, either with a whole Kudu
+// cluster or a particular Kudu RPC server.
+class RpcActionBuilder : public ActionBuilder {
+ public:
+  RpcActionBuilder(std::string name, ActionRunner runner);
+  std::unique_ptr<Action> Build() override;
+};
+
+// Builder for actions involving RPC communications with a Kudu cluster.
+class ClusterActionBuilder : public RpcActionBuilder {
+ public:
+  ClusterActionBuilder(std::string name, ActionRunner runner);
+};
+
+// Builder for actions involving RPC communications with a Kudu master.
+class MasterActionBuilder : public RpcActionBuilder {
+ public:
+  MasterActionBuilder(std::string name, ActionRunner runner);
+};
+
+// Builder for actions involving RPC communications with a Kudu tablet server.
+class TServerActionBuilder : public RpcActionBuilder {
+ public:
+  TServerActionBuilder(std::string name, ActionRunner runner);
+};
+
 // Utility methods used by multiple actions across different modes.
 
+// Build an RPC messenger with the specified name. The messenger has the
+// properties as defined by the command line flags/options.
+Status BuildMessenger(std::string name,
+                      std::shared_ptr<rpc::Messenger>* messenger);
+
 // Builds a proxy to a Kudu server running at 'address', returning it in
 // 'proxy'.
 //
@@ -105,21 +142,6 @@ Status PrintServerStatus(const std::string& address, uint16_t default_port);
 // If 'address' does not contain a port, 'default_port' is used instead.
 Status PrintServerTimestamp(const std::string& address, uint16_t default_port);
 
-// Retrieve flags from a remote server.
-//
-// If 'address' does not contain a port, 'default_port' is used instead.
-//
-// 'all_flags' controls whether all flags are returned, or only flags which are
-// explicitly set.
-//
-// 'flag_tags' is a comma-separated list of tags used to restrict which flags
-// are returned. An empty value matches all tags.
-Status GetServerFlags(const std::string& address,
-                      uint16_t default_port,
-                      bool all_flags,
-                      const std::string& flag_tags,
-                      std::vector<server::GetFlagsResponsePB_Flag>* flags) WARN_UNUSED_RESULT;
-
 // Prints the values of the gflags set for the Kudu server running at 'address'.
 //
 // If 'address' does not contain a port, 'default_port' is used instead.
@@ -141,6 +163,13 @@ Status DumpMemTrackers(const std::string& address, uint16_t default_port);
 // 'patterns' is empty.
 bool MatchesAnyPattern(const std::vector<std::string>& patterns, const std::string& str);
 
+// Create a Kudu client connected to the cluster with the set of the specified
+// master addresses. The 'can_see_all_replicas' controls whether the client sees
+// other than VOTER (e.g., NON_VOTER) tablet replicas.
+Status CreateKuduClient(const std::vector<std::string>& master_addresses,
+                        client::sp::shared_ptr<client::KuduClient>* client,
+                        bool can_see_all_replicas = false);
+
 // Creates a Kudu client connected to the cluster whose master addresses are specified by
 // 'master_addresses_arg'
 Status CreateKuduClient(const RunnerContext& context,
@@ -187,6 +216,13 @@ Status ParseMasterAddresses(
     const RunnerContext& context,
     std::vector<std::string>* master_addresses);
 
+// Parses a comma separated list of "host:port" pairs into an unordered set of
+// HostPort objects. If no port is specified for an entry in the comma separated list,
+// the default master port is used for that entry's pair.
+Status MasterAddressesToSet(
+    const std::string& master_addresses_arg,
+    kudu::UnorderedHostPortSet* res);
+
 // A table of data to present to the user.
 //
 // Supports formatting based on the --format flag.
@@ -230,13 +266,18 @@ class LeaderMasterProxy {
   LeaderMasterProxy() = default;
   explicit LeaderMasterProxy(client::sp::shared_ptr<client::KuduClient> client);
 
-  // Initializes the leader master proxy with the given master addresses and timeout.
-  Status Init(const std::vector<std::string>& master_addrs, const MonoDelta& timeout);
+  // Initializes the leader master proxy with the given master addresses,
+  // RPC timeout, and connection negotiation timeout.
+  Status Init(const std::vector<std::string>& master_addrs,
+              const MonoDelta& timeout,
+              const MonoDelta& connection_negotiation_timeout);
 
   // Initialize the leader master proxy given the provided tool context.
   //
-  // Uses the required 'master_addresses' option for the master addresses, and
-  // the optional 'timeout_ms' flag to control admin and operation timeouts.
+  // Uses the required 'master_addresses' option for the master addresses,
+  // the optional 'timeout_ms' flag to control admin and operation timeouts, and
+  // the optional 'connection_negotiation_timeout_ms' flag to control
+  // the client-side RPC connection negotiation timeout.
   Status Init(const RunnerContext& context);
 
   // Calls a master RPC service method on the current leader master.
