@@ -25,10 +25,10 @@
 #include <string>
 #include <type_traits>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <boost/algorithm/string/predicate.hpp>
 #include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
@@ -46,14 +46,14 @@
 #include "kudu/hms/hms_client.h"
 #include "kudu/tools/tool_action.h"
 #include "kudu/tools/tool_action_common.h"
-#include "kudu/util/monotime.h"
+#include "kudu/util/net/net_util.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
+#include "kudu/util/string_case.h"
 
-DECLARE_string(columns);
 DECLARE_bool(force);
 DECLARE_bool(hive_metastore_sasl_enabled);
-DECLARE_int64(timeout_ms);
+DECLARE_string(columns);
 DECLARE_string(hive_metastore_uris);
 
 DEFINE_bool(dryrun, false,
@@ -73,18 +73,13 @@ DEFINE_bool(ignore_other_clusters, true,
     "Whether to ignore entirely separate Kudu clusters, as indicated by a "
     "different set of master addresses.");
 
-namespace kudu {
-
-namespace tools {
-
-using client::KuduClient;
-using client::KuduClientBuilder;
-using client::KuduSchema;
-using client::KuduTable;
-using client::KuduTableAlterer;
-using client::sp::shared_ptr;
-using hms::HmsCatalog;
-using hms::HmsClient;
+using kudu::client::KuduClient;
+using kudu::client::KuduSchema;
+using kudu::client::KuduTable;
+using kudu::client::KuduTableAlterer;
+using kudu::client::sp::shared_ptr;
+using kudu::hms::HmsCatalog;
+using kudu::hms::HmsClient;
 using std::cout;
 using std::endl;
 using std::make_pair;
@@ -98,6 +93,9 @@ using std::vector;
 using strings::Split;
 using strings::Substitute;
 
+namespace kudu {
+namespace tools {
+
 // Only alter the table in Kudu but not in the Hive Metastore.
 Status RenameTableInKuduCatalog(KuduClient* kudu_client,
                                 const string& name,
@@ -126,10 +124,8 @@ Status Init(const RunnerContext& context,
   RETURN_NOT_OK(ParseMasterAddressesStr(context, &master_addrs_flag));
 
   // Create a Kudu Client.
-  RETURN_NOT_OK(KuduClientBuilder()
-      .default_rpc_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms))
-      .master_server_addrs(Split(master_addrs_flag, ","))
-      .Build(kudu_client));
+  const vector<string> master_addresses(Split(master_addrs_flag, ","));
+  RETURN_NOT_OK(CreateKuduClient(master_addresses, kudu_client));
 
   // Get the configured master addresses from the leader master. It's critical
   // that the check and fix tools use the exact same master address
@@ -186,7 +182,6 @@ bool IsSynced(const set<string>& master_addresses,
   DCHECK(!master_addresses.empty());
   auto schema = KuduSchema::ToSchema(kudu_table.schema());
   hive::Table hms_table_copy(hms_table);
-  // TODO(ghenke): Get the owner from Kudu?
   const string* hms_masters_field = FindOrNull(hms_table.parameters,
                                                HmsClient::kKuduMasterAddrsKey);
   if (!hms_masters_field ||
@@ -194,8 +189,8 @@ bool IsSynced(const set<string>& master_addresses,
     return false;
   }
   Status s = HmsCatalog::PopulateTable(kudu_table.id(), kudu_table.name(), kudu_table.owner(),
-                                       schema, *hms_masters_field, hms_table.tableType,
-                                       &hms_table_copy);
+                                       schema, kudu_table.client()->cluster_id(),
+                                       *hms_masters_field, hms_table.tableType, &hms_table_copy);
   return s.ok() && hms_table_copy == hms_table;
 }
 
@@ -225,6 +220,7 @@ Status PrintTables(const string& master_addrs,
   DataTable table({
       "Kudu table",
       "Kudu table ID",
+      "Kudu cluster ID",
       "Kudu owner",
       "Kudu master addresses",
       "HMS database",
@@ -233,21 +229,25 @@ Status PrintTables(const string& master_addrs,
       "HMS owner",
       Substitute("HMS $0", HmsClient::kKuduTableNameKey),
       Substitute("HMS $0", HmsClient::kKuduTableIdKey),
+      Substitute("HMS $0", HmsClient::kKuduClusterIdKey),
       Substitute("HMS $0", HmsClient::kKuduMasterAddrsKey),
       Substitute("HMS $0", HmsClient::kStorageHandlerKey),
   });
   for (auto& pair : tables) {
     vector<string> row;
     if (pair.first) {
+      // Note: If adding or removing fields, be sure to adjust `row.resize` below.
       const KuduTable& kudu_table = *pair.first;
       row.emplace_back(kudu_table.name());
       row.emplace_back(kudu_table.id());
+      row.emplace_back(kudu_table.client()->cluster_id());
       row.emplace_back(kudu_table.owner());
       row.emplace_back(master_addrs);
     } else {
-      row.resize(4);
+      row.resize(5);
     }
     if (pair.second) {
+      // Note: If adding or removing fields, be sure to adjust `row.resize` below.
       hive::Table& hms_table = *pair.second;
       row.emplace_back(hms_table.dbName);
       row.emplace_back(hms_table.tableName);
@@ -255,10 +255,11 @@ Status PrintTables(const string& master_addrs,
       row.emplace_back(hms_table.owner);
       row.emplace_back(hms_table.parameters[HmsClient::kKuduTableNameKey]);
       row.emplace_back(hms_table.parameters[HmsClient::kKuduTableIdKey]);
+      row.emplace_back(hms_table.parameters[HmsClient::kKuduClusterIdKey]);
       row.emplace_back(hms_table.parameters[HmsClient::kKuduMasterAddrsKey]);
       row.emplace_back(hms_table.parameters[HmsClient::kStorageHandlerKey]);
     } else {
-      row.resize(12);
+      row.resize(14);
     }
     table.AddRow(std::move(row));
   }
@@ -270,35 +271,39 @@ Status PrintHMSTables(vector<hive::Table> tables, ostream& out) {
   DataTable table({});
   for (const auto& column : strings::Split(FLAGS_columns, ",", strings::SkipEmpty())) {
     vector<string> values;
-    if (boost::iequals(column, "database")) {
+    if (iequals(column.ToString(), "database")) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.dbName);
       }
-    } else if (boost::iequals(column, "table")) {
+    } else if (iequals(column.ToString(), "table")) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.tableName);
       }
-    } else if (boost::iequals(column, "type")) {
+    } else if (iequals(column.ToString(), "type")) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.tableType);
       }
-    } else if (boost::iequals(column, "owner")) {
+    } else if (iequals(column.ToString(), "owner")) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.owner);
       }
-    } else if (boost::iequals(column, HmsClient::kKuduTableNameKey)) {
+    } else if (iequals(column.ToString(), HmsClient::kKuduTableNameKey)) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.parameters[HmsClient::kKuduTableNameKey]);
       }
-    } else if (boost::iequals(column, HmsClient::kKuduTableIdKey)) {
+    } else if (iequals(column.ToString(), HmsClient::kKuduTableIdKey)) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.parameters[HmsClient::kKuduTableIdKey]);
       }
-    } else if (boost::iequals(column, HmsClient::kKuduMasterAddrsKey)) {
+    } else if (iequals(column.ToString(), HmsClient::kKuduClusterIdKey)) {
+      for (auto& hms_table : tables) {
+        values.emplace_back(hms_table.parameters[HmsClient::kKuduClusterIdKey]);
+      }
+    } else if (iequals(column.ToString(), HmsClient::kKuduMasterAddrsKey)) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.parameters[HmsClient::kKuduMasterAddrsKey]);
       }
-    } else if (boost::iequals(column, HmsClient::kStorageHandlerKey)) {
+    } else if (iequals(column.ToString(), HmsClient::kStorageHandlerKey)) {
       for (auto& hms_table : tables) {
         values.emplace_back(hms_table.parameters[HmsClient::kStorageHandlerKey]);
       }
@@ -614,7 +619,15 @@ Status FixHmsMetadata(const RunnerContext& context) {
     for (hive::Table& hms_table : report.orphan_hms_tables) {
       string table_name = Substitute("$0.$1", hms_table.dbName, hms_table.tableName);
       const string& master_addrs_param = hms_table.parameters[HmsClient::kKuduMasterAddrsKey];
-      if (master_addrs_param != master_addrs && !FLAGS_force) {
+
+      // Normalize the master addresses to allow for an equality check that ignores
+      // missing default ports, duplicate addresses, and address order.
+      UnorderedHostPortSet param_set;
+      MasterAddressesToSet(master_addrs_param, &param_set);
+      UnorderedHostPortSet cluster_set;
+      MasterAddressesToSet(master_addrs, &cluster_set);
+
+      if (param_set != cluster_set && !FLAGS_force) {
         LOG(INFO) << "Skipping drop of orphan HMS table " << table_name
                   << " with master addresses parameter " << master_addrs_param
                   << " because it does not match the --" << kMasterAddressesArg << " argument"
@@ -642,6 +655,7 @@ Status FixHmsMetadata(const RunnerContext& context) {
   if (FLAGS_create_missing_hms_tables) {
     for (const auto& kudu_table : report.missing_hms_tables) {
       const string& table_id = kudu_table->id();
+      const string& cluster_id = kudu_table->client()->cluster_id();
       const string& table_name = kudu_table->name();
       auto schema = KuduSchema::ToSchema(kudu_table->schema());
       string normalized_table_name(table_name.data(), table_name.size());
@@ -650,7 +664,8 @@ Status FixHmsMetadata(const RunnerContext& context) {
       if (FLAGS_dryrun) {
         LOG(INFO) << "[dryrun] Creating HMS table for Kudu table " << TableIdent(*kudu_table);
       } else {
-        Status s = hms_catalog->CreateTable(table_id, table_name, kudu_table->owner(), schema);
+        Status s = hms_catalog->CreateTable(table_id, table_name, cluster_id,
+            kudu_table->owner(), schema);
         if (s.IsAlreadyPresent()) {
           LOG(ERROR) << "Failed to create HMS table for Kudu table "
                      << TableIdent(*kudu_table)
@@ -704,8 +719,8 @@ Status FixHmsMetadata(const RunnerContext& context) {
                   << hms_table_name;
       } else {
         RETURN_NOT_OK_PREPEND(hms_catalog->UpgradeLegacyImpalaTable(
-                  kudu_table.id(), hms_table.dbName, hms_table.tableName,
-                  KuduSchema::ToSchema(kudu_table.schema())),
+                  kudu_table.id(), kudu_table.client()->cluster_id(), hms_table.dbName,
+                  hms_table.tableName, KuduSchema::ToSchema(kudu_table.schema())),
             Substitute("failed to upgrade legacy Impala HMS metadata for table $0",
               hms_table_name));
       }
@@ -793,7 +808,8 @@ Status FixHmsMetadata(const RunnerContext& context) {
         RETURN_NOT_OK_PREPEND(
             // Disable table ID checking to support fixing tables with unsynchronized IDs.
             hms_catalog->AlterTable(kudu_table.id(), hms_table_name, hms_table_name,
-                owner, schema, /* check_id */ false),
+                                    kudu_table.client()->cluster_id(), owner, schema,
+                                    /* check_id */ false),
             Substitute("failed to refresh HMS table metadata for Kudu table $0",
               TableIdent(kudu_table)));
       }
@@ -822,11 +838,8 @@ Status Precheck(const RunnerContext& context) {
   string master_addrs;
   RETURN_NOT_OK(ParseMasterAddressesStr(context, &master_addrs));
   shared_ptr<KuduClient> client;
-  RETURN_NOT_OK(KuduClientBuilder()
-      .default_rpc_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms))
-      .master_server_addrs(Split(master_addrs, ","))
-      .Build(&client));
-
+  const vector<string> master_addresses(Split(master_addrs, ","));
+  RETURN_NOT_OK(CreateKuduClient(master_addresses, &client));
   vector<string> tables;
   RETURN_NOT_OK(client->ListTables(&tables));
 
@@ -889,60 +902,56 @@ unique_ptr<Mode> BuildHmsMode() {
     "Metastore configuration.";
 
   unique_ptr<Action> hms_check =
-      ActionBuilder("check", &CheckHmsMetadata)
-          .Description("Check metadata consistency between Kudu and the Hive Metastore catalogs")
-          .AddRequiredParameter({ kMasterAddressesArg, kMasterAddressesArgDesc })
-          .AddOptionalParameter("hive_metastore_sasl_enabled", boost::none, kHmsSaslEnabledDesc)
-          .AddOptionalParameter("hive_metastore_uris", boost::none, kHmsUrisDesc)
-          .AddOptionalParameter("ignore_other_clusters")
-          .Build();
+      ClusterActionBuilder("check", &CheckHmsMetadata)
+      .Description("Check metadata consistency between Kudu and the Hive Metastore catalogs")
+      .AddOptionalParameter("hive_metastore_sasl_enabled", boost::none, kHmsSaslEnabledDesc)
+      .AddOptionalParameter("hive_metastore_uris", boost::none, kHmsUrisDesc)
+      .AddOptionalParameter("ignore_other_clusters")
+      .Build();
 
   unique_ptr<Action> hms_downgrade =
-      ActionBuilder("downgrade", &HmsDowngrade)
-          .Description("Downgrade the metadata to legacy format for Kudu and the Hive Metastores")
-          .AddRequiredParameter({ kMasterAddressesArg, kMasterAddressesArgDesc })
-          .AddOptionalParameter("hive_metastore_sasl_enabled", boost::none, kHmsSaslEnabledDesc)
-          .AddOptionalParameter("hive_metastore_uris", boost::none, kHmsUrisDesc)
-          .Build();
+      ClusterActionBuilder("downgrade", &HmsDowngrade)
+      .Description("Downgrade the metadata to legacy format for Kudu and the Hive Metastores")
+      .AddOptionalParameter("hive_metastore_sasl_enabled", boost::none, kHmsSaslEnabledDesc)
+      .AddOptionalParameter("hive_metastore_uris", boost::none, kHmsUrisDesc)
+      .Build();
 
   unique_ptr<Action> hms_fix =
-    ActionBuilder("fix", &FixHmsMetadata)
-        .Description("Fix automatically-repairable metadata inconsistencies in the "
-                     "Kudu and Hive Metastore catalogs")
-        .AddRequiredParameter({ kMasterAddressesArg, kMasterAddressesArgDesc })
-        .AddOptionalParameter("dryrun")
-        .AddOptionalParameter("drop_orphan_hms_tables")
-        .AddOptionalParameter("create_missing_hms_tables")
-        .AddOptionalParameter("fix_inconsistent_tables")
-        .AddOptionalParameter("upgrade_hms_tables")
-        .AddOptionalParameter("hive_metastore_sasl_enabled", boost::none, kHmsSaslEnabledDesc)
-        .AddOptionalParameter("hive_metastore_uris", boost::none, kHmsUrisDesc)
-        .AddOptionalParameter("ignore_other_clusters")
-        .Build();
+      ClusterActionBuilder("fix", &FixHmsMetadata)
+      .Description("Fix automatically-repairable metadata inconsistencies in the "
+                   "Kudu and Hive Metastore catalogs")
+      .AddOptionalParameter("dryrun")
+      .AddOptionalParameter("drop_orphan_hms_tables")
+      .AddOptionalParameter("create_missing_hms_tables")
+      .AddOptionalParameter("fix_inconsistent_tables")
+      .AddOptionalParameter("upgrade_hms_tables")
+      .AddOptionalParameter("hive_metastore_sasl_enabled", boost::none, kHmsSaslEnabledDesc)
+      .AddOptionalParameter("hive_metastore_uris", boost::none, kHmsUrisDesc)
+      .AddOptionalParameter("ignore_other_clusters")
+      .Build();
 
   unique_ptr<Action> hms_list =
-      ActionBuilder("list", &List)
-          .Description("List the Kudu table HMS entries")
-          .AddRequiredParameter({ kMasterAddressesArg, kMasterAddressesArgDesc })
-          .AddOptionalParameter("columns",
-                                Substitute("database,table,type,$0",
-                                                  HmsClient::kKuduTableNameKey),
-                                Substitute("Comma-separated list of HMS entry fields to "
-                                           "include in output.\nPossible values: database, "
-                                           "table, type, owner, $0, $1, $2, $3",
-                                           HmsClient::kKuduTableNameKey,
-                                           HmsClient::kKuduTableIdKey,
-                                           HmsClient::kKuduMasterAddrsKey,
-                                           HmsClient::kStorageHandlerKey))
-          .AddOptionalParameter("format")
-          .Build();
+      ClusterActionBuilder("list", &List)
+      .Description("List the Kudu table HMS entries")
+      .AddOptionalParameter("columns",
+                            Substitute("database,table,type,$0",
+                                              HmsClient::kKuduTableNameKey),
+                            Substitute("Comma-separated list of HMS entry fields to "
+                                       "include in output.\nPossible values: database, "
+                                       "table, type, owner, $0, $1, $2, $3, $4",
+                                       HmsClient::kKuduTableNameKey,
+                                       HmsClient::kKuduTableIdKey,
+                                       HmsClient::kKuduClusterIdKey,
+                                       HmsClient::kKuduMasterAddrsKey,
+                                       HmsClient::kStorageHandlerKey))
+      .AddOptionalParameter("format")
+      .Build();
 
   unique_ptr<Action> hms_precheck =
-    ActionBuilder("precheck", &Precheck)
-        .Description("Check that the Kudu cluster is prepared to enable the Hive "
-                     "Metastore integration")
-        .AddRequiredParameter({ kMasterAddressesArg, kMasterAddressesArgDesc })
-        .Build();
+      ClusterActionBuilder("precheck", &Precheck)
+      .Description("Check that the Kudu cluster is prepared to enable "
+                   "the Hive Metastore integration")
+      .Build();
 
   return ModeBuilder("hms").Description("Operate on remote Hive Metastores")
                            .AddAction(std::move(hms_check))
