@@ -75,6 +75,9 @@ using std::thread;
 using std::vector;
 
 DECLARE_double(env_inject_eio);
+DECLARE_double(tablet_copy_fault_crash_during_download_block);
+DECLARE_double(tablet_copy_fault_crash_during_download_wal);
+DECLARE_int32(tablet_copy_download_threads_nums_per_session);
 DECLARE_string(block_manager);
 DECLARE_string(env_inject_eio_globs);
 
@@ -289,6 +292,32 @@ TEST_F(TabletCopyClientTest, TestDownloadBlock) {
   ASSERT_OK(ReadLocalBlockFile(fs_manager_.get(), new_block_id, &scratch, &slice));
 }
 
+// Test that error status is properly reported if there was a failure in any
+// of multiple threads downloading tablet's data blocks.
+TEST_F(TabletCopyClientTest, TestDownloadBlockMayFail) {
+  FLAGS_tablet_copy_fault_crash_during_download_block = 0.5;
+  FLAGS_tablet_copy_download_threads_nums_per_session = 16;
+
+  ASSERT_OK(ResetTabletCopyClient());
+  ASSERT_OK(StartCopy());
+  Status s = client_->DownloadBlocks();
+  ASSERT_TRUE(s.IsIOError());
+  ASSERT_STR_CONTAINS(s.ToString(), "Injected failure on downloading block");
+}
+
+// Test that error status is properly reported if there was a failure in any
+// of multiple threads downloading tablet's wal segments.
+TEST_F(TabletCopyClientTest, TestDownloadWalMayFail) {
+  FLAGS_tablet_copy_fault_crash_during_download_wal = 1;
+  FLAGS_tablet_copy_download_threads_nums_per_session = 4;
+
+  ASSERT_OK(ResetTabletCopyClient());
+  ASSERT_OK(StartCopy());
+  Status s = client_->DownloadWALs();
+  ASSERT_TRUE(s.IsIOError());
+  ASSERT_STR_CONTAINS(s.ToString(), "Injected failure on downloading wal");
+}
+
 // Basic WAL segment download unit test.
 TEST_F(TabletCopyClientTest, TestDownloadWalSegment) {
   ASSERT_OK(StartCopy());
@@ -360,7 +389,7 @@ TEST_F(TabletCopyClientTest, TestDownloadAllBlocks) {
   // test is to exemplify the difference in syncs between the log and file
   // block managers, but it would be nice to formulate a bound here.
   if (FLAGS_block_manager == "log") {
-    ASSERT_GE(9, down_cast<Counter*>(
+    ASSERT_GE(15, down_cast<Counter*>(
         metric_entity_->FindOrNull(METRIC_block_manager_total_disk_sync).get())->value());
   } else {
     ASSERT_GE(22, down_cast<Counter*>(
