@@ -17,10 +17,12 @@
 
 #include "kudu/util/flags.h"
 
+#include <strings.h>
 #include <sys/stat.h>
 #include <unistd.h> // IWYU pragma: keep
 
 #include <cstdlib>
+#include <cstring>
 #include <functional>
 #include <iostream>
 #include <map>
@@ -29,7 +31,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/algorithm/string/predicate.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #ifdef TCMALLOC_ENABLED
@@ -72,6 +73,11 @@ DEFINE_bool(dump_metrics_json, false,
             "by this binary.");
 TAG_FLAG(dump_metrics_json, hidden);
 
+DEFINE_bool(dump_metrics_xml, false,
+            "Dump an XML document describing all of the metrics which may be emitted "
+            "by this binary.");
+TAG_FLAG(dump_metrics_xml, hidden);
+
 #ifdef TCMALLOC_ENABLED
 DEFINE_bool(enable_process_lifetime_heap_profiling, false, "Enables heap "
     "profiling for the lifetime of the process. Profile output will be stored in the "
@@ -493,6 +499,9 @@ void HandleCommonFlags() {
   } else if (FLAGS_dump_metrics_json) {
     MetricPrototypeRegistry::get()->WriteAsJson();
     exit(0);
+  } else if (FLAGS_dump_metrics_xml) {
+    MetricPrototypeRegistry::get()->WriteAsXML();
+    exit(0);
   } else if (FLAGS_version) {
     cout << VersionInfo::GetAllVersionInfo() << endl;
     exit(0);
@@ -585,11 +594,11 @@ GFlagsMap GetFlagsMap() {
 
 Status ParseTriState(const char* flag_name, const std::string& flag_value,
     TriStateFlag* tri_state) {
-  if (boost::iequals(flag_value, "required")) {
+  if (iequals(flag_value, "required")) {
     *tri_state = TriStateFlag::REQUIRED;
-  } else if (boost::iequals(flag_value, "optional")) {
+  } else if (iequals(flag_value, "optional")) {
     *tri_state = TriStateFlag::OPTIONAL;
-  } else if (boost::iequals(flag_value, "disabled")) {
+  } else if (iequals(flag_value, "disabled")) {
     *tri_state = TriStateFlag::DISABLED;
   } else {
     return Status::InvalidArgument(strings::Substitute(
@@ -599,4 +608,24 @@ Status ParseTriState(const char* flag_name, const std::string& flag_value,
   return Status::OK();
 }
 
+// Get the value of an environment variable that has boolean semantics.
+bool GetBooleanEnvironmentVariable(const char* env_var_name) {
+  const char* const e = getenv(env_var_name);
+  if ((e == nullptr) ||
+      (strlen(e) == 0) ||
+      (strcasecmp(e, "false") == 0) ||
+      (strcasecmp(e, "0") == 0) ||
+      (strcasecmp(e, "no") == 0)) {
+    return false;
+  }
+  if ((strcasecmp(e, "true") == 0) ||
+      (strcasecmp(e, "1") == 0) ||
+      (strcasecmp(e, "yes") == 0)) {
+    return true;
+  }
+  LOG(FATAL) << Substitute("$0: invalid value for environment variable $0",
+                           e, env_var_name);
+  return false;  // unreachable
+}
+
 } // namespace kudu
