@@ -66,6 +66,9 @@ void CheckStandardFds() {
 
 Status CheckCPUFlags() {
   base::CPU cpu;
+  if (!cpu.has_broken_neon() && cpu.cpu_brand()=="ARM64") {
+    return Status::OK();
+  }
   if (!cpu.has_sse42()) {
     return BadCPUStatus(cpu, "SSE4.2");
   }
