@@ -17,6 +17,8 @@
 
 #pragma once
 
+#include <atomic>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <map>
@@ -32,22 +34,23 @@
 
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/util/atomic.h"
 #include "kudu/util/condition_variable.h"
+#include "kudu/util/locks.h"
+#include "kudu/util/maintenance_manager_metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/mutex.h"
 #include "kudu/util/random.h"
 #include "kudu/util/status.h"
 
 namespace kudu {
-template<class T>
-class AtomicGauge;
 class Histogram;
 class MaintenanceManager;
 class MaintenanceManagerStatusPB;
 class MaintenanceManagerStatusPB_OpInstancePB;
+class MetricEntity;
 class Thread;
 class ThreadPool;
+template <typename T> class AtomicGauge;
 
 class MaintenanceOpStats {
  public:
@@ -228,7 +231,7 @@ class MaintenanceOp {
   // Returns the gauge for this op that tracks when this op is running. Cannot be NULL.
   virtual scoped_refptr<AtomicGauge<uint32_t>> RunningGauge() const = 0;
 
-  uint32_t running() { return running_; }
+  uint32_t running() const { return running_; }
 
   const std::string& name() const { return name_; }
 
@@ -236,7 +239,7 @@ class MaintenanceOp {
 
   // Return true if the operation has been cancelled due to a pending Unregister().
   bool cancelled() const {
-    return cancel_.Load();
+    return cancel_;
   }
 
   // Cancel this operation, which prevents new instances of it from being scheduled
@@ -244,7 +247,7 @@ class MaintenanceOp {
   // optionally poll 'cancelled()' on a periodic basis to know if they should abort a
   // lengthy operation in the middle of Perform().
   void CancelAndDisable() {
-    cancel_.Store(true);
+    cancel_ = true;
   }
 
  protected:
@@ -258,25 +261,28 @@ class MaintenanceOp {
   // The name of the operation.  Op names must be unique.
   const std::string name_;
 
-  // The number of instances of this op that are currently running.
-  uint32_t running_;
+  IOUsage io_usage_;
+
+  // The number of instances of this op that are currently running. The field
+  // is updated by MaintenanceManager which guards the access as needed. If the
+  // access isn't guarded by the 'MaintenanceManager::running_instances_lock_',
+  // use the 'running()' accessor to read the value lock-free.
+  std::atomic<uint32_t> running_;
 
   // Set when we are trying to unregister the maintenance operation.
   // Ongoing operations could read this boolean and cancel themselves.
   // New operations will not be scheduled when this boolean is set.
-  AtomicBool cancel_;
+  std::atomic<bool> cancel_;
 
   // Condition variable which the UnregisterOp function can wait on.
   //
-  // Note: 'cond_' is used with the MaintenanceManager's mutex. As such,
-  // it only exists when the op is registered.
+  // Note: 'cond_' is used with the MaintenanceManager::running_instances_lock_
+  // mutex. As such, it only exists when the op is registered.
   std::unique_ptr<ConditionVariable> cond_;
 
   // The MaintenanceManager with which this op is registered, or null
   // if it is not registered.
   std::shared_ptr<MaintenanceManager> manager_;
-
-  IOUsage io_usage_;
 };
 
 struct MaintenanceOpComparator {
@@ -298,7 +304,9 @@ class MaintenanceManager : public std::enable_shared_from_this<MaintenanceManage
     uint32_t history_size;
   };
 
-  MaintenanceManager(const Options& options, std::string server_uuid);
+  MaintenanceManager(const Options& options,
+                     std::string server_uuid,
+                     const scoped_refptr<MetricEntity>& metric_entity);
   ~MaintenanceManager();
 
   // Start running the maintenance manager.
@@ -306,14 +314,19 @@ class MaintenanceManager : public std::enable_shared_from_this<MaintenanceManage
   Status Start();
   void Shutdown();
 
-  // Register an op with the manager.
+  // Register an op with the manager. If the op cannot be put into 'ops_'
+  // immediately because 'lock_' is currently held, buffers the op into
+  // 'ops_pending_registration_', and MergePendingOpRegistrationsUnlocked() can
+  // be used to coalesce them the next time 'lock_' is taken.
   void RegisterOp(MaintenanceOp* op);
 
   // Unregister an op with the manager.
-  // If the Op is currently running, it will not be interrupted.  However, this
+  // If the Op is currently running, it will not be interrupted. However, this
   // function will block until the Op is finished.
   void UnregisterOp(MaintenanceOp* op);
 
+  // Fill out 'out_pb' based on the registered ops, merging in any ops pending
+  // registration in case any exist.
   void GetMaintenanceManagerStatusDump(MaintenanceManagerStatusPB* out_pb);
 
   void set_memory_pressure_func_for_tests(std::function<bool(double*)> f) {
@@ -329,7 +342,7 @@ class MaintenanceManager : public std::enable_shared_from_this<MaintenanceManage
   FRIEND_TEST(MaintenanceManagerTest, TestOpFactors);
 
   typedef std::map<MaintenanceOp*, MaintenanceOpStats,
-          MaintenanceOpComparator> OpMapTy;
+          MaintenanceOpComparator> OpMapType;
 
   // Return true if tests have currently disabled the maintenance
   // manager by way of changing the gflags at runtime.
@@ -337,8 +350,6 @@ class MaintenanceManager : public std::enable_shared_from_this<MaintenanceManage
 
   void RunSchedulerThread();
 
-  bool FindAndLaunchOp(std::unique_lock<Mutex>* guard);
-
   // Find the best op, or null if there is nothing we want to run.
   //
   // Returns the op, as well as a string explanation of why that op was chosen,
@@ -358,22 +369,52 @@ class MaintenanceManager : public std::enable_shared_from_this<MaintenanceManage
   bool CouldNotLaunchNewOp(bool prev_iter_found_no_work);
 
   void IncreaseOpCount(MaintenanceOp *op);
-  void DecreaseOpCount(MaintenanceOp *op);
+  void DecreaseOpCountAndNotifyWaiters(MaintenanceOp *op);
+
+  // Adds ops in 'ops_pending_registration_' to 'ops_'. Must be called while
+  // 'lock_' is held.
+  void MergePendingOpRegistrationsUnlocked();
 
   const std::string server_uuid_;
   const int32_t num_threads_;
-  OpMapTy ops_; // Registered operations.
+  const MonoDelta polling_interval_;
+
+  // Ops for which RegisterOp() has been called, but that have not yet been
+  // added to 'ops_'. Since adding to 'ops_' requires taking 'lock_', rather
+  // than blocking registration (and bootstrapping), this serves as a buffer to
+  // be added to 'ops_' once the lock is available. If this lock is taken
+  // concurrently with 'lock_', 'lock_' must be taken first.
+  simple_spinlock registration_lock_;
+  OpMapType ops_pending_registration_;
+
+  // Registered operations: the access guarded by the 'lock_' field.
+  OpMapType ops_;
+
+  // Scheduler's lock used to guard the access to the 'ops_' container and
+  // notify the scheduler via the 'cond_' condition variable.
+  // This lock should be used very sparingly, since it's taken when finding
+  // the best operation to schedule, and the latter might be time consuming
+  // especially if there are many maintenance operation candidates.
   Mutex lock_;
+  // Condition variable used to wake up/notify the scheduler right after
+  // performing scheduled maintenance operation (based on 'lock_').
+  ConditionVariable cond_;
+
   scoped_refptr<kudu::Thread> monitor_thread_;
   std::unique_ptr<ThreadPool> thread_pool_;
-  ConditionVariable cond_;
   bool shutdown_;
-  int32_t polling_interval_ms_;
-  int32_t running_ops_;
+
+  // This field is atomic because it's written under 'running_instances_lock_'
+  // and read when the latter lock isn't held.
+  std::atomic<int32_t> running_ops_;
+
+  // Lock to guard access to 'completed_ops_' and 'completed_ops_count_'.
+  simple_spinlock completed_ops_lock_;
   // Vector used as a circular buffer for recently completed ops. Elements need to be added at
   // the completed_ops_count_ % the vector's size and then the count needs to be incremented.
   std::vector<OpInstance> completed_ops_;
-  int64_t completed_ops_count_;
+  size_t completed_ops_count_;
+
   Random rand_;
 
   // Function which should return true if the server is under global memory pressure.
@@ -395,6 +436,9 @@ class MaintenanceManager : public std::enable_shared_from_this<MaintenanceManage
   // Protected by running_instances_lock_;
   std::unordered_map<int64_t, OpInstance*> running_instances_;
 
+  // MM-specific metrics.
+  MaintenanceManagerMetrics metrics_;
+
   DISALLOW_COPY_AND_ASSIGN(MaintenanceManager);
 };
 
