@@ -199,14 +199,21 @@ fetch_and_patch \
  $GPERFTOOLS_SOURCE \
  $GPERFTOOLS_PATCHLEVEL \
  "patch -p1 < $TP_DIR/patches/gperftools-Replace-namespace-base-with-namespace-tcmalloc.patch" \
- "patch -p1 < $TP_DIR/patches/gperftools-unbreak-memz.patch" \
  "autoreconf -fvi"
 
+# NOTE: creating an empty 'third_party/googletest/m4' subdir is a recipe from
+# the $PROTOBUF_SOURCE/autogen.sh file:
+#
+#   The absence of a m4 directory in googletest causes autoreconf to fail when
+#   building under the CentOS docker image. It's a warning in regular build on
+#   Ubuntu/gLinux as well.
+#
 PROTOBUF_PATCHLEVEL=0
 fetch_and_patch \
- protobuf-${PROTOBUF_VERSION}.tar.gz \
+ protobuf-cpp-${PROTOBUF_VERSION}.tar.gz \
  $PROTOBUF_SOURCE \
  $PROTOBUF_PATCHLEVEL \
+ "mkdir -p third_party/googletest/m4" \
  "autoreconf -fvi"
 
 # Returns 0 if cmake should be patched to work around this bug [1].
@@ -255,11 +262,12 @@ fetch_and_patch \
  $ZLIB_SOURCE \
  $ZLIB_PATCHLEVEL
 
-LIBEV_PATCHLEVEL=0
+LIBEV_PATCHLEVEL=1
 fetch_and_patch \
  libev-${LIBEV_VERSION}.tar.gz \
  $LIBEV_SOURCE \
- $LIBEV_PATCHLEVEL
+ $LIBEV_PATCHLEVEL \
+ "patch -p1 < $TP_DIR/patches/libev-c17.patch"
 
 RAPIDJSON_PATCHLEVEL=1
 fetch_and_patch \
@@ -319,14 +327,15 @@ fetch_and_patch \
  $PYTHON_SOURCE \
  $PYTHON_PATCHLEVEL
 
-LLVM_PATCHLEVEL=5
+LLVM_PATCHLEVEL=4
 fetch_and_patch \
  llvm-${LLVM_VERSION}-iwyu-${IWYU_VERSION}.src.tar.gz \
  $LLVM_SOURCE \
  $LLVM_PATCHLEVEL \
   "patch -p1 < $TP_DIR/patches/llvm-add-iwyu.patch" \
   "patch -p1 < $TP_DIR/patches/llvm-iwyu-include-picker.patch" \
-  "patch -p0 < $TP_DIR/patches/llvm-iwyu-sized-deallocation.patch"
+  "patch -p0 < $TP_DIR/patches/llvm-iwyu-sized-deallocation.patch" \
+  "patch -d projects -p1 < $TP_DIR/patches/llvm-947f9692440836dcb8d88b74b69dd379d85974ce.patch"
 
 LZ4_PATCHLEVEL=0
 fetch_and_patch \
@@ -346,13 +355,11 @@ fetch_and_patch \
  $TRACE_VIEWER_SOURCE \
  $TRACE_VIEWER_PATCHLEVEL
 
-BOOST_PATCHLEVEL=2
+BOOST_PATCHLEVEL=0
 fetch_and_patch \
  boost_${BOOST_VERSION}.tar.gz \
  $BOOST_SOURCE \
- $BOOST_PATCHLEVEL \
- "patch -p0 < $TP_DIR/patches/boost-issue-12179-fix-compilation-errors.patch" \
- "patch -p0 < $TP_DIR/patches/boost-issue-440-darwin-version.patch"
+ $BOOST_PATCHLEVEL
 
 # Return 0 if the current system appears to be el6 (either CentOS or proper RHEL)
 needs_openssl_workaround() {
@@ -463,7 +470,6 @@ fetch_and_patch \
  $RANGER_NAME.tar.gz \
  $RANGER_SOURCE \
  $RANGER_PATCHLEVEL \
- "patch -p1 < $TP_DIR/patches/ranger-python3.patch" \
  "patch -p0 < $TP_DIR/patches/ranger-fixscripts.patch"
 
 echo "---------------"
